#include "rmf_funcs.i"
#include "curreg.i"
#include "ds9reg.i"
#include "island.i"

// A struct for the command with parameters

struct s_Command {
   string command;
   pointer rparms;
   pointer iparms;
   pointer sparms;
}

// A struct for the command key words

struct s_Keyword {
   string keyword;
   string keyvalue;
}

// A struct for the regions

struct s_Reg {
   string type;
   string shape;
   double xcen;
   double ycen;
   long area;
   long ncts;
   pointer params;
   pointer imsegment;
   pointer n_outline;
   pointer p_outline;
   pointer x_outline;
   pointer y_outline;
}

Version = "2.6"; // 2013-05-14

void_value = [];

write,"";
write,"         --------------------------------------------------------";
write,"         |   Welcome to the spectral analysis tool: aspec-"+Version+"   |";
write,"         --------------------------------------------------------";
write,"";
write,"aspec is invoked as:  > aspec, event_file_name, ..";
write,"";
write,"   Accepts several event files that will be concatenated.";
write,"   'aspec' is command driven.";
write,"   Entering a '?' will show the command options. After exit 'aspec'";
write,"   can be continued (in the same Yorick session) by simply entering:";
write,"  > aspec";
write,"";
write,"But first choose the telescope:";
write,"      'n' for NuSTAR";
write,"      's' for SXT";
write,"      'o' for other (starts dialogue) :";
write,"";
answer = rdline(prompt=" ... ");
chat = 1;
if( answer == "n" ) {
   write,"";
   write,"*** This is the NuSTAR verson ***";
   write,"";
   telescop = "NuSTAR";
   parfile = pfiles_path("aspec_nustar.par");
   if( is_void(parfile) ) error,"No aspec_nustar.par found";
   pixel_size = get_par(parfile,"pixel_size"); // mm
   n_det_pixels = get_par(parfile,"n_det_pixels");
   xdetcen = get_par(parfile,"xdetcen"); // pixels
   ydetcen = get_par(parfile,"ydetcen"); // pixels
   Rmf_file = get_par(parfile,"rmf_file");
   Arf_file = get_par(parfile,"arf_file");
   hdr = headfits(Arf_file+"+1");
   Scal_a = fxpar(hdr,"SCAL_A");
   if( is_void(Scal_a) ) error,"Missing SCAL_A keyword in "+Arf_file;
   Scal_b = fxpar(hdr,"SCAL_B");
   if( is_void(Scal_b) ) error,"Missing SCAL_B keyword in "+Arf_file;
   Ebegin = emin_default = get_par(parfile,"emin_default"); // keV
   Eend =   emax_default = get_par(parfile,"emax_default"); // keV
   dead_pixel_map_dol = get_par(parfile,"dead_pixel_map");
   if( is_void(dead_pixel_map_dol) || dead_pixel_map_dol == "none" ) {
      Dead_pixel_map = array(short,n_det_pixels,n_det_pixels);
   } else {
      Dead_pixel_map = readfits(dead_pixel_map_dol);
      dmsdpm = dimsof(Dead_pixel_map);
      if( dmsdpm(2) != n_det_pixels || dmsdpm(3) != n_det_pixels ) \
         error,"##63## Inconsistency in detector pixels and dpm";
   }
   dead_idx = where(Dead_pixel_map);
   pixel_area_cm2 = 0.01 * pixel_size^2;
   region_type = "simple"; // other option is 'detailed'
} else if( answer == "s" ) {
   write,"";
   write,"*** This is the SXT verson ***";
   write,"";
   telescop = "SXT";
   if( !file_test("aspec_sxt.par") ) error,"No aspec_sxt.par found in current directory";
   pixel_size = get_par("aspec_sxt.par","pixel_size"); // mm
   n_det_pixels = get_par("aspec_sxt.par","n_det_pixels");
   xdetcen = get_par("aspec_sxt.par","xdetcen"); // pixels
   ydetcen = get_par("aspec_sxt.par","ydetcen"); // pixels
   Rmf_file = get_par("aspec_sxt.par","rmf_file");
   Arf_file = get_par("aspec_sxt.par","arf_file");
   Ebegin = emin_default = get_par("aspec_sxt.par","emin_default"); // keV
   Eend =   emax_default = get_par("aspec_sxt.par","emax_default"); // keV
   pixel_area_cm2 = 0.01 * pixel_size^2;
   region_type = "simple"; // other option is 'detailed'
} else {
   write,"Not implemented yet";
   chat = 0;
}
instrume = "MT_RAYOR-4.4";
if( chat ) {
   write,format=" pixel_size = %.2f mm\n", pixel_size;
   write,format=" n_det_pixels = %i\n", n_det_pixels;
   write,format=" xdetcen    = %.2f pixels\n", xdetcen;
   write,format=" ydetcen    = %.2f pixels\n", ydetcen;
   write,format=" RMF: %s\n", Rmf_file;
   write,format=" ARF: %s\n", Arf_file;
   write,"";
}


func aspec( event_file, .., keep_reg=, addexpo=, batch=, fbatch=, expr= )
/* DOCUMENT aspec[,event_file][,event_file2, ..][, keep_reg=][,addexpo=] \
               [,batch=][,fbatch=][,expr=]

    A package for spectral analysis of X-ray mission event data.
    Enter '?' for a list of commands and '? <command>' for some
    more explanation.

    Giving an event_file will reset and read the event data.

    Keywords
     keep_reg   : will stop resetting the defined regions
     addexpo    : The exposure times for each event file will be added
                  otherwise the same exposure time is expected in the
                  event files.
     batch      : A string array with commands to be executed.
     fbatch     : A text file with commands to be executed.
     expr       : An expression for a selection on numerical values in
                  the event file columns (see 'fselect' yorick version).

    Calling without an event_file will make it resume operations on
    existing data (if the Yorick session has not been interrupted).

    2010-10-20/NJW
    2010-10-28/NJW updated with normalization of spectra
*/
{

   extern Event_files, Rawx, Rawy, Rawxy, Energy, Nevents;
   extern SRegions, BRegions, Exposure, E_min, E_max, Arfs, Energ_lo, Energ_hi;
   extern Arf_file, Rmf_file, SOceans, BOceans, Scal_a, Scal_b;
   extern Rate_src, Rate_err_src, Rate_bkg, Rate_err_bkg, Spec_norm_factor;
   extern Rate_net, Rate_err_net, Spec_norm_rule, Spec_norm_text, Spec_ser_str;
   extern Ebegin, Eend, Esel; // Defines current energy interval for image display
                        // and regions

   // coordinate system from first event file
   extern Coords;

  /*   Spec_norm_rule == 1  :  no renormalization i.e. spectrum in source area
   *                     2  :  spectrum per pixel
   *                     3  :  spectrum per cm2
   */
   Spec_norm_rule = 1; // default starting value
   Spec_norm_factor = 1.; // default starting value
   Spec_norm_text = "Normalized to source region area";

   // Check for batch run
   if( !is_void(fbatch) ) batch = rdfile(fbatch);
   ibatch = 1; // first command
   nbatch = 0; // invoke batch commands as long as ibatch <= nbatch
   if( !is_void(batch) ) {
      nbatch = numberof(batch); // number of batch commands
   }

   if( !is_void(event_file) ) {
      Event_files = event_file;
      logfilename = get_next_filename("aspec_????.txt");
      lg = gen_log_init(logfilename,logflag=5);
      while( more_args() ) grow, Event_files, next_arg();
      n_event_files = numberof(Event_files);
      gen_log_entry,1,"Event files:",Event_files,id=lg;
   } else { // continue on current events
      if( is_void(Event_files) ) {
         write,"No session to continue ...";
         write,"Syntax:  aspec, eventfile[, eventfile2[, eventfile3, ..]][,keep_reg=][,..]";
         return;
      }
      logfilename = get_next_filename("aspec_????.txt",latest=1);
      // continue logging in previous log file
      lg = gen_log_init(logfilename,logflag=5,cont=1);
   }
   write,"Logging file: "+logfilename;
   gen_log_entry,1,"","Log of 'aspec' on "+ndate(3),"",id=lg;

   // initiate error tracking

   erlog = gen_log_init("aspec.log",logflag=5,erase=1);

   if( !is_void(event_file) ) {
      for( i = 1; i <= n_event_files; i++ ) {
         if( !file_test(Event_files(i)) ) error,"File "+Event_files(i)+" is not found!";
      }
      gen_log_entry,1,"##1## resetting",id=erlog;
      if( !keep_reg ) SRegions = BRegions = [];

      write,"Reading "+Event_files(1)+" ...";
      dol = Event_files(1)+"+1";
      local hdr, nrowstot;
      ptr = rdfitsbin( dol, hdr, nrowstot,silent=1);
      Rawx = *ptr(fits_colnum(hdr,"rawx"));
      Rawy = *ptr(fits_colnum(hdr,"rawy"));
      Energy = *ptr(fits_colnum(hdr,"energy"));
      Exposure = fxpar(hdr,"exposure");
      n_events = fxpar(hdr,"naxis2");
      sss = swrite(format="Event_file : %s,  %i events, expo = %.1f s", Event_files(1), n_events, Exposure);
      gen_log_entry,1,sss,id=lg;
      write,format="Event_file : %s,  %i events, expo = %.1f s\n", Event_files(1), n_events, Exposure;
      if( structof(expr) == string ) {
         wexpr = fselect( dol, "-", expr );
         n_events = numberof(wexpr);
         if( n_events ) {
            Rawx = Rawx(wexpr);
            Rawy = Rawy(wexpr);
            Energy = Energy(wexpr);
         } else {
            Rawx = Rawy = Energy = [];
         }
         sss = swrite(format=" Selection with expression '%s' gave %i events", expr, n_events);
         gen_log_entry,1,sss,id=lg;
         write,format=" Selection with expression '%s' gave %i events\n", expr, n_events;
      }
      n_events = 0;
      if( !is_void(Energy) ) {
         // Initial energy selection between default boundaries
         w = where( Energy > emin_default & Energy < emax_default );
         n_events = numberof(w);
         if( n_events ) {
            Rawx = Rawx(w); 
            Rawy = Rawy(w); 
            Energy = Energy(w); 
         } else {
            Rawx = Rawy = Energy = [];
         }
      }
      sss = swrite(format=" Initial energy selection gave %i events", n_events);
      gen_log_entry,1,sss,id=lg;
      write,format=" Initial energy selection gave %i events\n", n_events;

      // get the WCS coordinate system
      Coords = get_wcs( hdr ); // Using recent (120817) function in mfits.i

      if( n_event_files > 1 ) {
         for( i = 2; i <= n_event_files; i++ ) {
            write,"Reading "+Event_files(i)+" ...";
            dol = Event_files(i)+"+1";
            ptr = rdfitsbin( dol, hdr, nrowstot,silent=1);
            tmprawx = *ptr(fits_colnum(hdr,"rawx"));
            tmprawy = *ptr(fits_colnum(hdr,"rawy"));
            tmpenergy = *ptr(fits_colnum(hdr,"energy"));
            exposure = fxpar(hdr,"exposure");
            n_events = fxpar(hdr,"naxis2");
            if( addexpo ) { // add the exposure times
               Exposure += exposure;
            } else { // make sure that all have the same exposure time
               if( !near( Exposure, exposure, 1. ) ) error,"Wrong exposure in file #"+itoa(i);
            }
            sss = swrite(format="Event_file : %s,  %i events, expo = %.1f s", Event_files(i), n_events, exposure);
            gen_log_entry,1,sss,id=lg;
            write,format="Event_file : %s,  %i events, expo = %.1f s\n", Event_files(i), n_events, exposure;
            if( structof(expr) == string ) {
               wexpr = fselect( dol, "-", expr );
               n_events = numberof(wexpr);
               if( n_events ) {
                  tmprawx = tmprawx(wexpr);
                  tmprawy = tmprawy(wexpr);
                  tmpenergy = tmpenergy(wexpr);
               } else {
                  tmprawx = tmprawy = tmpenergy = [];
               }
               sss = swrite(format=" Selection with expression '%s' gave %i events", expr, n_events);
               gen_log_entry,1,sss,id=lg;
               write,format=" Selection with expression '%s' gave %i events\n", expr, n_events;
            }
            n_events = 0;
            if( !is_void(tmpenergy) ) {
               // Initial energy selection between default boundaries
               w = where( tmpenergy > emin_default & tmpenergy < emax_default );
               n_events = numberof(w);
               if( n_events ) {
                  tmprawx = tmprawx(w); 
                  tmprawy = tmprawy(w); 
                  tmpenergy = tmpenergy(w); 
               } else {
                  tmprawx = tmprawy = tmpenergy = [];
               }
            }
            sss = swrite(format=" Initial energy selection gave %i events", n_events);
            gen_log_entry,1,sss,id=lg;
            write,format=" Initial energy selection gave %i events\n", n_events;
            grow, Rawx, tmprawx;
            grow, Rawy, tmprawy;
            grow, Energy, tmpenergy;
         }
      }

      // Filter out the dead detector areas

      if( !is_void(dead_idx) ) {
         ev_idx = (Rawy - 1) * n_det_pixels + Rawx;
         idx = whereany( ev_idx, dead_idx );
         if( numberof(idx) ) {
            //+ ev_idxn = rem_elem( ev_idx, idx );
            ev_idx_yes = rem_elem( indgen(numberof(Rawx)), idx );
            Rawx = Rawx(ev_idx_yes);
            Rawy = Rawy(ev_idx_yes);
            Energy = Energy(ev_idx_yes);
         }
      }

      Nevents = numberof(Rawx);
      sss = swrite(format=" Active detector pixel selection gave %i events", Nevents);
      gen_log_entry,1,sss,id=lg;
      write,format=" Active detector pixel selection gave %i events\n", Nevents;

      Esel = indgen(Nevents);
      // Sanity check
      if( Nevents != numberof(Rawy) || Nevents != numberof(Energy) ) error,"##123## Sanity check";
      Rawxy = (Rawy - 1)*n_det_pixels + Rawx;
      write,"Using RMF in: "+Rmf_file;
      E_min = rdfitscol(Rmf_file+"[EBOUNDS]","e_min");
      E_max = rdfitscol(Rmf_file+"[EBOUNDS]","e_max");
      write,"Loading ARFs from: "+Arf_file;
      Arfs = rdfitscol(Arf_file+"[SPECRESP]","SPECRESP");
      Energ_lo = rdfitscol(Arf_file+"[SPECRESP]","ENERG_LO")(,1);
      Energ_hi = rdfitscol(Arf_file+"[SPECRESP]","ENERG_HI")(,1);
   }

   im = array(int,n_det_pixels,n_det_pixels);
   for(i=1;i<=Nevents;i++) im(Rawx(i),Rawy(i)) += 1;
   mlogxy,0,0;
   disp,im,title="All events";

   command_string = "";
   while( 1 ) {
      if( ibatch <= nbatch ) {
         command_string = batch(ibatch);
         ibatch++;
      } else {
         read,format="%[^\n]",prompt="What now? ... ", command_string;
      }
      command_string = strtrim(command_string);
      gen_log_entry,1,"##4## command given: "+command_string,id=erlog;
      gen_log_entry,1,"Command given: "+command_string,id=lg;
      cans = comparse(command_string);
      cmd = cans.command;

      if( cmd == "x" ) {
         return;
      }

//
// ************************************** info *********************************************************
//
      if( cmd == "info" ) { // cominfo   -  give information on current settings
         cans = comparse(command_string, "i"); // information level
         niparms = numberof(*cans.iparms);

         write,format="Energy range: %.3f - %.3f keV\n", Ebegin, Eend;
         write,format="Exposure time: %.1f s\n", Exposure;
         write," Event file(s):";
         prstrarr, Event_files;
         write,format="Total number of events: %i\n", Nevents;
         write,format="Number of energy selected events: %i\n", numberof(Esel);

//
// ************************************** batch *********************************************************
//
      } else if( cmd == "batch" ) { // combatch   -  initiate a series of batch commands
         cans = comparse(command_string, "s"); // batch file name
         nsparms = numberof(*cans.sparms);
         if( nsparms == 1 ) {
            if( file_test((*cans.sparms)(1)) ) {
               batch = rdfile((*cans.sparms)(1));
               nbatch = numberof(batch);
               ibatch = 1;
            } else {
               write,"Could not find: "+(*cans.sparms)(1);
            }
         } else {
            write,"Bad number of parameters";
         }
//
// ************************************** pal *********************************************************
//
      } else if( cmd == "pal" ) { // compal   -  set the name of the palette
         cans = comparse(command_string, "s"); // name of palette
         nsparms = numberof(*cans.sparms);
         if( nsparms == 1 ) {
            pal = (*cans.sparms)(1);
            if( strpart( pal, -2:0 ) != ".gp" ) pal += ".gp";
            palette, pal;
         }
//
// ************************************** ebds *********************************************************
//
      } else if( cmd == "ebds" ) { // comebds  -  set energy boundaries
         cans = comparse(command_string, "rr"); // Ebegin and Eend
         nrparms = numberof(*cans.rparms);
         // Define the new energy boundaries Ebegin and Eend:
         if( nrparms == 0 ) {
            Ebegin = emin_default;
            Eend   = emax_default;
         } else if( nrparms == 1 ) {
            Ebegin = (*cans.rparms)(1);
            Eend   = emax_default;
         } else {
            Ebegin = (*cans.rparms)(1);
            Eend   = (*cans.rparms)(2);
         }
         // Find the event indices that match the new boundaries:
         Esel = where( Energy > Ebegin & Energy < Eend );
         nsel = numberof(Esel);
         write,format="Energy range: %.3f - %.3f keV\n", Ebegin, Eend;
         write,format="       containing %i events\n", nsel;
         dsel = where( E_max > Ebegin & E_min < Eend );
         write,format="       and %i detector energy bins\n", numberof(dsel);

      //   Update image and regions wrt. number of counts

         // Start a new image:
         im = array(int,n_det_pixels,n_det_pixels);
         // And populate with events:
         for(i=1;i<=nsel;i++) im(Rawx(Esel(i)),Rawy(Esel(i))) += 1;
         sss = swrite(format="##5## emin emax nsel : %.2f %.2f %i", Ebegin, Eend, nsel);
         gen_log_entry,1,sss,id=erlog;

         // update the regions
         nreg = numberof(SRegions);
         if( nreg ) {
            for( i = 1; i <= nreg; i++ ) {
               if( region_type == "simple" ) {
                  ssel = selreg( im, SRegions(i).shape, *SRegions(i).params );
                  bsel = selreg( im, BRegions(i).shape, *BRegions(i).params );
               } else { // Namely when region_type is "detailed"
                  ssel = *SRegions(i).imsegment; // area selection
                  bsel = *BRegions(i).imsegment; // area selection
               }
               SRegions(i).ncts = numberof(whereany(Rawxy(Esel), ssel));
               BRegions(i).ncts = is_void(bsel) ? 0 : numberof(whereany(Rawxy(Esel), bsel));
            }
         }

//
// ************************************** imdisp & lmdisp *********************************************************
//
      } else if( cmd == "imdisp" || cmd == "lmdisp" ) { // comimdisp, comlmdisp   -  display detector image
         cans = comparse(command_string, "r"); // image folding sigma

         rim = double(im);
         if( !is_void(*cans.rparms) ) rim = gfconvol(rim,(*cans.rparms)(1));
         mlogxy,0,0;
         if( cmd == "lmdisp" ) disp,log(1.0+rim);
         else disp,rim;

         // add the regions
         nreg = numberof(SRegions);
         for( i = 1; i <= nreg; i++ ) {
            if( region_type == "detailed" ) {
               nout = *SRegions(i).n_outline;
               pout = *SRegions(i).p_outline;
               xout = *SRegions(i).x_outline;
               yout = *SRegions(i).y_outline;
               for( j = 1; j <= numberof(nout); j++ ) {
                  oplot,xout(pout(j):pout(j)-1+nout(j)),yout(pout(j):pout(j)-1+nout(j)),color="green";
               }
            } else {
               reg_params = *SRegions(i).params;
               shape      = SRegions(i).shape;
               if( shape == "c" ) {
                  angles = span(0,2*pi,100);
                  oplot,reg_params(1) + reg_params(3)*cos(angles), \
                        reg_params(2) + reg_params(3)*sin(angles), color="green";
               }
               if( shape == "a" ) {
                  angles = span(0,2*pi,100);
                  oplot,reg_params(1) + reg_params(3)*cos(angles), \
                        reg_params(2) + reg_params(3)*sin(angles), color="green";
                  oplot,reg_params(1) + reg_params(4)*cos(angles), \
                        reg_params(2) + reg_params(4)*sin(angles), color="green";
               }
               if( shape == "b" ) {
                  oplot,[reg_params(1),reg_params(3),reg_params(3),reg_params(1), reg_params(1)], \
                        [reg_params(2),reg_params(2),reg_params(4),reg_params(4), reg_params(2)], \
                        color="green";
               }
            }
            if( BRegions(i).shape != "v" ) {
               if( region_type == "detailed" ) {
                  nout = *BRegions(i).n_outline;
                  pout = *BRegions(i).p_outline;
                  xout = *BRegions(i).x_outline;
                  yout = *BRegions(i).y_outline;
                  for( j = 1; j <= numberof(nout); j++ ) {
                     oplot,xout(pout(j):pout(j)-1+nout(j)),yout(pout(j):pout(j)-1+nout(j)),color="red";
                  }
               } else {
                  reg_params = *BRegions(i).params;
                  shape      = BRegions(i).shape;
                  if( shape == "c" ) {
                     angles = span(0,2*pi,100);
                     oplot,reg_params(1) + reg_params(3)*cos(angles), \
                           reg_params(2) + reg_params(3)*sin(angles), color="red";
                  }
                  if( shape == "a" || shape == "d" ) {
                     angles = span(0,2*pi,100);
                     oplot,reg_params(1) + reg_params(3)*cos(angles), \
                           reg_params(2) + reg_params(3)*sin(angles), color="red";
                     oplot,reg_params(1) + reg_params(4)*cos(angles), \
                           reg_params(2) + reg_params(4)*sin(angles), color="red";
                  }
                  if( shape == "b" ) {
                     oplot,[reg_params(1),reg_params(3),reg_params(3),reg_params(1), reg_params(1)], \
                           [reg_params(2),reg_params(2),reg_params(4),reg_params(4), reg_params(2)], \
                           color="red";
                  }
               }
            }
         }
//
// ************************************** imsave *********************************************************
//
      } else if( cmd == "imsave"  ) { // comimsave  -  save detector image to FITS
         cans = comparse(command_string, "s"); // given filename

         if( is_void(*cans.sparms) ) {
            fits_name = get_next_filename("aspec_image_????.fits");
         } else {
            fits_name = (*cans.sparms)(1);
         }
         gen_log_entry,1,"Writing to: "+fits_name,id=lg;
         write,"Writing to: "+fits_name;
         kwds_init;
         kwds_set,"DATE",ndate(3),"Date/time of file creation";
         kwds_set,"ORIGIN","aspec-"+Version,"Software that produced it";
         kwds_set,"E_MIN", Ebegin, "[keV] Lower energy limit";
         kwds_set,"E_MAX", Eend, "[keV] Upper energy limit";
         for( i = 1; i <= numberof(Event_files); i++ ) {
            kwds_set,"EVFILE"+itoa(i),Event_files(i);
         }
         if( Coords.flag) {
            if( strlen(Coords.ctype1) ) {
               kwds_set,"CTYPE1", Coords.ctype1, "Type of coord.system";
            } else kwds_set,"CTYPE1", "RA---TAN", "Type of coord.system";
            if( strlen(Coords.ctype2) ) {
               kwds_set,"CTYPE2", Coords.ctype2, "Type of coord.system";
            } else kwds_set,"CTYPE2", "DEC--TAN", "Type of coord.system";
            kwds_set,"CUNIT1", "deg", "Unit for CTYPE1";
            kwds_set,"CUNIT2", "deg", "Unit for CTYPE2";
            kwds_set,"CRPIX1", Coords.crpix1, "Reference pixel position";
            kwds_set,"CRPIX2", Coords.crpix2, "Reference pixel position";
            kwds_set,"CRVAL1", Coords.crval1, "Reference coord value";
            kwds_set,"CRVAL2", Coords.crval2, "Reference coord value";
            kwds_set,"CD1_1", Coords.cd1_1, "Part of transf. matrix";
            kwds_set,"CD1_2", Coords.cd1_2, "Part of transf. matrix";
            kwds_set,"CD2_1", Coords.cd2_1, "Part of transf. matrix";
            kwds_set,"CD2_2", Coords.cd2_2, "Part of transf. matrix";
         }
         kwds_set,"EXTNAME","ASPEC_IMAGE","Name of this extension";
         writefits, fits_name, im;
//
// ************************************** rpos *********************************************************
//
      } else if( cmd == "rpos" ) { // comrpos  -  read position in image
         pos = curmark1( style=0,prompt="Mark the position ... " );
         write,format="Pixel position: %7.3f %7.3f\n", pos(1), pos(2);
         local coord1, coord2;
         skypos_fits, Coords, pos(1), pos(2), coord1, coord2, to_sky=1,silent=1;
         write,format="Coord position: %7.3f %7.3f\n", coord1, coord2;
         write,format="Pixel value : %i counts\n", im(int(pos(1)+0.5),int(pos(2)+0.5));
//
// ************************************** gpix *********************************************************
//
      } else if( cmd == "gpix" ) { // comgpix  -  get pixel numbers from WCS
         cans = comparse(command_string, "rr"); // RA and dec
         nrparms = numberof(*cans.rparms);
         if( nrparms != 2 ) {
            write,"Exactly two values must be entered ...";
         } else {
            local pix1, pix2;
            skypos_fits, Coords, pix1, pix2, (*cans.rparms)(1), (*cans.rparms)(2), to_pix=1;
            write,format="Coord position: %7.3 %7.3\n", (*cans.rparms)(1), (*cans.rparms)(2);
            write,format="Pixel position: %7.3f %7.3f\n", pix1, pix2;
            write,format="Pixel value : %i counts\n", im(int(pix1+0.5),int(pix2+0.5));
         }
//
// ************************************** mpos *********************************************************
//
      } else if( cmd == "mpos" ) { // commpos  -  mark WCS position in the image
         cans = comparse(command_string, "rr"); // RA and dec
         nrparms = numberof(*cans.rparms);
         if( nrparms != 2 ) {
            write,"Exactly two values must be entered ...";
         } else {
            local pix1, pix2;
            skypos_fits, Coords, pix1, pix2, (*cans.rparms)(1), (*cans.rparms)(2), to_pix=1;
            oplot,[pix1],[pix2],ps=16,thick=2,color="white",symsize=1.;
         }
//
// ************************************** mreg *********************************************************
//
      } else if( cmd == "mreg" ) { // commreg  -  make region pair
         cans = comparse(command_string, "srrrrrrrr");
         // For RADecRad selection
         rpars = *cans.rparms;
         nrpars = numberof(rpars);

         if( is_void(*cans.sparms) ) {
            s_shape = "c";  // circular region with annulus is the default
            b_shape = "d";  // Annulus with predefined center
         } else {
            shape = (*cans.sparms)(1);
            if( strlen(shape) != 2 ) {
               write,"Parameter must be two-letter string";
               continue;
            }
            s_shape = strpart(shape,1:1);
            b_shape = strpart(shape,2:2);
            if( s_shape != "a" && s_shape != "b" && s_shape != "c" ) {
               write,"Parameter must have a, b, or c as first letter";
               continue;
            }
            if( b_shape != "a" && b_shape != "b" && b_shape != "c" && b_shape != "v" ) {
               write,"Parameter must have a, b, c, or v as second letter";
               continue;
            }
         }
         // ******  Prepare the regions arrays
         local xcen, ycen, sreg_params, breg_params;
         if( is_void(SRegions) ) {
            //+ 101119 SRegions = s_Reg();
            //+ 101119 BRegions = s_Reg();
            SRegions = array(s_Reg,1);
            BRegions = array(s_Reg,1);
         } else {
            grow, SRegions, s_Reg();
            grow, BRegions, s_Reg();
         }
         nreg = numberof(SRegions);
         // ******  Define the source region
         write,"Define the source region:";
         if( nrpars ) {
            did_ok = argreg( s_shape, rpars, xcen, ycen, sreg_params );
            sel = selreg( im, s_shape, sreg_params );
         } else {
            sel = int(curreg(im,s_shape, xcen, ycen, sreg_params, color="green",thick=2));
         }
         if( region_type != "simple" ) { // i.e. region_type == "detailed"
            if( numberof(sel) > 2000 ) {
               write,"Warning! *** number of source region pixels exceeds 2000."
            }
            ocean = array(short,n_det_pixels,n_det_pixels);
            ocean(sel) = 1;
            isle = island(ocean);
            c_outline, isle, ocean, x_outline, y_outline, n_outline, p_outline;
            n_outline = int(n_outline);
            p_outline = int(p_outline);
            SRegions(nreg).imsegment = &sel;
            SRegions(nreg).n_outline = &n_outline;
            SRegions(nreg).p_outline = &p_outline;
            SRegions(nreg).x_outline = &x_outline;
            SRegions(nreg).y_outline = &y_outline;
         }
         SRegions(nreg).type = region_type;
         SRegions(nreg).params = &sreg_params;
         SRegions(nreg).shape = s_shape;
         SRegions(nreg).xcen = xcen;
         SRegions(nreg).ycen = ycen;
         SRegions(nreg).area = numberof(sel);
         SRegions(nreg).ncts = numberof(whereany(Rawxy(Esel), sel));
         // ******  Define the background region
         if( b_shape == "v" ) { // void i.e. no background but dummy to be defined
            BRegions(nreg).shape = b_shape;
            BRegions(nreg).xcen = xcen;
            BRegions(nreg).ycen = ycen;
            BRegions(nreg).area = 1; // to avoid division by zero
            //+ BRegions(nreg).imsegment = &sel;
            BRegions(nreg).imsegment = &void_value;
            BRegions(nreg).ncts = 0;
            BRegions(nreg).type = region_type;
         } else { // 'a', 'd', or 'b' (annulus or box)
            write,"Define the background region:";
            if( nrpars ) {
               offset = s_shape == "c" ? 4 : 5;
               did_ok = argreg( b_shape, rpars(offset:0), xcen, ycen, breg_params );
               sel = selreg( im, b_shape, breg_params );
            } else {
               sel = int(curreg(im, b_shape, xcen, ycen, breg_params, color="red",thick=2));
            }
            if( region_type != "simple" ) { // i.e. region_type == "detailed"
               if( numberof(sel) > 2000 ) {
                  write,"Warning! *** number of source region pixels exceeds 2000."
               }
               ocean = array(short,n_det_pixels,n_det_pixels);
               ocean(sel) = 1;
               isle = island(ocean);
               c_outline, isle, ocean, x_outline, y_outline, n_outline, p_outline;
               n_outline = int(n_outline);
               p_outline = int(p_outline);
               BRegions(nreg).imsegment = &sel;
               BRegions(nreg).n_outline = &n_outline;
               BRegions(nreg).p_outline = &p_outline;
               BRegions(nreg).x_outline = &x_outline;
               BRegions(nreg).y_outline = &y_outline;
            }
            BRegions(nreg).type = region_type;
            BRegions(nreg).shape = b_shape;
            BRegions(nreg).params = &breg_params;
            BRegions(nreg).xcen = xcen;
            BRegions(nreg).ycen = ycen;
            BRegions(nreg).area = numberof(sel);
            BRegions(nreg).ncts = numberof(whereany(Rawxy(Esel), sel));
         }

//
// ************************************** dreg *********************************************************
//
      } else if( cmd == "dreg" ) { // comdreg  -  delete region(s)
         cans = comparse(command_string, "i");
         nreg = numberof(SRegions);
         if( nreg == 0 ) {
            write,"No regions have been defined ...";
         } else {
            if( !is_void(*cans.iparms) ) {
               // delete a single region
               i = (*cans.iparms)(1);
               if( i < 1 || i > nreg ) {
                  write,"Requested region is not found ...";
                  continue;
               }
               if( nreg == 1 ) {
                  SRegions = BRegions = [];
               } else {
                  SRegions = rem_elem( SRegions, i );
                  BRegions = rem_elem( BRegions, i );
               }
            } else {
               SRegions = BRegions = [];
            }
         }
//
// ************************************** ireg *********************************************************
//
      } else if( cmd == "ireg" ) { // comireg  -  show region information
         cans = comparse(command_string, "i");
         nreg = numberof(SRegions);
         if( nreg == 0 ) {
            write,"No regions have been defined ...";
         } else {
            i1 = 1; i2 = nreg; // default, show all regions
            if( !is_void(*cans.iparms) ) {
               i1 = i2 = (*cans.iparms)(1);
               if( i1 < 1 || i1 > nreg ) {
                  write,"Requested region is not found ...";
                  i1 = 1; i2 = nreg;
               }
            }
            write,"      #    xcen   ycen     area  shape      ncts";
            for( i = i1; i <= i2; i++ ) {
               write,format="Src: %3i  %6.2f %6.2f %8i    %s %11i\n", \
                  i, SRegions(i).xcen, SRegions(i).ycen, SRegions(i).area, \
                         SRegions(i).shape, SRegions(i).ncts;
               write,format="Bkg: %3i  %6.2f %6.2f %8i    %s %11i\n", \
                  i, BRegions(i).xcen, BRegions(i).ycen, BRegions(i).area, \
                         BRegions(i).shape, BRegions(i).ncts;
               n = SRegions(i).ncts;
               k = BRegions(i).area > 0 ? double(SRegions(i).area)/BRegions(i).area : 1.0;
               b = BRegions(i).ncts;
               dn = sqrt(n + b*(k^2));
               write,format=" Net counts: %.2f +- %.2f\n", n - b*k, dn;
            }
         }
//
// ************************************** sreg *********************************************************
//
      } else if( cmd == "sreg" ) { // comsreg  -  save region information
         nreg = numberof(SRegions);
         cans = comparse(command_string, "i"); // If number > 0 then save that one
                                               // If number == 0 then save all
                                               // If number is not given then save most recent
                                               // In any case a new file will be created
         if( is_void(*cans.iparms) ) {
            n = nreg;
         } else {
            n = (*cans.iparms)(1);
         }
    /*
     *   A region is described by a single row in a FITS files
     */
         local serstr;
         sregfile = get_next_filename("s_????.freg", serstr);
         bregfile = "b_"+serstr+".freg";
         write,"      saving into "+sregfile+" and "+bregfile;
         gen_log_entry,1,"      saving into "+sregfile+" and "+bregfile,id=lg;
         if( n == 0 ) { // save all
           // --     source regions
            kwds_init;
            kwds_set,"EXTNAME","ASPEC_SREGFILE","Name of this extension";
            kwds_set,"DATE",ndate(3),"Date/time of creation";
            kwds_set,"ORIGIN","aspec-"+Version,"Software that produced it";
            kwds_set,"EMIN", Ebegin,"[keV] Lower energy limit";
            kwds_set,"EMAX", Eend,"[keV] Upper energy limit";
            kwds_set,"COMMENT","This is a number of source regions";
            kwds_set,"COMMENT","from a running of 'aspec' version "+Version;
            wrmfitscols,sregfile,"TYPE",SRegions.type, \
                                "SHAPE",SRegions.shape, \
                                "XCEN",SRegions.xcen, \
                                "YCEN",SRegions.ycen, \
                                "AREA",SRegions.area, \
                                "NCTS",SRegions.ncts, \
                                "IMSEGMENT",int(0), \
                                "N_OUTLINE",int(0), \
                                "P_OUTLINE",int(0), \
                                "X_OUTLINE",double(0), \
                                "Y_OUTLINE",double(0), \
                                "PARAMS",double(0), \
                                var=[7,8,9,10,11,12];
            for( i = 1; i <= nreg; i++ ) {
               //+ if( !is_void(*SRegions(i).imsegment) ) {
                  if( region_type == "detailed" ) {
                     fits_bintable_poke,sregfile+"+1",i,"IMSEGMENT",*SRegions(i).imsegment;
                     fits_bintable_poke,sregfile+"+1",i,"N_OUTLINE",*SRegions(i).n_outline;
                     fits_bintable_poke,sregfile+"+1",i,"P_OUTLINE",*SRegions(i).p_outline;
                     fits_bintable_poke,sregfile+"+1",i,"X_OUTLINE",*SRegions(i).x_outline;
                     fits_bintable_poke,sregfile+"+1",i,"Y_OUTLINE",*SRegions(i).y_outline;
                  }
                  if( !is_void( *SRegions(i).params ) ) {
                     write,"Updating region file with source.params ...";
                     fits_bintable_poke,sregfile+"+1",i,"PARAMS",*SRegions(i).params;
                  }
               //+ }
            }
           // --     background regions
            kwds_init;
            kwds_set,"EXTNAME","ASPEC_BREGFILE","Name of this extension";
            kwds_set,"DATE",ndate(3),"Date/time of creation";
            kwds_set,"ORIGIN","aspec-"+Version,"Software that produced it";
            kwds_set,"EMIN", Ebegin,"[keV] Lower energy limit";
            kwds_set,"EMAX", Eend,"[keV] Upper energy limit";
            kwds_set,"COMMENT","This is a number of background regions";
            kwds_set,"COMMENT","from a running of 'aspec' version "+Version;
            wrmfitscols,bregfile,"TYPE",BRegions.type, \
                                "SHAPE",BRegions.shape, \
                                "XCEN",BRegions.xcen, \
                                "YCEN",BRegions.ycen, \
                                "AREA",BRegions.area, \
                                "NCTS",BRegions.ncts, \
                                "IMSEGMENT",int(0), \
                                "N_OUTLINE",int(0), \
                                "P_OUTLINE",int(0), \
                                "X_OUTLINE",double(0), \
                                "Y_OUTLINE",double(0), \
                                "PARAMS",double(0), \
                                var=[7,8,9,10,11,12];
            for( i = 1; i <= nreg; i++ ) {
               if( !is_void(*BRegions(i).imsegment) ) {
                  fits_bintable_poke,bregfile+"+1",i,"IMSEGMENT",*BRegions(i).imsegment;
               }
               if( region_type == "detailed" ) {
                  fits_bintable_poke,bregfile+"+1",i,"N_OUTLINE",*BRegions(i).n_outline;
                  fits_bintable_poke,bregfile+"+1",i,"P_OUTLINE",*BRegions(i).p_outline;
                  fits_bintable_poke,bregfile+"+1",i,"X_OUTLINE",*BRegions(i).x_outline;
                  fits_bintable_poke,bregfile+"+1",i,"Y_OUTLINE",*BRegions(i).y_outline;
               }
               if( !is_void( *BRegions(i).params ) ) {
                  write,"Updating region file with bkg.params ...";
                  fits_bintable_poke,sregfile+"+1",i,"PARAMS",*BRegions(i).params;
               }
            }
         } else { // save the specified one
           //  --  the source region
            kwds_init;
            kwds_set,"EXTNAME","ASPEC_SREGFILE","Name of this extension";
            kwds_set,"DATE",ndate(3),"Date/time of creation";
            kwds_set,"ORIGIN","aspec-"+Version,"Software that produced it";
            kwds_set,"EMIN", Ebegin,"[keV] Lower energy limit";
            kwds_set,"EMAX", Eend,"[keV] Upper energy limit";
            kwds_set,"COMMENT","This is a singls source region";
            kwds_set,"COMMENT","from a running of 'aspec' version "+Version;
            wrmfitscols,sregfile,"TYPE",[SRegions(n).type], \
                                "SHAPE",[SRegions(n).shape], \
                                "XCEN",[SRegions(n).xcen], \
                                "YCEN",[SRegions(n).ycen], \
                                "AREA",[SRegions(n).area], \
                                "NCTS",[SRegions(n).ncts], \
                                "IMSEGMENT",int(0), \
                                "N_OUTLINE",int(0), \
                                "P_OUTLINE",int(0), \
                                "X_OUTLINE",double(0), \
                                "Y_OUTLINE",double(0), \
                                "PARAMS",double(0), \
                                var=[7,8,9,10,11,12];
            //+ if( !is_void(*SRegions(n).imsegment) ) {
               if( region_type == "detailed" ) {
                  fits_bintable_poke,sregfile+"+1",1,"IMSEGMENT",*SRegions(n).imsegment;
                  fits_bintable_poke,sregfile+"+1",1,"N_OUTLINE",*SRegions(n).n_outline;
                  fits_bintable_poke,sregfile+"+1",1,"P_OUTLINE",*SRegions(n).p_outline;
                  fits_bintable_poke,sregfile+"+1",1,"X_OUTLINE",*SRegions(n).x_outline;
                  fits_bintable_poke,sregfile+"+1",1,"Y_OUTLINE",*SRegions(n).y_outline;
               }
               if( !is_void(*SRegions(n).params) ) {
                  write,"Updating region file with source.params ...";
                  fits_bintable_poke,sregfile+"+1",n,"PARAMS",*SRegions(n).params;
               }
            //+ }
           // --  the background region
            kwds_init;
            kwds_set,"EXTNAME","ASPEC_BREGFILE","Name of this extension";
            kwds_set,"DATE",ndate(3),"Date/time of creation";
            kwds_set,"ORIGIN","aspec-"+Version,"Software that produced it";
            kwds_set,"EMIN", Ebegin,"[keV] Lower energy limit";
            kwds_set,"EMAX", Eend,"[keV] Upper energy limit";
            kwds_set,"COMMENT","This is a number of source regions";
            kwds_set,"COMMENT","from a running of 'aspec' version "+Version;
            wrmfitscols,bregfile,"TYPE",[BRegions(n).type], \
                                "SHAPE",[BRegions(n).shape], \
                                "XCEN",[BRegions(n).xcen], \
                                "YCEN",[BRegions(n).ycen], \
                                "AREA",[BRegions(n).area], \
                                "NCTS",[BRegions(n).ncts], \
                                "IMSEGMENT",int(0), \
                                "N_OUTLINE",int(0), \
                                "P_OUTLINE",int(0), \
                                "X_OUTLINE",double(0), \
                                "Y_OUTLINE",double(0), \
                                "PARAMS",double(0), \
                                var=[7,8,9,10,11,12];
            //+ if( !is_void(*BRegions(n).imsegment) ) {
               if( region_type == "detailed" ) {
                  fits_bintable_poke,bregfile+"+1",1,"IMSEGMENT",*BRegions(n).imsegment;
                  fits_bintable_poke,bregfile+"+1",1,"N_OUTLINE",*BRegions(n).n_outline;
                  fits_bintable_poke,bregfile+"+1",1,"P_OUTLINE",*BRegions(n).p_outline;
                  fits_bintable_poke,bregfile+"+1",1,"X_OUTLINE",*BRegions(n).x_outline;
                  fits_bintable_poke,bregfile+"+1",1,"Y_OUTLINE",*BRegions(n).y_outline;
               }
               if( !is_void(*BRegions(n).params) ) {
                  write,"Updating region file with bkg.params ...";
                  fits_bintable_poke,bregfile+"+1",n,"PARAMS",*BRegions(n).params;
               }
            //+ }
         }
//
// ************************************** lreg *********************************************************
//
      } else if( cmd == "lreg" ) { // comlreg  -  load region information
         nreg = numberof(SRegions);
         //   Command:   lreg NNNN [num]
         //              lreg file=regionfilename

         cans = comparse(command_string, "si"); // If number > 0 is given then load that one
                                                // If number == 0 then load all in file
                                                // If number is not given then load first
                                                // region in the file
        /*
         *   Check the command and the existence of region files
         */

         if( is_void(*cans.sparms) ) {
            error,"Illegal command for lreg";
         } else {
            // see if a keyword has been given:
            kc = keyparse( (*cans.sparms)(1) );
            if( is_void( kc ) ) {
               from_ds9reg = 0;
               serstr = (*cans.sparms)(1);
               lserstr = strlen(serstr);
               if( lserstr != 4 ) error,"Illegal length of str for lreg";
               sregfile = "s_"+serstr+".freg";
               if( !file_test(sregfile) ) error,sregfile+" is not found";
               bregfile = "b_"+serstr+".freg";
               if( !file_test(bregfile) ) error,bregfile+" is not found";
            } else {
               if( kc.keyword != "file" ) error,"Wrong keyword";
               if( !file_test(kc.keyvalue) ) error,"Did not find: "+kc.keyvalue;
               from_ds9reg = 1;
            }
         }

         if( from_ds9reg ) {

            sel = ds9reg(kc.keyvalue,im,Coords,s_shape, xcen, ycen, sreg_params, color="green",thick=2);
            n_sel = numberof(sel);

            while( n_sel > 1 ) {

               grow, SRegions, array(s_Reg,1);
               grow, BRegions, array(s_Reg,1);
               nreg = numberof(SRegions);

               // ******  Define the source region
               if( region_type != "simple" ) {
                  ocean = array(short,n_det_pixels,n_det_pixels);
                  ocean(sel) = 1;
                  isle = island(ocean);
                  c_outline, isle, ocean, x_outline, y_outline, n_outline, p_outline;
                  n_outline = int(n_outline);
                  p_outline = int(p_outline);
                  SRegions(nreg).imsegment = &sel;
                  SRegions(nreg).n_outline = &n_outline;
                  SRegions(nreg).p_outline = &p_outline;
                  SRegions(nreg).x_outline = &x_outline;
                  SRegions(nreg).y_outline = &y_outline;
               }
               SRegions(nreg).type = region_type;
               SRegions(nreg).params = &sreg_params;
               SRegions(nreg).shape = s_shape;
               SRegions(nreg).xcen = xcen;
               SRegions(nreg).ycen = ycen;
               SRegions(nreg).area = numberof(sel);
               SRegions(nreg).ncts = numberof(whereany(Rawxy(Esel), sel));
               // ******  Define a dummy background region
               BRegions(nreg).shape = "v";
               BRegions(nreg).xcen = xcen;
               BRegions(nreg).ycen = ycen;
               BRegions(nreg).area = 1; // to avoid division by zero
               //+ BRegions(nreg).imsegment = &sel;
               BRegions(nreg).imsegment = &void_value;
               BRegions(nreg).ncts = 0;
               BRegions(nreg).type = region_type;

               sel = ds9reg( ,im,Coords,s_shape, xcen, ycen, sreg_params, color="green",thick=2);
               n_sel = numberof(sel);

            }

         } else {

            if( is_void(*cans.iparms) ) {
               n = 1; // 'n' is the row number
            } else {
               n = (*cans.iparms)(1);
            }
   
            shdr = headfits(sregfile+"+1");
            bhdr = headfits(bregfile+"+1");
            nrows = fxpar( shdr, "naxis2" );
            if( nrows != fxpar( bhdr, "naxis2" ) ) error,"Different number of rows";
   
            if( n > nrows ) error,"Region number exceeds number of rows";
   
            // make room for the regions to be loaded
   
            if( n ) {
               grow, SRegions, array(s_Reg,1);
               grow, BRegions, array(s_Reg,1);
            } else {
               grow, SRegions, array(s_Reg,nrows);
               grow, BRegions, array(s_Reg,nrows);
            }
   
       /*
        *   A region is described by a single row in a FITS files
        */
            if( n == 0 ) { // load all
              // --     source regions
               type  = rdfitscol(sregfile+"+1","TYPE");
               shape = rdfitscol(sregfile+"+1","SHAPE");
               xcen = rdfitscol(sregfile+"+1","XCEN");
               ycen = rdfitscol(sregfile+"+1","YCEN");
               area = rdfitscol(sregfile+"+1","AREA");
               ncts = rdfitscol(sregfile+"+1","NCTS");
               SRegions(nreg+1:nreg+nrows).type  = type;
               SRegions(nreg+1:nreg+nrows).shape = shape;
               SRegions(nreg+1:nreg+nrows).xcen = xcen;
               SRegions(nreg+1:nreg+nrows).ycen = ycen;
               SRegions(nreg+1:nreg+nrows).area = area;
               SRegions(nreg+1:nreg+nrows).ncts = ncts;
               for( i = 1; i <= nrows; i++ ) {
                  if( region_type != "simple" ) {
                     d1 = fits_bintable_peek(sregfile+"+1",i,"IMSEGMENT");
                     SRegions(nreg+i).imsegment = &d1;
                     // update number of counts
                     if( !is_void(d1) ) SRegions(nreg+i).ncts = numberof(whereany(Rawxy, d1));
                     d1 = fits_bintable_peek(sregfile+"+1",i,"N_OUTLINE");
                     SRegions(nreg+i).n_outline = &d1;
                     d1 = fits_bintable_peek(sregfile+"+1",i,"P_OUTLINE");
                     SRegions(nreg+i).p_outline = &d1;
                     d1 = fits_bintable_peek(sregfile+"+1",i,"X_OUTLINE");
                     SRegions(nreg+i).x_outline = &d1;
                     d1 = fits_bintable_peek(sregfile+"+1",i,"Y_OUTLINE");
                     SRegions(nreg+i).y_outline = &d1;
                  }
                  d1 = fits_bintable_peek(sregfile+"+1",i,"PARAMS");
                  SRegions(nreg+i).params = &d1;
                  write,"Loading SRegion "+itoa(nreg+i);
                  write,d1;
                  if( region_type == "simple" ) {
                     d1 = selreg( im, SRegions(nreg+i).shape, *SRegions(nreg+i).params );
                     SRegions(nreg+i).ncts = numberof(whereany(Rawxy, d1));
                  }
               }
              // --     background regions
               type  = rdfitscol(bregfile+"+1","TYPE");
               shape = rdfitscol(bregfile+"+1","SHAPE");
               xcen = rdfitscol(bregfile+"+1","XCEN");
               ycen = rdfitscol(bregfile+"+1","YCEN");
               area = rdfitscol(bregfile+"+1","AREA");
               ncts = rdfitscol(bregfile+"+1","NCTS");
               BRegions(nreg+1:nreg+nrows).type = type;
               BRegions(nreg+1:nreg+nrows).shape = shape;
               BRegions(nreg+1:nreg+nrows).xcen = xcen;
               BRegions(nreg+1:nreg+nrows).ycen = ycen;
               BRegions(nreg+1:nreg+nrows).area = area;
               BRegions(nreg+1:nreg+nrows).ncts = ncts;
               for( i = 1; i <= nrows; i++ ) {
                  if( region_type != "simple" ) {
                     d1 = fits_bintable_peek(bregfile+"+1",i,"IMSEGMENT");
                     BRegions(nreg+i).imsegment = &d1;
                     // update number of counts
                     if( !is_void(d1) ) BRegions(nreg+i).ncts = numberof(whereany(Rawxy, d1));
                     d1 = fits_bintable_peek(bregfile+"+1",i,"N_OUTLINE");
                     BRegions(nreg+i).n_outline = &d1;
                     d1 = fits_bintable_peek(bregfile+"+1",i,"P_OUTLINE");
                     BRegions(nreg+i).p_outline = &d1;
                     d1 = fits_bintable_peek(bregfile+"+1",i,"X_OUTLINE");
                     BRegions(nreg+i).x_outline = &d1;
                     d1 = fits_bintable_peek(bregfile+"+1",i,"Y_OUTLINE");
                     BRegions(nreg+i).y_outline = &d1;
                  }
                  d1 = fits_bintable_peek(bregfile+"+1",i,"PARAMS");
                  BRegions(nreg+i).params = &d1;
                  write,"Loading BRegion "+itoa(nreg+i);
                  write,d1;
                  if( region_type == "simple" ) {
                     if( BRegions(nreg+i).shape == "v" ) {
                        BRegions(nreg+i).ncts = 0;
                     } else {
                        d1 = selreg( im, BRegions(nreg+i).shape, *BRegions(nreg+i).params );
                        BRegions(nreg+i).ncts = is_void(d1) ? 0 : numberof(whereany(Rawxy, d1));
                     }
                  }
               }
            } else { // load the specified one(s)
              //  --  the source region
               type  = rdfitscol(sregfile+"+1","TYPE");
               shape = rdfitscol(sregfile+"+1","SHAPE");
               xcen = rdfitscol(sregfile+"+1","XCEN");
               ycen = rdfitscol(sregfile+"+1","YCEN");
               area = rdfitscol(sregfile+"+1","AREA");
               ncts = rdfitscol(sregfile+"+1","NCTS");
               SRegions(nreg+1).type  = type(n);
               SRegions(nreg+1).shape = shape(n);
               SRegions(nreg+1).xcen = xcen(n);
               SRegions(nreg+1).ycen = ycen(n);
               SRegions(nreg+1).area = area(n);
               SRegions(nreg+1).ncts = ncts(n);
   
               if( region_type != "simple" ) {
                  d1 = fits_bintable_peek(sregfile+"+1",n,"IMSEGMENT");
                  SRegions(nreg+1).imsegment = &d1;
                  // update number of counts
                  if( !is_void(d1) ) SRegions(nreg+1).ncts = numberof(whereany(Rawxy, d1));
                  d1 = fits_bintable_peek(sregfile+"+1",n,"N_OUTLINE");
                  SRegions(nreg+1).n_outline = &d1;
                  d1 = fits_bintable_peek(sregfile+"+1",n,"P_OUTLINE");
                  SRegions(nreg+1).p_outline = &d1;
                  d1 = fits_bintable_peek(sregfile+"+1",n,"X_OUTLINE");
                  SRegions(nreg+1).x_outline = &d1;
                  d1 = fits_bintable_peek(sregfile+"+1",n,"Y_OUTLINE");
                  SRegions(nreg+1).y_outline = &d1;
               }
               d1 = fits_bintable_peek(sregfile+"+1",n,"PARAMS");
               SRegions(nreg+1).params = &d1;
               write,"Loading SRegion "+itoa(nreg+1);
               write,d1;
               if( region_type == "simple" ) {
                  d1 = selreg( im, SRegions(nreg+1).shape, *SRegions(nreg+1).params );
                  SRegions(nreg+1).ncts = is_void(d1) ? 0 : numberof(whereany(Rawxy, d1));
               }
   
              // --  the background region
               type  = rdfitscol(bregfile+"+1","TYPE");
               shape = rdfitscol(bregfile+"+1","SHAPE");
               xcen = rdfitscol(bregfile+"+1","XCEN");
               ycen = rdfitscol(bregfile+"+1","YCEN");
               area = rdfitscol(bregfile+"+1","AREA");
               ncts = rdfitscol(bregfile+"+1","NCTS");
               BRegions(nreg+1).type = type(n);
               BRegions(nreg+1).shape = shape(n);
               BRegions(nreg+1).xcen = xcen(n);
               BRegions(nreg+1).ycen = ycen(n);
               BRegions(nreg+1).area = area(n);
               BRegions(nreg+1).ncts = ncts(n);
   
               if( region_type != "simple" ) {
                  d1 = fits_bintable_peek(bregfile+"+1",n,"IMSEGMENT");
                  BRegions(nreg+1).imsegment = &d1;
                  // update number of counts
                  if( !is_void(d1) ) BRegions(nreg+1).ncts = numberof(whereany(Rawxy, d1));
                  d1 = fits_bintable_peek(bregfile+"+1",n,"N_OUTLINE");
                  BRegions(nreg+1).n_outline = &d1;
                  d1 = fits_bintable_peek(bregfile+"+1",n,"P_OUTLINE");
                  BRegions(nreg+1).p_outline = &d1;
                  d1 = fits_bintable_peek(bregfile+"+1",n,"X_OUTLINE");
                  BRegions(nreg+1).x_outline = &d1;
                  d1 = fits_bintable_peek(bregfile+"+1",n,"Y_OUTLINE");
                  BRegions(nreg+1).y_outline = &d1;
               }
               d1 = fits_bintable_peek(bregfile+"+1",n,"PARAMS");
               BRegions(nreg+1).params = &d1;
               write,"Loading BRegion "+itoa(nreg+1);
               write,d1;
               if( region_type == "simple" ) {
                  if( BRegions(nreg+1).shape == "v" ) {
                     BRegions(nreg+1).ncts = 0;
                  } else {
                     d1 = selreg( im, BRegions(nreg+1).shape, *BRegions(nreg+1).params );
                     BRegions(nreg+1).ncts = numberof(whereany(Rawxy, d1));
                  }
               }
            }
         }
//
// ************************************** ps *********************************************************
//
      } else if( cmd == "ps" ) { // comps - dump plot to ps file
         cans = comparse(command_string, "s");
         plotsign;
         plotname,"Produced by aspec ("+logfilename+")";
         if( is_void(*cans.sparms) ) zps;
         else zps,outfile=(*cans.sparms)(1);
//
// ************************************** ss *********************************************************
//
      } else if( cmd == "ss" ) { // comss - save spectrum and background
         // accept up to four parameters (keyvalues)
         //    1) single number - the region number e.g. 'ss 3'
         //    2) by keyword e.g. 'ss reg=3'
         //    3) by keyword only e.g. 'ss ignbkg=1 reg=3' (remember no spaces)
         bb = comparse(command_string,"ssss");
         nbb = numberof(*bb.sparms);
         ireg = 0;
         ignbkg = 0; // ignore background
         if( nbb ) {
            for( i = 1; i <= nbb; i++ ) {
               // get the 'keyword=value' if this format is entered
               //  else [] is returned
               kbb = keyparse((*bb.sparms)(i));
               if( is_void(kbb) ) { // simple value interpreted as the region number
                  cans = comparse(command_string, "i");
                  ireg = (*cans.iparms)(i);
                  write,"Setting ireg ("+itoa(ireg)+") from simple value ...";
               } else {
                  if( kbb.keyword == "ignbkg" ) {
                     ignbkg = atoi(kbb.keyvalue);
                     write,"Setting ignbkg ("+itoa(ignbkg)+") from keyword ...";
                  }
                  if( kbb.keyword == "reg" ) {
                     ireg = atoi(kbb.keyvalue);
                     write,"Setting ireg ("+itoa(ireg)+") from keyword ...";
                  }
               }
            }
         }
         nreg = numberof(SRegions);
         if( ireg == 0 ) ireg = nreg;
         gen_log_entry,1,"Extracting spectrum for region # "+itoa(ireg),id=lg;

         //  Extract the spectrum

         if( region_type == "simple" ) {
            sel = selreg( im, SRegions(ireg).shape, *SRegions(ireg).params); 
            s_sel = whereany(Rawxy, sel);
            sel = BRegions(ireg).shape == "v" ? [] : selreg( im, BRegions(ireg).shape, *BRegions(ireg).params);
            b_sel = is_void(sel) ? [] : whereany(Rawxy,sel);
         } else {
            s_sel = whereany(Rawxy, *SRegions(ireg).imsegment);
            b_sel = BRegions(ireg).shape == "v" ? [] : whereany(Rawxy, *BRegions(ireg).imsegment);
         }
         if( ignbkg ) b_sel = [];

         //     Rate_src (Rate_err_src) is for total counts in source region
         //     Rate_bkg (Rate_err_bkg) is for renormalized counts in background region
         //     Rate_net (Rate_err_net) is for background subtracted counts

         specbinning, Energy(s_sel), E_min, E_max, Rate_src, Rate_err_src, exposure=Exposure;
         if( is_void(b_sel) ) {
            // if background is not considered make null spectrum
            Rate_bkg = Rate_err_bkg = Rate_src*0;
         } else {
            specbinning, Energy(b_sel), E_min, E_max, Rate_bkg, Rate_err_bkg, exposure=Exposure;
         }

         //  Renormalize background to adapt to source region size

         corf = float(SRegions(ireg).area)/BRegions(ireg).area;
         Rate_bkg *= corf;    // normalize to source area
         Rate_err_bkg *= corf;

         //  Get the net spectrum

         Rate_net = Rate_src - Rate_bkg;
         Rate_err_net = sqrt(Rate_err_src^2 + Rate_err_bkg^2);

         //  Renormalize the spectrum to pixel or unit area if requested

         if( Spec_norm_rule == 2 ) { // per pixel
            Spec_norm_factor = 1.0/SRegions(ireg).area;
         } else if( Spec_norm_rule == 3 ) { // per cm2
            Spec_norm_factor = 1.0/(SRegions(ireg).area*pixel_area_cm2);
         } else Spec_norm_factor = 1.0;

         Rate_src *= Spec_norm_factor;
         Rate_err_src *= Spec_norm_factor;
         Rate_bkg *= Spec_norm_factor;
         Rate_err_bkg *= Spec_norm_factor;
         Rate_net *= Spec_norm_factor;
         Rate_err_net *= Spec_norm_factor;

         // select the ARF based on distance to center
         xoptaxis = Coords.flag ? Coords.crpix1 : xdetcen;
         yoptaxis = Coords.flag ? Coords.crpix2 : ydetcen;
         gen_log_entry,1,"xdetcen = "+ftoa(xdetcen,ndec=2),id=lg;
         gen_log_entry,1,"xoptaxis = "+ftoa(xoptaxis,ndec=2),id=lg;
         gen_log_entry,1,"ydetcen = "+ftoa(ydetcen,ndec=2),id=lg;
         gen_log_entry,1,"yoptaxis = "+ftoa(yoptaxis,ndec=2),id=lg;
         dist = sqrt((SRegions(ireg).xcen - xoptaxis)^2 + (SRegions(ireg).ycen - yoptaxis)^2);
         //  now in pixels, convert to arcmin
         dist *= (pixel_size*180.*60.)/(10140.*pi);
         gen_log_entry,1,"dist = "+ftoa(dist,ndec=3)+" arcmin",id=lg;
         // given that the focal length is 10140 mm
         // find number of ARFs in the array
         dms = dimsof(Arfs);
         num_arfs = dms(3);
         d = (indgen(num_arfs) - Scal_b) * Scal_a;
         d(1) = 0.0; // otherwise the code becomes more complicated [2013-05-06/NJW]
         w = where( d > dist ); // now w(1) is the index on one side
                 // and w(1)-1 is the index on the other side
         if( !numberof(w) ) {
            write,"The region center is too far away";
         } else {

            // Allways make three spectral files: net, total, and bkg
            // Place these statements before the ARF production to reuse
            // the Spec_ser_str for name-giving
            if( file_test("specserial.txt") ) serial_num = atoi(rdfile("specserial.txt"))+1;
            else serial_num = 0;
            Spec_ser_str = itoa(serial_num,4);
            write_slist,"specserial.txt",Spec_ser_str;
            spec_net_file  = "spec_net_"+Spec_ser_str+".pha";
            spec_bkg_file  = "spec_bkg_"+Spec_ser_str+".pha";
            spec_tot_file  = "spec_tot_"+Spec_ser_str+".pha";

            // NB! This scheme only works if 'd' begins with zero
            //   see adhoc statment above
            i = w(1) - 1;
            gen_log_entry,1,"ARF selection i = "+itoa(i),id=lg;
            arf = float(Arfs(,i) + ((dist - d(i))/(d(i+1)-d(i)))*(Arfs(,i+1)-Arfs(,i)));
            arffile = fullpath("arf_"+Spec_ser_str+".arf");
            arf2phaii, arffile, arf, Energ_lo, Energ_hi, extname="SPECRESP",\
                instrume=instrume,telescop=telescop;
            write,"ARF file made : "+arffile;
            gen_log_entry,1,"ARF saved into: "+arffile,id=lg;

            kwds_init;
            kwds_set,"NUMSRCPX", SRegions(ireg).area,"Number of source pixels";
            kwds_set,"NUMBKGPX", BRegions(ireg).area,"Number of background pixels";
            kwds_set,"REGXCEN", SRegions(ireg).xcen,"X pixel for region center";
            kwds_set,"REGYCEN", SRegions(ireg).ycen,"Y pixel for region center";
            kwds_set,"SRGSHAPE", SRegions(ireg).shape,"Shape of source region";
            kwds_set,"BRGSHAPE", BRegions(ireg).shape,"Shape of background region";
            kwds_set,"NORMFACT", Spec_norm_factor,"Spectral renormalization factor";
            kwds_set,"NORMTEXT", Spec_norm_text,"Explanation";
            kwds_set,"ORIGIN","aspec-"+Version,"Software that produced it";

            // Write net spectrum

            // Note that here the background file is omitted so that it is not
            // subtracted once more - the keyword 'HDUCLAS2='NET'' does not prevent
            // xspec from subtracting the background spectrum

            spec2phaii,spec_net_file,Rate_net,Rate_err_net, \
               ra_obj=[float(1.)*ireg],\
               dec_obj=[float(1.)*ireg], \
               exposure=[Exposure], \
               name=["source"+itoa(ireg)], \
               ancrfile=[arffile],\
               backfile=[""], \
               type="net", \
               respfile=Rmf_file, \
               telescop=telescop, \
               instrume=instrume, \
               no_kwds_init=1;
            write,"Net spectrum has been written to : "+spec_net_file;
            gen_log_entry,1,"Net spectrum has been written to : "+spec_net_file,id=lg;

            // Write bkg spectrum

            spec2phaii,spec_bkg_file,Rate_bkg,Rate_err_bkg, \
               ra_obj=[float(1.)*ireg],\
               dec_obj=[float(1.)*ireg], \
               exposure=[Exposure], \
               name=["source"+itoa(ireg)], \
               ancrfile=[arffile],\
               type="bkg", \
               respfile=Rmf_file, \
               telescop=telescop, \
               instrume=instrume, \
               no_kwds_init=1;
            write,"Background spectrum has been written to : "+spec_bkg_file;
            gen_log_entry,1,"Background spectrum has been written to : "+spec_bkg_file,id=lg;

            // Write total spectrum

            spec2phaii,spec_tot_file,Rate_src,Rate_err_src, \
               ra_obj=[float(1.)*ireg],\
               dec_obj=[float(1.)*ireg], \
               exposure=[Exposure], \
               name=["source"+itoa(ireg)], \
               ancrfile=[arffile],\
               respfile=Rmf_file, \
               backfile=[spec_bkg_file], \
               type="total", \
               telescop=telescop, \
               instrume=instrume, \
               no_kwds_init=1;
            write,"Total spectrum has been written to : "+spec_tot_file;
            gen_log_entry,1,"Total spectrum has been written to : "+spec_tot_file,id=lg;
         }
//
// ************************************** normspec *********************************************************
//
      } else if( cmd == "normspec" ) { // comnormspec - normalize spectrum
         cans = comparse(command_string,"s");
         if( !is_void(*cans.sparms) ) {
            if( (*cans.sparms)(1) == "1" ) {
               Spec_norm_rule = 1;
               Spec_norm_text = "Normalized to source region area";
            } else if( (*cans.sparms)(1) == "pixel" ) {
               Spec_norm_rule = 2;
               Spec_norm_text = "Normalized to a single pixel";
            } else if( (*cans.sparms)(1) == "cm2" ) {
               Spec_norm_rule = 3;
               Spec_norm_text = "Normalized to a cm2 on the detector";
            }
            write,"Next spectrum will be "+Spec_norm_text;
         }
//
// ************************************** pspec *********************************************************
//
      } else if( cmd == "pspec" ) { // compspec  - plot most recent spectrum
         cans = comparse(command_string,"r");
         frac = is_void(*cans.rparms) ? 0.2 : (*cans.rparms)(1);
         //+ 101123 plot_spectrum,E_min,E_max,Rate_src,Rate_err_src,itype=3;
         // Rebin and plot total spectrum from region
         local ob1, ob2, orate, orate_err;
         REBIN = specrebinninga( E_min, E_max, Rate_src, Rate_err_src, frac, ob1, ob2, orate, orate_err);
         plot_spectrum, ob1, ob2, orate, orate_err, itype=3;
         if(!is_void(b_sel)) {
            // If a background has been chosen then add the background spectrum in red with same binning
            //+ specrebinninga, E_min, E_max, Rate_bkg, Rate_err_bkg, frac, ob1, ob2, orate, orate_err;
            specrebinning, E_min, E_max, Rate_bkg, Rate_err_bkg, REBIN, ob1, ob2, orate, orate_err;
            oplot_spectrum, ob1, ob2, orate, orate_err,color="red";
         }
         //+ specrebinninga, E_min, E_max, Rate_net, Rate_err_net, frac, ob1, ob2, orate, orate_err;
         // Finally add the net spectrum in green, again with same binning
         specrebinning, E_min, E_max, Rate_net, Rate_err_net, REBIN, ob1, ob2, orate, orate_err;
         oplot_spectrum, ob1, ob2, orate, orate_err,color="green";
//
// ************************************** ? *********************************************************
//
      } else if( cmd == "?" ) { // com?
         cans = comparse(command_string,"s");
         if( is_void(*cans.sparms) ) { // show overview
            write,"info        show current settings,";
            write,"ebds        set energy boundaries,";
            write,"imdisp      display detector image between current energy limits,";
            write,"lmdisp      display detector logarithmic image between current energy limits,";
            write,"pal         define the name of the palette to use for images,";
            write,"ps          dump current image to .ps file,";
            write,"imsave      write image between current energy limits to FITS file,";
            write,"evtsave     write events in region to an event file (not implemented),";
            write,"mreg        make a region pair,";
            write,"dreg        delete a region pair,";
            write,"sreg        save region pair(s),";
            write,"lreg        load region pair(s),";
            write,"ireg        show info on loaded/created regions,";
            write,"normspec    setup a normalization of spectra,";
            write,"ss          create and save spectrum,";
            write,"pspec       plot most recent spectrum,";
            write,"rpos        read position in image in pixels and in WCS,";
            write,"mpos        mark position given by RA,Dec in image,";
            write,"gpix        get pixel numbers from WCS coordinates,";
            write,"batch       take commands from named batch file,";
            write,"x           exit aspec,";
            write,"?           help - this overview,";
            write,"? <command> help on specified command.";
         } else {
            if( (*cans.sparms)(1) == "imdisp" || (*cans.sparms)(1) == "lmdisp" ) {
               write,"imdisp [sigma] - display image for current setting of emin and emax";
               write,"lmdisp [sigma] - display logarithmic image for current setting of emin and emax";
               write,"                A Gauss kernel smoothing is applied if sigma is given.";
            } else if( (*cans.sparms)(1) == "info" ) {
               write,"info      - show information on current settings";
            } else if( (*cans.sparms)(1) == "pal" ) {
               write,"pal name  - Define name of palette (the .gp postfix may be left out)";
            } else if( (*cans.sparms)(1) == "ebds" ) {
               write,"ebds [emin [emax]] - set the energy boundaries for";
               write,"                image display and number counts in regions.";
            } else if( (*cans.sparms)(1) == "imsave" ) {
               write,"imsave [filename] - save image for current setting of emin and emax to FITS file";
               write,"                The file name will be automatically assigned unless given.";
            } else if( (*cans.sparms)(1) == "mreg" ) {
               write,"mreg [shape] - define a region pair, src + bkg, by help of the cursor";
               write,"          shape (if given, default is 'ca') must be a two letter string.";
               write,"                The first letter refers to the source region and it can";
               write,"                one of \"c\" (circle) and \"b\" (box).";
               write,"                The second letter refers to the background region and can";
               write,"                be one of \"a\" (annulus with same center as the source),";
               write,"                \"b\" (box), and \"v\" (void, no background will be subtracted).";
            } else if( (*cans.sparms)(1) == "ireg" ) {
               write,"ireg [number] - show region properties for all regions";
               write,"               unless the requested one is given.";
            } else if( (*cans.sparms)(1) == "dreg" ) {
               write,"dreg [number] - delete all regons";
               write,"               unless the requested one is given.";
            } else if( (*cans.sparms)(1) == "sreg" ) {
               write,"sreg [number] - save a region pair, src + bkg, into files";
               write,"               s_nnnn.freg and b_nnnn.freg, where 'nnnn'"
               write,"               is a serial number. 'number' is the number";
               write,"               in the current region list. If omitted then the most recent";
               write,"               region will be saved. If set to zero the all regions will be saved.";
            } else if( (*cans.sparms)(1) == "lreg" ) {
               write,"lreg nnnn [number] - load a region pair, src + bkg, from files";
               write,"               s_nnnn.freg and b_nnnn.freg, where 'nnnn'"
               write,"               is a serial number. 'number' is the row number";
               write,"               in the table. If omitted the first region will be";
               write,"               loaded, if zero all regions (rows) will be loaded.";
               write,"               The number of counts information will be updated";
               write,"               based on the current event list.";
            } else if( (*cans.sparms)(1) == "normspec" ) {
               write,"normspec word - setup the spectral normalization";
               write,"          1      return to spectrum in source area";
               write,"          cm2    give spectrum per cm2 on the detector";
               write,"          pixel  give spectrum per pixel on the detector";
               write," Must be defined before the spectrum is extracted and saved.";
            } else if( (*cans.sparms)(1) == "ss" ) {
               write,"ss [region]  - create and save spectrum with standard filename";
               write,"    The serial number is kept in the external 'Spec_ser_str'.";
               write,"    'region' is the number from the list of regions,";
               write,"    if not given the highest region number will be used.";
               write,"ss ignbkg=1 [reg=n]   alternative input";
               write,"  In this case the background is omitted. The region number";
               write,"  must be entered in the keyword style.";
               write,"  If spectral normalization is requested then the \"normspec\"";
               write,"  command must be given first.";
            } else if( (*cans.sparms)(1) == "pspec" ) {
               write,"pspec [error_fraction]  - plot the most recent spectrum.";
               write,"         A rebinning is done so that the relative error is";
               write,"         less than 0.2. A different value for this fraction";
               write,"         can be given.";
            } else if( (*cans.sparms)(1) == "rpos" ) {
               write,"rpos  -  a position in an image is marked by the cursor";
               write,"         and the coordinates are printed in pixels and in WCS.";
               write,"         The pixel value is also printed.";
            } else if( (*cans.sparms)(1) == "mpos" ) {
               write,"mpos RA dec  Mark the position in the image for the";
               write,"             indicated celestial position";
            } else if( (*cans.sparms)(1) == "gpix" ) {
               write,"gpix RA dec  Print pixel values for the indicated celestial position";
               write,"         in decimal degrees.";
            } else if( (*cans.sparms)(1) == "batch" ) {
               write,"batch filename - take commands from the named file";
               write,"         and return to prompt (if the last command isn't 'x').";
            } else if( (*cans.sparms)(1) == "ps" ) {
               write,"ps [plotfilename] -  produce a PS file from current plot.";
               write,"         A specific name may be given.";
            } else if( (*cans.sparms)(1) == "?" ) {
               write,"? [command] - get more detailed help on command";
            }
         }
      }
   }
}

/* Function comparse */

func comparse( str, types )
/* DOCUMENT res = comparse( command_string, types )

     Returns an instance of the struct: s_Command
        res.command    holds the command itself
        res.rparms     is a pointer to the real parameters
        res.iparms     is a pointer to the integer parameters
        res.sparms     is a pointer to the string parameters

    Example:
       > res = comparse( "mreg circle 22 1.0 2.0","sirr" )
       > res.command
       "mreg"
       > *res.sparms
       ["circle"]
       > *res.iparms
       [22]
       > *res.rparms
       [1,2]
*/
{
   // parameters must be space separated

   command = s_Command();
   str = strtrim(strcompress(str));
   keys = strsplit(str," ");
   nkeys = numberof(keys);
   // initialize
   s_nill = [];
   command.rparms = &s_nill;
   command.iparms = &s_nill;
   command.sparms = &s_nill;

   if( nkeys == 1 ) {
      command.command = keys;
   } else {
      command.command = keys(1);
      if( typeof(types) == "string" ) {
         ntypes = strlen(types);
         if( ntypes < nkeys-1 ) {
            write,"comparse: too many keywords - truncated";
            nkeys = ntypes+1;
            keys = keys(nkeys);
         }
         vreal = vint = vstr = [];
         if( ntypes ) {
            for( i = 1; i < nkeys; i++ ) {
               strp = keys(i+1);
               typ = strpart(types,i:i);
               if( typ=="r" ) {
                  grow,vreal,atof(strp);
               } else if( typ=="i" ) {
                  grow,vint,atoi(strp);
               } else if( typ=="s" ) {
                  grow,vstr,strp;
               } else {
                  error,"Invalid type definition";
               }
            } // for( i = ...
         } // if( ntypes )
         command.rparms = &vreal;
         command.iparms = &vint;
         command.sparms = &vstr;
      }
   }
   return command;
}

/* Function keyparse */

func keyparse( str )
/* DOCUMENT res = keyparse( str )

    Interprets a string of shape 'keyword=value' that contains no spaces.
    If the input string fulfills this requirement a struct (s_Keyword)
    is returned, else nil.

     2012-01-16/NJW
*/
{
   // 'str' must be a string
   if( typeof(str) != "string" ) return [];
   // a '=' sign must be present
   pe = strpos( str, "=" );
   if( pe == 0 ) return [];
   // no spaces can be present
   if( strpos(str," ") ) return [];
   // the '=' cannot be the first nor the last character
   len = strlen(str);
   if( pe == 1 || pe == len) return [];
   res = s_Keyword();
   res.keyword = strpart( str, 1:pe-1 );
   res.keyvalue = strpart( str, pe+1:len );

   return res;
}
