/* Function argreg */

func argreg( shape, params_in, &xcen, &ycen, &params )
/* DOCUMENT s = argreg( shape, params_in, >xcen, >ycen, >params )

         shape    params_in
           c       RA Dec radius          (in arcmin)
           a       RA Dec radius1 radius2 (in arcmin)
           b       RA1 Dec1 RA2 Dec2

    The WCS struct 'Coords' must exist as an external variable.
*/
{
   extern Coords;

   if( shape == "c" ) {
      skypos_fits, Coords, xcen, ycen, params_in(1), params_in(2), to_pix=1,silent=1;
      pix2deg = sqrt(Coords.cd1_1^2 + Coords.cd2_2^2);
      radius_pix = (params_in(3)/60.) / pix2deg;
      params = [xcen, ycen, radius_pix];
   } else if( shape == "a" ) {
      skypos_fits, Coords, xcen, ycen, params_in(1), params_in(2), to_pix=1,silent=1;
      pix2deg = sqrt(Coords.cd1_1^2 + Coords.cd2_2^2);
      radius_pix1 = (params_in(3)/60.) / pix2deg;
      radius_pix2 = (params_in(4)/60.) / pix2deg;
      params = [xcen, ycen, radius_pix1, radius_pix2];
   } else if( shape == "b" ) {
      local x1, y1, x2, y2;
      skypos_fits, Coords, x1, y1, params_in(1), params_in(2), to_pix=1,silent=1;
      skypos_fits, Coords, x2, y2, params_in(3), params_in(4), to_pix=1,silent=1;
      if( x2 < x1 ) { z = x1; x1 = x2; x2 = z;}
      if( y2 < y1 ) { z = y1; y1 = y2; y2 = z;}
      params = [x1, y1, x2, y2];
      xcen = 0.5*( x1 + x2 );
      ycen = 0.5*( y1 + y2 );
   } else return 0;

   return 1;

}

/* Function curreg */

func curreg( im, shape, &xcen, &ycen, &params, color=, thick= )
/* DOCUMENT s = curreg( im, shape, >xcen, >ycen, >params, color=, thick= )

     A function to make a cursor marked region in an image.

     Returns an array with image indices.

     The shape can be "c" for circular (default), "a" for annulus, or "b" for box.
     When "d" is given an annulus with center (xcen,ycen) is defined.

     Returned in 'params':
           circular : [xcen, ycen, radius]
           annulus  : [xcen, ycen, radius1, radius2]
           box      : [x1, y1, x2, y2]
*/
{
   dms = dimsof(im);
   if( is_void(shape) ) shape = "c";
   if( typeof(shape) != "string" ) error,"CURREG: Second argument must be a string";
   sh = strpart(shape,1:1);
   if( sh == "c" ) { // circular region
      cent = curmark1(prompt="Mark the center ...");
      xcen = cent(1);
      ycen = cent(2);
      rad  = curmark1(prompt="Mark the periphery ...");
      v = span(0,2*pi,50);
      write,"     - calculating ...";
      r = sqrt((xcen-rad(1))^2 + (ycen-rad(2))^2);
      params = [xcen, ycen, r];
      d = distances(dms(2),dms(3),xcen,ycen);
      w = where(d < r);
      oplot,xcen+r*cos(v),ycen+r*sin(v),color=color,thick=thick;
   } else if( sh == "a" || sh == "d" ) { // annular region
      if( sh == "a" ) {
         cent = curmark1(prompt="Mark the center ...");
         xcen = cent(1);
         ycen = cent(2);
      }
      v = span(0,2*pi,50);
      write,"     - calculating ...";
      d = distances(dms(2),dms(3),xcen,ycen);

      rad  = curmark1(prompt="Mark the inner periphery ...");
      r1 = sqrt((xcen-rad(1))^2 + (ycen-rad(2))^2);
      oplot,xcen+r1*cos(v),ycen+r1*sin(v),color=color,thick=thick;
      rad  = curmark1(prompt="Mark the outer periphery ...");
      r2 = sqrt((xcen-rad(1))^2 + (ycen-rad(2))^2);
      params = [xcen, ycen, r1, r2];
      oplot,xcen+r2*cos(v),ycen+r2*sin(v),color=color,thick=thick;
      w = where( d > r1 & d < r2 );
   } else {          // box region
      corners = curmark1(prompt="Mark the box corners ...",style=2);
      x1 = min(corners(1:3:2));
      x2 = max(corners(1:3:2));
      xcen = 0.5*(x1+x2);
      y1 = min(corners(2:4:2));
      y2 = max(corners(2:4:2));
      params = [x1, y1, x2, y2 ];
      ycen = 0.5*(y1+y2);
      xnet = indgen(dms(2))(,-:1:dms(3));
      ynet = indgen(dms(3))(-:1:dms(2),);
      write,"     - calculating ...";
      w = where(xnet > x1 & xnet < x2 & ynet > y1 & ynet < y2 );
      oplot,[x1,x2,x2,x1,x1],[y1,y1,y2,y2,y1],color=color,thick=thick;
   }
   return w;
}

/* Function selreg */

func selreg( im, shape, params )
/* DOCUMENT s = selreg( im, shape, params )

     A function to return selected pixels from a region defined
     by 'shape' and 'params'.

     Returns an array with image indices.

     The shape can be "c" for circular (default), "a" for annulus, or "b" for box.
     When "d" is given an annulus with center (xcen,ycen) is defined.

*/
{
   dms = dimsof(im);
   if( typeof(shape) != "string" ) error,"SELREG: Second argument must be a string";
   sh = strpart(shape,1:1);
   if( sh == "c" ) { // circular region
      // params is [xcen, ycen, radius]
      d = distances(dms(2),dms(3),params(1),params(2));
      w = where(d < params(3));
   } else if( sh == "a" || sh == "d" ) { // annular region
      // params is [xcen, ycen, radius1, radius2]
      d = distances(dms(2),dms(3),params(1),params(2));
      w = where( d > params(3) & d < params(4) );
   } else if( sh == "b" ) {          // box region
      // params is [x1, y1, x2, y2 ]
      xnet = indgen(dms(2))(,-:1:dms(3));
      ynet = indgen(dms(3))(-:1:dms(2),);
      w = where(xnet > params(1) & xnet < params(3) & ynet > params(2) & ynet < params(4) );
   } else { // unknown or void "v"
      return [];
   }
   return w;
}
