extern datafitdoc;
/* DOCUMENT datafit *********************************
 *    Collection of data fitting and function
 *    minimization functions

                            gaussfit2d_uam        
      _reduce_p             gaussfit2ds_uam       
      _reduce_y             gaussfit_2d           
      amoeba                gaussfit_am           
      amoeba_freeze         gaussfitx             
      brent                 golden                
      edf_modelf            lin_regress           
      expodecayfit          linmin                
      f1dim                 lmfit                 
      fte1                  mnbrak                
      fte2d                 model_amoeba          
      ftex                  modela                
      funk_amoeba           modelb                
      funka                 powell                
      funkb                 prepare_gf2d          
      gaussfit              autoesti

      2008-11-03/NJW, added autoesti 2010-11-22
****************************************************/

/* Function lin_regress */

func lin_regress( x, y, dy )
/* DOCUMENT coefs = lin_regress( x, y, dy )

   Linear regression for a single independent variable 'x'
   Measurements are 'y' with errors 'dy'

   yfit = coefs(1) + coefs(2) * x

   2007-01-05/NJW

*/
{
   require, "matrix.i";

   mat = array(double, 2,2 );
   rhs = array(double, 2);
   n = numberof(x);
   for( i = 1; i <= n; i++ ) {
      weight = 1./(dy(i)*dy(i));
      mat(2,2) += x(i)*x(i)*weight;
      mat(1,2) += x(i)*weight;
      mat(1,1) += weight;
      rhs(2) += x(i)*y(i)*weight;
      rhs(1) += y(i)*weight;
   }
   mat(2,1) = mat(1,2);
   return LUsolve(mat,rhs);
}


/* Function gaussfit2d_uam */

func gaussfit2d_uam(x, y, z, ftol=, nterm=, esti=, delta=, chat= )
/* DOCUMENT coefs = gaussfit2d_uam( x, y, ftol=, nterm=, esti=, delta=, chat= )

     Perform a 2D gauss fit for array 'z' with coordinates in 'x' and 'y'
     It returns a double array:
          [amplitude, mean_x, mean_y, sigma_x, sigma_y, bkg].

     This is the 'unconstrained sampling' version where the coordinate
     pairs (x,y) can be chosen freely in contrast to the mesh version
     - gaussfit2d_mam - where (x,y) belong to an MxN mesh of values.
     Here x, y, and z must be 1D arrays of same number of elements.

     If nterm == 5 or not given:
     yfit = coefs(1)*exp(-0.5*(((x-coefs(2))/coefs(4))^2 \
                            + ((y-coefs(3))/coefs(5))^2))
     If nterm == 6:
     yfit = <previous expression> + coefs(6)

     KEYWORD   ftol: fitting tolerance default value: 1.e-4
              nterm: number of terms (5 or 6)
               esti: estimated start values of coefficients
              delta: fractional parameter adjustment [default: 0.1]
               chat: chattiness level 1: some, 5: a LOT

     SEE ALSO:  autoesti

     2006-02-23/NJW  Cloned from gaussfit_am.i

*/
{

   require, "varfuncs.i";

   extern xval, yval, zval, n_points;

   if( is_void(nterm) ) nterm = 5;
   if( nterm != 5 && nterm != 6 ) {
      print,"Keyword nterm must be 5 or 6";
      return -1;
   }
   if( is_void(delta) ) delta = 0.1;
   if( is_void(ftol) ) ftol = 1.e-4;
   if( is_void(chat) ) chat = 0;

   p = array(double,nterm+1,nterm);
   parm = array(double,nterm);
   delta_parm = array(double,nterm);
   chi2_arr = array(double,nterm+1);

   xval = x;
   yval = y;
   zval = z;

   n_points = numberof(x);
   if( chat > 2 ) print,"n_points = ", n_points;


   /* parm(1) is amplitude,
      parm(2) is mean_x,
      parm(3) is mean_y,
      parm(4) is sigma_x,
      parm(5) is sigma_y,
      parm(6) is background if nterm==6 */

   /* prediction of values */

   if( is_void(esti) ) {
      // First estimation - broad peak
      parm_b = parm;
      sumn = sum(z);
      mean_x = sum(x*z) / sumn;
      stdev_x = (sum(x*x*z) - sumn*mean_x*mean_x)/sumn;
      mean_y = sum(y*z) / sumn;
      stdev_y = (sum(y*y*z) - sumn*mean_y*mean_y)/sumn;
      parm_b(1) = max(z);
      parm_b(2) = mean_x;
      parm_b(3) = mean_y;
      parm_b(4) = sqrt(stdev_x);
      parm_b(5) = sqrt(stdev_y);
      if( nterm == 6 ) {
         r = sqrt(((x-mean_x)/stdev_x)^2 + ((y-mean_y)/stdev_y)^2);
         w = where( r > 2. );
         parm_b(6) = numberof(w) > 0 ? avg(z(w)) : 0.0;
         parm_b(1) -= parm_b(6);
      }
      if( chat > 2 ) {
         print,"start parm_b: ", parm_b;
         print,"chi2_b: ", funk(parm_b);
      }
      // Next estimation - narrow peak
      parm_n = parm;
      w = where( z > avg(z) );
      sumn = sum(z(w));
      mean_x = sum(x(w)*z(w)) / sumn;
      stdev_x = (sum(x(w)*x(w)*z(w)) - sumn*mean_x*mean_x)/sumn;
      mean_y = sum(y(w)*z(w)) / sumn;
      stdev_y = (sum(y(w)*y(w)*z(w)) - sumn*mean_y*mean_y)/sumn;
      parm_n(1) = max(z(w));
      parm_n(2) = mean_x;
      parm_n(3) = mean_y;
      parm_n(4) = sqrt(stdev_x);
      parm_n(5) = sqrt(stdev_y);
      if( nterm == 6 ) {
         r = sqrt(((x-mean_x)/stdev_x)^2 + ((y-mean_y)/stdev_y)^2);
         w = where( r > 2. );
         parm_n(6) = numberof(w) > 0 ? avg(z(w)) : 0.0;
         parm_n(1) -= parm_n(6);
      }
      if( chat > 2 ) {
         print,"start parm_n: ", parm_n;
         print,"chi2_n: ", funk(parm_n);
      }
      //parm = funk(parm_n) < funk(parm_b) ? parm_n : parm_b;
      chi2_b = funk(parm_b);
      chi2_n = funk(parm_n);
      if( chi2_n < chi2_b ) {
         print,"Choosing parm_n";
         parm = parm_n;
      } else {
         print,"Choosing parm_b";
         parm = parm_b;
      }
      print,"typeof parm: ", typeof(parm);
      print,"dimsof parm: ", dimsof(parm);

   } else {

      if( nterm != numberof(esti) ) {
         write,"Mismatch between nterm and numberof(esti)";
         return -1;
      }
      parm() = esti();
      if( chat > 2 ) print,"esti input: ", parm;
   }
   if( chat > 5 ) return parm;

   ndim = nterm;
   delta_parm = delta * parm;

   for(i=1; i<=ndim+1; i++ ) {
      for(j=1; j<=ndim; j++) {
         p(i,j) = parm(j);
         if( j+1 == i ) p(i,j) += delta_parm(j);
      }
   }
   for( i=1; i<=ndim+1; i++ ) {
      //for( j=1; j<=ndim; j++ ) parm(j) = p(i,j);
      parm() = p(i,);
      chi2_arr(i) = funk(parm);
      if( chat > 1 ) {
         if( nterm == 6 ) {
            print,"p c2 (init)= ", parm(1),parm(2),parm(3),parm(4),parm(5),parm(6),chi2_arr(i);
         } else {
            print,"p c2 (init)= ", parm(1),parm(2),parm(3),parm(4),parm(5),chi2_arr(i);
         }
      }
   }
   p_res = amoeba( funk, p, chi2_arr, ftol, iter);
   print,"iter = ",iter;
   if( chat > 1 ) {
      for( i=1; i<=ndim+1; i++ ) {
         if( nterm == 6 ) {
            print, p(i,1),p(i,2),p(i,3),p(i,4),p(i,5),p(i,6),chi2_arr(i);
         } else {
            print, p(i,1),p(i,2),p(i,3),p(i,4),p(i,5),chi2_arr(i);
         }
      }
      if( nterm == 6 ) {
         print,"Amplitude   Meanx  Meany     Sigmax  Sigmay   Bckg    Chi2";
      } else {
         print,"Amplitude   Meanx  Meany     Sigmax  Sigmay    Chi2";
      }
   }
   parm() = p(1,);

   return parm;

}

/* Function funk_amoeba */

func funk_amoeba(parm)
/* The Chi-square function */
{

   extern xval, yval, zval, n_points;

   arg = zval - model_amoeba(xval, yval, parm);
   return sum(arg^2);
}

/* Function model_amoeba */

func model_amoeba(x, y, parm)
{

   arg = ((x-parm(2))/parm(4))^2 + ((y-parm(3))/parm(5))^2;
   term1 = parm(1) * exp(-0.5*arg);

   return numberof(parm) == 6 ? term1+parm(6) : term1;
}

/* Function funkb */

func funkb(parm)
/* The Chi-square function */
{

   extern xval, yval, zval, n_points;

   arg = zval - modelb(xval, yval, parm);
   return sum(arg^2);
}

/* Function modelb */

func modelb(x, y, parm)
{

   arg = ((x-parm(2))/parm(4))^2 + ((y-parm(3))/parm(5))^2;
   term1 = parm(1) * exp(-0.5*arg);

   return numberof(parm) == 6 ? term1+parm(6) : term1;
}


/* Function gaussfit2ds_uam */

func gaussfit2ds_uam(x, y, z, ftol=, nterm=, esti=, delta=, chat= )
/* DOCUMENT coefs = gaussfit2ds_uam( x, y, ftol=, nterm=, esti=, delta=, chat= )

     Perform a 2D gauss fit for array 'z' with coordinates in 'x' and 'y'
     It returns a double array:
          [amplitude, mean_x, mean_y, sigma, bkg].

     This is the 'unconstrained sampling' version where the coordinate
     pairs (x,y) can be chosen freely in contrast to the mesh version
     - gaussfit2d_mam - where (x,y) belong to an MxN mesh of values.
     Here x, y, and z must be 1D arrays of same number of elements.

     If nterm == 4 or not given:
     yfit = coefs(1)*exp(-0.5*(((x-coefs(2))/coefs(4))^2 \
                            + ((y-coefs(3))/coefs(4))^2))
     If nterm == 5:
     yfit = <previous expression> + coefs(5)

     KEYWORD   ftol: fitting tolerance default value: 1.e-4
              nterm: number of terms (4 or 5)
               esti: estimated start values of coefficients
              delta: fractional parameter adjustment [default: 0.1]
               chat: chattiness level 1: some, 5: a LOT

     See also: gaussfit2d_uam where sigma_x and sigma_y are derived
     SEE ALSO:  autoesti

     2006-02-23/NJW  Cloned from gaussfit_am.i

*/
{
   require,"varfuncs.i";

   extern xval, yval, zval, n_points;

   if( is_void(nterm) ) nterm = 4;
   if( nterm != 4 && nterm != 5 ) {
      print,"Keyword nterm must be 4 or 5";
      return -1;
   }
   if( is_void(delta) ) delta = 0.1;
   if( is_void(ftol) ) ftol = 1.e-4;
   if( is_void(chat) ) chat = 0;

   p = array(double,nterm+1,nterm);
   parm = array(double,nterm);
   delta_parm = array(double,nterm);
   chi2_arr = array(double,nterm+1);

   xval = x;
   yval = y;
   zval = z;

   n_points = numberof(x);
   if( chat > 2 ) print,"n_points = ", n_points;


   /* parm(1) is amplitude,
      parm(2) is mean_x,
      parm(3) is mean_y,
      parm(4) is sigma_x,
      parm(5) is background if nterm==5 */

   /* prediction of values */

   if( is_void(esti) ) {
      // First estimation - broad peak
      parm_b = parm;
      sumn = sum(z);
      mean_x = sum(x*z) / sumn;
      stdev_x = (sum(x*x*z) - sumn*mean_x*mean_x)/sumn;
      mean_y = sum(y*z) / sumn;
      stdev_y = (sum(y*y*z) - sumn*mean_y*mean_y)/sumn;
      stdev = 0.5*(stdev_x + stdev_y);
      parm_b(1) = max(z);
      parm_b(2) = mean_x;
      parm_b(3) = mean_y;
      parm_b(4) = sqrt(stdev);
      if( nterm == 5 ) {
         r = sqrt(((x-mean_x)/stdev)^2 + ((y-mean_y)/stdev)^2);
         w = where( r > 2. );
         parm_b(5) = numberof(w) > 0 ? avg(z(w)) : 0.0;
         parm_b(1) -= parm_b(5);
      }
      if( chat > 2 ) {
         print,"start parm_b: ", parm_b;
         print,"chi2_b: ", funk(parm_b);
      }
      // Next estimation - narrow peak
      parm_n = parm;
      w = where( z > avg(z) );
      sumn = sum(z(w));
      mean_x = sum(x(w)*z(w)) / sumn;
      stdev_x = (sum(x(w)*x(w)*z(w)) - sumn*mean_x*mean_x)/sumn;
      mean_y = sum(y(w)*z(w)) / sumn;
      stdev_y = (sum(y(w)*y(w)*z(w)) - sumn*mean_y*mean_y)/sumn;
      stdev = sqrt(stdev_x*stdev_y);
      parm_n(1) = max(z(w));
      parm_n(2) = mean_x;
      parm_n(3) = mean_y;
      parm_n(4) = sqrt(stdev);
      if( nterm == 5 ) {
         r = sqrt(((x-mean_x)/stdev)^2 + ((y-mean_y)/stdev)^2);
         w = where( r > 2. );
         parm_n(5) = numberof(w) > 0 ? avg(z(w)) : 0.0;
         parm_n(1) -= parm_n(5);
      }
      if( chat > 2 ) {
         print,"start parm_n: ", parm_n;
         print,"chi2_n: ", funk(parm_n);
      }
      //parm = funk(parm_n) < funk(parm_b) ? parm_n : parm_b;
      chi2_b = funk(parm_b);
      chi2_n = funk(parm_n);
      if( chi2_n < chi2_b ) {
         print,"Choosing parm_n";
         parm = parm_n;
      } else {
         print,"Choosing parm_b";
         parm = parm_b;
      }
      print,"typeof parm: ", typeof(parm);
      print,"dimsof parm: ", dimsof(parm);

   } else {

      if( nterm != numberof(esti) ) {
         write,"Mismatch between nterm and numberof(esti)";
         return -1;
      }
      parm() = esti();
      if( chat > 2 ) print,"esti input: ", parm;
   }
   if( chat > 5 ) return parm;

   ndim = nterm;
   delta_parm = delta * parm;

   for(i=1; i<=ndim+1; i++ ) {
      for(j=1; j<=ndim; j++) {
         p(i,j) = parm(j);
         if( j+1 == i ) p(i,j) += delta_parm(j);
      }
   }
   for( i=1; i<=ndim+1; i++ ) {
      //for( j=1; j<=ndim; j++ ) parm(j) = p(i,j);
      parm() = p(i,);
      chi2_arr(i) = funk(parm);
      if( chat > 1 ) {
         if( nterm == 5 ) {
            print,"p c2 (init)= ", parm(1),parm(2),parm(3),parm(4),parm(5),chi2_arr(i);
         } else {
            print,"p c2 (init)= ", parm(1),parm(2),parm(3),parm(4),chi2_arr(i);
         }
      }
   }
   p_res = amoeba( funk, p, chi2_arr, ftol, iter);
   print,"iter = ",iter;
   if( chat > 1 ) {
      for( i=1; i<=ndim+1; i++ ) {
         if( nterm == 5 ) {
            print, p(i,1),p(i,2),p(i,3),p(i,4),p(i,5),chi2_arr(i);
         } else {
            print, p(i,1),p(i,2),p(i,3),p(i,4),chi2_arr(i);
         }
      }
      if( nterm == 5 ) {
         print,"Amplitude   Meanx  Meany     Sigma  Bckg    Chi2";
      } else {
         print,"Amplitude   Meanx  Meany     Sigma  Chi2";
      }
   }
   parm() = p(1,);

   return parm;

}





/* Function gaussfit_am */

func gaussfit_am(x, y, ftol=, nterm=, esti=, delta= )
/* DOCUMENT res = gaussfit_am( x, y, ftol=, nterm=, esti=, delta= )

     Perform a gauss fit for array 'y' with abscissae in 'x'
     It returns a double array with amplitude, mean, sigma, bkg.

     If nterm == 3 or not given:
     yfit = res(1)*exp(-0.5*((x-res(2))/res(3))^2)
     If nterm == 4:
     yfit = res(1)*exp(-0.5*((x-res(2))/res(3))^2) + res(4)

     KEYWORD   ftol: fitting tolerance default value: 1.e-4
              nterm: number of terms (3 or 4)
               esti: estimated start values of coefficients
              delta: fractional parameter adjustment [default: 0.1]


     SEE ALSO:  autoesti

     2005-04-12/NJW  Translated from C-program

*/
{
   require,"varfuncs.i";

   extern bds, yval;

   if( is_void(nterm) ) nterm = 3;
   if( nterm != 3 && nterm != 4 ) {
      print,"Keyword nterm must be 3 or 4";
      return -1;
   }
   if( is_void(delta) ) delta = 0.1;
   if( is_void(ftol) ) ftol = 1.e-4;

   p = array(double,nterm+1,nterm);
   parm = array(double,nterm);
   delta_parm = array(double,nterm);
   chi2_arr = array(double,nterm+1);

  /*
   *   define boundaries
   */
   b = 0.5*(x + shift(x,1));
   bds = 2*x(1) - b(1);
   grow,bds,b;
   bds(0) = 2*x(0) - b(-1);
   yval = y;

   n_points = numberof(y);


  /*
   *  parm(1) is constant,
   *  parm(2) is mean,
   *  parm(3) is sigma,
   *  parm(4) is background if nterm==4
   */

  /*
   *    prediction of values
   */

   if( is_void(esti) ) {
      sumn = sum(y);
      sumx = sum(x*y);
      sumx2 = sum(x*x*y);
      mean = sumx / sumn;
      stdev = (sumx2 - sumn*mean*mean)/sumn;
      parm(3) = sqrt(stdev);
      /* parm(1) = sumn * 0.398942280 / parm(3); */
      parm(1) = max(y);
      parm(2) = mean;
   } else {
      if( nterm != numberof(esti) ) {
         write,"Mismatch between nterm and numberof(esti)";
         return -1;
      }
      parm() = esti();
   }

   ndim = nterm;
   delta_parm(1) = delta * parm(1);
   delta_parm(2) = delta * parm(2);
   delta_parm(3) = delta * parm(3);
   if( nterm == 4 ) {
      parm(4) = 0.5*(y(1)+y(0));
      delta_parm(4) = delta * parm(1);
   }

   for(i=1; i<=ndim+1; i++ ) {
      for(j=1; j<=ndim; j++) {
         p(i,j) = parm(j);
         if( j+1 == i ) p(i,j) += delta_parm(j);
      }
   }
   for( i=1; i<=ndim+1; i++ ) {
      for( j=1; j<=ndim; j++ ) parm(j) = p(i,j);
      chi2_arr(i) = funka(parm);
      if( nterm == 4 ) {
         print,"p c2 = ", parm(1),parm(2),parm(3),parm(4),chi2_arr(i);
      } else {
         print,"p c2 = ", parm(1),parm(2),parm(3),chi2_arr(i);
      }
   }
   p_res = amoeba(funka, p, chi2_arr, ftol, iter);
   print,"iter = ",iter;
   for( i=1; i<=ndim+1; i++ ) {
      if( nterm == 4 ) {
         print, p(i,1),p(i,2),p(i,3),p(i,4),chi2_arr(i);
      } else {
         print, p(i,1),p(i,2),p(i,3),chi2_arr(i);
      }
   }
   if( nterm == 4 ) {
      print,"Amplitude   Mean     Sigma   Bckg    Chi2";
      parm(4) = p(1,4);
   } else {
      print,"Amplitude   Mean     Sigma    Chi2";
   }
   parm(1) = p(1,1);
   parm(2) = p(1,2);
   parm(3) = p(1,3);

   return parm;

}

/* Function funka */

func funka(parm)
/* The Chi-square function */
{

   extern bds, yval;

   sm = 0.0;
   for( i = 1; i <= n_points; i++ ) {
      sigma = 1.0;
      arg = yval(i) - modela(bds(i), bds(i+1), parm);
      if( sigma != 0.0 ) arg /= sigma;
      sm += arg*arg;
   }
   return sm;
}

/* Function modela */

func modela(x1, x2, parm)
{
   term1 = 2.506628275*parm(1)*parm(3)*(e0((x2-parm(2))/parm(3)) \
        -e0((x1-parm(2))/parm(3)))/(x2 - x1);

   if( numberof(parm) == 4 ) {
      return term1 + parm(4);
   } else {
      return term1;
   }
}


/* Function amoeba */

func amoeba(fitfun, p, y, ftol, &iter)
/* DOCUMENT p_res = amoeba( fitfun, p, y, ftol, >iter )

      Multidimensional minimization of the function 'fitfun(x)' where
      'x' is an ndim-dimensional vector, by the downhill Simplex method
      of Nelder and Mead. Input is a matrix 'p' whose ndim+1 rows are
      ndim-dimensional vectors which are vertices of the starting simplex.
      (Logical dimensions of 'p' are p(ndim+1,ndim); physical dimensions
      are input as p(mp,np)). Also input is the vector 'y' of length
      ndim+1, whose components must be preinitialized to the values of
      funk_amoeba evaluated at the ndim+1 vertices (rows) of 'p'; and ftol the
      fractional convergence tolerance to be achieved in the function value
      (NB!). On output, 'p' and 'y' will have been reset to ndim+1 new points
      all within ftol of a minimum function value, and iter gives the
      number of iterations taken.

      Translated from "Numerical Recipes"

      2008-03-14/NJW Updated 1) 'fitfun' is now an argument
                             2) returns best fit
*/

{

   sz = dimsof(p);
   ndim = sz(3);
   szy = dimsof(y);
   if( sz(2) != sz(3) + 1 || szy(2) != sz(2) ) {
      print,"Bad dimensions!";
      return [];
   }

   alpha = 1.0;
   beta  = 0.5;
   gamma = 2.0;
   itmax = 500;

   mpts = ndim + 1;
   pbar = array(double, ndim);
   pr = array(double, ndim);
   prr = array(double, ndim);
   iter = 0;

 while( 1 ) { /* replacement of FORTRAN goto statement */
   ilo = 1;
   if( y(1) > y(2) ) {
        ihi = 1;
        inhi = 2;
   }
   else {
        ihi = 2;
        inhi = 1;
   }
   for( i=1; i<= mpts; i++ ) {
        if( y(i) < y(ilo) ) ilo = i;
        if( y(i) > y(ihi) ) {
           inhi = ihi;
           ihi = i;
        }
        else if( y(i) > y(inhi) ) {
           if( i != ihi ) inhi = i;
        }
   }

 /* following 3 lines are updated with respect to original
          'Numerical Recipes' code    */

   div = abs( y(ihi)) + abs( y(ilo));
   if( div < ftol ) rtol = 2.*abs((y(ihi) - y(ilo)));
   else rtol = 2.0*abs((y(ihi) - y(ilo)))/div;

 /*  ------------------------------------  */

   if( rtol < ftol ) {
      write,format="%s","\n";
      return p(1,);
   }
   if( iter == itmax ) {
        write,format="\namoeba exceeding maximum number of iterations (%i).\n", itmax;
        return p(1,);
   }
   iter++;
   write,format="%s","*";
   if( iter%50 == 0 ) write,format="%s","\n";

   for( j=1; j<=ndim; j++) pbar(j) = 0.0; /* loop 12 */

   for( i=1; i<=mpts; i++) { /* loop 14 */
        if( i != ihi ) {
           for( j=1; j<=ndim; j++) pbar(j) += p(i,j);
        }
   }

   for( j=1; j<=ndim; j++) { /* loop 15 */
        pbar(j) /= ndim;
        pr(j) = (1. + alpha)*pbar(j) - alpha*p(ihi,j);
   }

   ypr = fitfun(pr);
   if( ypr <= y(ilo) ) {
        for( j=1; j<=ndim; j++) prr(j) = gamma*pr(j) + (1.-gamma)*pbar(j);
        yprr = fitfun(prr);
        if( yprr < y(ilo) ) {
           for( j=1; j<=ndim; j++) p(ihi,j) = prr(j);
           y(ihi) = yprr;
        }
        else {
           for( j=1; j<=ndim; j++ ) p(ihi,j) = pr(j);
           y(ihi) = ypr;
        }
   } else if( ypr >= y(inhi) ) {
        if( ypr < y(ihi) ) {
           for( j=1; j<=ndim; j++) p(ihi,j) = pr(j);
           y(ihi) = ypr;
        }
        for( j=1; j<=ndim; j++) prr(j) = beta*p(ihi,j) + (1.-beta)*pbar(j);
        yprr = fitfun(prr);
        if( yprr < y(ihi) ) {
           for( j=1; j<=ndim; j++) p(ihi,j) = prr(j);
           y(ihi) = yprr;
        }
        else {
           for( i=1; i<= mpts; i++) {
              if( i != ilo ) {
          for( j=1; j<=ndim; j++) {
             pr(j) = 0.5*(p(i,j) + p(ilo,j));
             p(i,j) = pr(j);
          }
          y(i) = fitfun(pr);
              }
           }
        }
   } else {
      for( j=1; j<=ndim; j++) p(ihi,j) = pr(j);
      y(ihi) = ypr;
   }
 } /* end of large dummy while loop */
}

/* Function ftex */

func ftex( x, p )
/* DOCUMENT res = ftex( x, p )

   Auxiliary function to 'gaussfitx' to return the model values.

   Expects 'x' to be bin boundaries and returns integrals
   of the gauss function over the individual bins divided
   with binsize i.e. an array with one less elements.

*/
{
   x1 = x(1:-1);
   x2 = x(2:0);
   sigma2 = p(3)*sqrt(2);
   xp = p(2);
   tb = erf((x2-xp)/sigma2);
   ta = erf((x1-xp)/sigma2);
   res = sqrt(pi/2)*p(3)*p(1)*(tb - ta)/(x2-x1);
   n = numberof(p);
   if( n > 3 ) res += p(4);
   if( n > 4 ) res += x(zcen)*p(5);
   if( n > 5 ) res += x(zcen)*x(zcen)*p(6);
   return res;
}

/* Function gaussfitx */

func gaussfitx( x, y, estimate, &yfit )
/* DOCUMENT coefs = gaussfitx( x, y, estimate, >yfit )

   The 'x' array are bin boundaries and must have one element
   more than 'y'.

   The number of elements in 'estimate' gives the number
   of terms in the fit.
   1: Amplitude
   2: Mean
   3: sigma
  [4: Constant]
  [5: Slope]
  [6: Curvature]

   If 'estimate' is not given then 'autoesti' is called.

     SEE ALSO:  autoesti

   2010-04-06/NJW Uses integrals over bins
*/
{

   if( is_void(estimate) ) {
      esti = autoesti( x, y );
   } else {
      esti = estimate;
   }
   r = lmfit( ftex, x, esti, y, 1. );

   yfit = ftex( x, esti );

   return esti;
}

/* Function fte1 */

func fte1( x, p )
/* DOCUMENT res = fte1( x, p )

   Auxiliary function to 'gaussfit' to return the model values.
*/
{
   res = p(1)*exp(-0.5*((x-p(2))/p(3))^2);
   n = numberof(p);
   if( n > 3 ) res += p(4);
   if( n > 4 ) res += x*p(5);
   if( n > 5 ) res += x*x*p(6);

   return res;
}

/* Function gaussfit */

func gaussfit( x, y, estimate, &yfit )
/* DOCUMENT coefs = gaussfit( x, y, estimate, >yfit )

   Uses 'lmfit' to perform a search for the best fit to the
   data (x,y) with a Gauss function on a 2. order background.

   The number of elements in 'estimate' gives the number
   of terms in the fit.
   1: Amplitude
   2: Mean
   3: sigma
  [4: Constant]
  [5: Slope]
  [6: Curvature]

   If 'estimate' is not given then 'autoesti' is called.

     SEE ALSO:  autoesti

   2007-05-03/NJW
*/
{

   if( is_void(estimate) ) {
      esti = autoesti( x, y );
   } else {
      esti = estimate;
   }
   r = lmfit( fte1, x, esti, y, 1. );

   yfit = array_gauss( x, esti );

   return esti;
}

/* Function _reduce_p */

func _reduce_p( p, freeze )
/* DOCUMENT newp = _reduce_p( p, freeze )

   Auxiliary function to amoeba_freeze
*/
{
   dms = dimsof( p );
   if( dms(1) != 2 ) error, "##1##";
   if( dms(2) != 1+dms(3) ) error, "##2##";

   dms -= [0,1,1];
   pm = array(double,dms);

   l = 1;
   for( k = 1; k <= dms(2)+1; k++ ) {
      if( k != freeze+1 ) {
         j = 1;
         for(i = 1; i<=dms(3)+1; i++ ) { // copy rows except frozen
            if( i != freeze ) pm(l,j++) = p(k,i);
         }
         l++;
      }
   }

   return pm;

}

/* Function fte2d */

func fte2d( im_tofit, p )
/* DOCUMENT model = fte2d( im_tofit, p )

   Auxiliary function for 'gaussfit_2d' to return the model image.

   'im_tofit' is only an argument for the dimensionality
   for the model.

   p = [amplitude, x-pos, y-pos, sigmax, sigmay] (5 elements)

*/
{
   tmpl = im_tofit*0.;
   return add_peak( tmpl, p(2), p(3), p(4), p(5), peak=p(1) );
}

/* Function gaussfit_2d */

func gaussfit_2d( im_tofit, estimate, &bestfit )
/* DOCUMENT coefs = gaussfit_2d( im_tofit, estimate, >bestfit )

   Fitting a 2D gaussian to an image ('im_tofit').
   'estimate' must hold 5 values for starting values:
      estimate = [amplitude, x-pos, y-pos, sigmax, sigmay]
   The fitted values are returned in 'coefs' and the best fit image
   in 'bestfit'.

*/
{
   esti = estimate;
   r = lmfit( fte2d, im_tofit, esti, im_tofit, 1. );

   bestfit = fte2d( im_tofit, esti );

   return esti;

}

/* Function prepare_gf2d */

func prepare_gf2d( im_tofit )
/* DOCUMENT estimate = prepare_gf2d( im_tofit )

   Return best guess for 2D gauss fitting around highest pixel value.

*/
{
   local maxval, xc, yc;

   dms = dimsof( im_tofit );
   maxim, im_tofit, maxval, xc, yc;
   res = array(double, 5);
   res(1) = maxval;

   // peak in x-direction

   y = im_tofit(,yc);
   x = 1.*indgen(dms(2));
   estimate = [maxval, xc, 1., 0., 0.];
   coefs = gaussfit( x, y, estimate, yfit );
   window,1;
   plot,x,y,ps=10,title="Fit in X";
   oplot, x, yfit, color="red";
   res(2) = coefs(2);
   res(4) = coefs(3);

   // peak in y-direction

   y = im_tofit(xc,);
   x = 1.*indgen(dms(3));
   estimate = [maxval, yc, 1., 0., 0.];
   coefs = gaussfit( x, y, estimate, yfit );
   window,2;
   plot,x,y,ps=10,title="Fit in Y";
   oplot, x, yfit, color="red";
   res(3) = coefs(2);
   res(5) = coefs(3);

   return res;

}


/* Function _reduce_y */

func _reduce_y( y, freeze )
/* DOCUMENT newy = _reduce_y( y, freeze )

   Auxiliary function to amoeba_freeze
*/
{
   dms = dimsof( y );
   if( dms(1) != 1 ) error, "##3##";

   dms -= [0,1];
   ym = array(double,dms);

   j = 1;
   for(i = 1; i<=dms(2)+1; i++ ) { // copy except frozen
      if( i != freeze+1 ) ym(j++) = y(i);
   }

   return ym;

}

/* Function amoeba_freeze */

func amoeba_freeze(fitfun, p, y, ftol, &iter, freeze= )
/* DOCUMENT p_res = amoeba_freeze( fitfun, p, y, ftol, >iter, freeze=  )

      Multidimensional minimization of the function 'fitfun(x)' where
      'x' is an ndim-dimensional vector, by the downhill Simplex method
      of Nelder and Mead. Input is a matrix 'p' whose ndim+1 rows are
      ndim-dimensional vectors which are vertices of the starting simplex.
      (Logical dimensions of 'p' are p(ndim+1,ndim); physical dimensions
      are input as p(mp,np)). Also input is the vector 'y' of length
      ndim+1, whose components must be preinitialized to the values of
      funk_amoeba evaluated at the ndim+1 vertices (rows) of 'p'; and ftol the
      fractional convergence tolerance to be achieved in the function value
      (NB!). On output, 'p' and 'y' will have been reset to ndim+1 new points
      all within ftol of a minimum function value, and iter gives the
      number of iterations taken.

      Translated from "Numerical Recipes"

      2008-03-14/NJW Updated 1) 'fitfun' is now an argument
                             2) returns best fit
*/

{

   if( freeze ) {
      frozen = p(1,freeze);
      p = _reduce_p( p, freeze);
      y = _reduce_y( y, freeze);
   }

   sz = dimsof(p);
   ndim = sz(3);
   szy = dimsof(y);
   if( sz(1) != 2 || sz(2) != sz(3) + 1 || szy(2) != sz(2) ) {
      print,"AMOEBA_F: sz = ", sz, " szy = ", szy;
      error,"AMOEBA_F: Bad dimensions!";
   }

   alpha = 1.0;
   beta  = 0.5;
   gamma = 2.0;
   itmax = 500;

   mpts = ndim + 1;
   pbar = array(double, ndim);
   pr = array(double, ndim);
   prr = array(double, ndim);
   iter = 0;

 while( 1 ) { /* replacement of FORTRAN goto statement */
   ilo = 1;
   if( y(1) > y(2) ) {
        ihi = 1;
        inhi = 2;
   }
   else {
        ihi = 2;
        inhi = 1;
   }
   for( i=1; i<= mpts; i++ ) {
        if( y(i) < y(ilo) ) ilo = i;
        if( y(i) > y(ihi) ) {
           inhi = ihi;
           ihi = i;
        }
        else if( y(i) > y(inhi) ) {
           if( i != ihi ) inhi = i;
        }
   }

 /* following 3 lines are updated with respect to original
          'Numerical Recipes' code    */

   div = abs( y(ihi)) + abs( y(ilo));
   if( div < ftol ) rtol = 2.*abs((y(ihi) - y(ilo)));
   else rtol = 2.0*abs((y(ihi) - y(ilo)))/div;

 /*  ------------------------------------  */

   if( rtol < ftol ) {
      write,format="%s","\n";
      res = p(1,);
      if( freeze ) res = insert( frozen, res, freeze );
      return res;
   }
   if( iter == itmax ) {
      write,format="\namoeba exceeding maximum number of iterations (%i).\n", itmax;
      res = p(1,);
      if( freeze ) res = insert( frozen, res, freeze );
      return res;
   }
   iter++;
   write,format="%s","*";
   if( iter%50 == 0 ) write,format="%s","\n";

   for( j=1; j<=ndim; j++) pbar(j) = 0.0; /* loop 12 */

   for( i=1; i<=mpts; i++) { /* loop 14 */
        if( i != ihi ) {
           for( j=1; j<=ndim; j++) pbar(j) += p(i,j);
        }
   }

   for( j=1; j<=ndim; j++) { /* loop 15 */
        pbar(j) /= ndim;
        pr(j) = (1. + alpha)*pbar(j) - alpha*p(ihi,j);
   }

   ypr = freeze ? fitfun(insert(frozen,pr,freeze)) : fitfun(pr);
   if( ypr <= y(ilo) ) {
        for( j=1; j<=ndim; j++) prr(j) = gamma*pr(j) + (1.-gamma)*pbar(j);
        yprr = freeze ? fitfun(insert(frozen,prr,freeze)) : fitfun(prr);
        if( yprr < y(ilo) ) {
           for( j=1; j<=ndim; j++) p(ihi,j) = prr(j);
           y(ihi) = yprr;
        }
        else {
           for( j=1; j<=ndim; j++ ) p(ihi,j) = pr(j);
           y(ihi) = ypr;
        }
   } else if( ypr >= y(inhi) ) {
        if( ypr < y(ihi) ) {
           for( j=1; j<=ndim; j++) p(ihi,j) = pr(j);
           y(ihi) = ypr;
        }
        for( j=1; j<=ndim; j++) prr(j) = beta*p(ihi,j) + (1.-beta)*pbar(j);
        yprr = freeze ? fitfun(insert(frozen,prr,freeze)) : fitfun(prr);
        if( yprr < y(ihi) ) {
           for( j=1; j<=ndim; j++) p(ihi,j) = prr(j);
           y(ihi) = yprr;
        }
        else {
           for( i=1; i<= mpts; i++) {
              if( i != ilo ) {
                 for( j=1; j<=ndim; j++) {
                    pr(j) = 0.5*(p(i,j) + p(ilo,j));
                    p(i,j) = pr(j);
                 }
                 y(i) = freeze ? fitfun(insert(frozen,pr,freeze)) : fitfun(pr);
              }
           }
        }
   } else {
      for( j=1; j<=ndim; j++) p(ihi,j) = pr(j);
      y(ihi) = ypr;
   }

 } /* end of large dummy while loop */

}

/* Function golden */

func golden( ax, bx, cx, f, tol, &xmin )
/* DOCUMENT f_val = golden( ax, bx, cx, function, tol, >xmin )

   Find the minimum of a function of a single variable 'f'.
   Returns the function value in the minimum and the
   x-value in the argument: 'ximin'.

   Input ax, bx, cx are bracketing x values (as from 'mnbrak')

   From Numerical Recipes, 2009-11-05/NJW
*/
{
    //c float f0,f1,f2,f3,x0,x1,x2,x3;

    R = 0.61803399;
    C = 1.0 - R;

    x0 = ax;
    x3 = cx;
    if( abs(cx-bx) > abs(bx-ax)) {
       x1 = bx;
       x2 = bx+C*(cx-bx);
    } else {
       x2 = bx;
       x1 = bx-C*(bx-ax);
    }
    f1 = f(x1);
    f2 = f(x2);

    // The stop criterion has been changed to
    //   max(1.,abs(x1)+abs(x2)) rather than
    //   abs(x1)+abs(x2), since a minimum near x=0
    //   leads to too many iterations, 2009-11-06/NJW

    while( abs(x3-x0) > tol*max(1.,abs(x1)+abs(x2))) {

       if( f2 < f1) {
          x0 = x1;
          x1 = x2;
          x2 = R*x1+C*x3;
          f1 = f2;
          f2 = f(x2);
       } else {
          x3 = x2;
          x2 = x1;
          x1 = R*x2+C*x0;
          f2 = f1;
          f1 = f(x1);
       }
    }
    if( f1 < f2) {
       xmin = x1;
       final = f1;
    } else {
       xmin = x2;
       final = f2;
    }
    return final;
}

/* Function mnbrak */

func mnbrak( &ax, &bx, &cx, &fa, &fb, &fc, f, glimit= )
/* DOCUMENT mnbrak, (>)ax, (>)bx, (>)cx, >fa, >fb, >fc, f, glimit=

   Seek three points ax, bx, cx, such that
      fb=f(bx) < fa=f(ax) && fb < fc=f(cx)
      and ax < bx < cx  or   cx < bx < ax

   Numerical Recipes p. 281, 2009-08-16/NJW
*/
{
   gold = 1.618034;
   if( is_void(glimit) ) glimit = 100.;
   tiny = 1.e-20;

   fa = f(ax);
   fb = f(bx);

   if( fb > fa ) { // switch roles of ax and bx
      dum = ax; ax = bx; bx = dum;
      dum = fb; fb = fa; fa = dum;
   }

   cx = bx + gold*(bx - ax); // first guess for cx
   fc = f(cx);

   // mark1:

   while( fb >= fc ) {
      r = (bx-ax)*(fb-fc);
      q = (bx-cx)*(fb-fa);
      divisor = q-r >= 0 ? 2.*max(abs(q-r),tiny) : -2.*max(abs(q-r),tiny);
      u = bx-((bx-cx)*q-(bx-ax)*r)/divisor;
      ulim = bx + glimit*(cx-bx);
      if( (bx-u)*(u-cx) > 0. ) { // parabolic u is between b and c
         fu = f(u);
         if( fu < fc ) {
            ax = bx;
            fa = fb;
            bx = u;
            fb = fu;
            continue; // goto mark1;
         } else if( fu > fb ) {
            cx = u;
            fc = fu;
            continue; // goto mark1;
         }
         u = cx + gold*(cx-bx);
         fu = f(u);
      } else if( (cx-u)*(u-ulim) > 0. ) {
         fu = f(u);
         if( fu < fc ) {
            bx = cx;
            cx = u;
            u = cx + gold*(cx-bx);
            fb = fc;
            fc = fu;
            fu = f(u);
         }
      } else if( (u-ulim)*(ulim-cx) >= 0. ) {
         u = ulim;
         fu = f(u);
      } else {
         u = cx + gold*(cx-bx);
         fu = f(u);
      }
      ax = bx;
      bx = cx;
      cx = u;
      fa = fb;
      fb = fc;
      fc = fu;
   }
}

/* Function mnbrak_lg */

func mnbrak_lg( &ax, &bx, &cx, &fa, &fb, &fc, f, glimit= )
/* DOCUMENT mnbrak_lg, (>)ax, (>)bx, (>)cx, >fa, >fb, >fc, f, glimit=

   Seek three points ax, bx, cx, such that
      fb=f(bx) < fa=f(ax) && fb < fc=f(cx)
      and ax < bx < cx  or   cx < bx < ax

   Numerical Recipes p. 281, 2009-08-16/NJW
   Adapted to use only positive values for x,  2012-05-09/NJW
*/
{
   gold = 1.618034;
   if( is_void(glimit) ) glimit = 100.;
   tiny = 1.e-20;

   fa = f(ax);
   fb = f(bx);

   if( fb > fa ) { // switch roles of ax and bx
      dum = ax; ax = bx; bx = dum;
      dum = fb; fb = fa; fa = dum;
   }

   cx = bx + gold*(bx - ax); // first guess for cx
   fc = f(cx);

   // mark1:

   while( fb >= fc ) {
      r = (bx-ax)*(fb-fc);
      q = (bx-cx)*(fb-fa);
      divisor = q-r >= 0 ? 2.*max(abs(q-r),tiny) : -2.*max(abs(q-r),tiny);
      u = bx-((bx-cx)*q-(bx-ax)*r)/divisor;
      ulim = bx + glimit*(cx-bx);
      if( (bx-u)*(u-cx) > 0. ) { // parabolic u is between b and c
         fu = f(u);
         if( fu < fc ) {
            ax = bx;
            fa = fb;
            bx = u;
            fb = fu;
            continue; // goto mark1;
         } else if( fu > fb ) {
            cx = u;
            fc = fu;
            continue; // goto mark1;
         }
         u = cx + gold*(cx-bx);
         fu = f(u);
      } else if( (cx-u)*(u-ulim) > 0. ) {
         fu = f(u);
         if( fu < fc ) {
            bx = cx;
            cx = u;
            u = cx + gold*(cx-bx);
            fb = fc;
            fc = fu;
            fu = f(u);
         }
      } else if( (u-ulim)*(ulim-cx) >= 0. ) {
         u = ulim;
         fu = f(u);
      } else {
         u = cx + gold*(cx-bx);
         fu = f(u);
      }
      ax = bx;
      bx = cx;
      cx = u;
      fa = fb;
      fb = fc;
      fc = fu;
   }
}

/* Function mnbrak_lg_simple */

func mnbrak_lg_simple( &ax, &bx, &cx, &fa, &fb, &fc, f )
/* DOCUMENT mnbrak_lg_simple, (>)ax, (>)bx, (>)cx, >fa, >fb, >fc, f

   Seek three points ax, bx, cx, such that
      fb=f(bx) < fa=f(ax) && fb < fc=f(cx)
      and ax < bx < cx  or   cx < bx < ax

   Adapted to use only positive values for x,  2012-05-09/NJW
*/
{

   exponen = 1.3;
   fa = f(ax);
   fb = f(bx);

   if( fb > fa ) { // switch roles of ax and bx
      dum = ax; ax = bx; bx = dum;
      dum = fb; fb = fa; fa = dum;
   }

   cx = bx * (bx/ax)^exponen; // first guess for cx
   fc = f(cx);

   while( fc < fb ) { // keep going in this direction
      u = cx * (cx/bx)^exponen;
      fu = f(u);
      zax = ax; // if a change of step size is necessary
      ax = bx;
      bx = cx;
      cx = u;
      zfa = fa;
      fa = fb;
      fb = fc;
      fc = fu;
   }
   if( fc > fb ) return;
   // fc == fb, go back and try with smaller step size
   cx = bx;
   bx = ax;
   ax = zax;
   fc = fb;
   fb = fa;
   fa = zfa;
   exponen = 1.;
   while( fc < fb ) {
      u = cx * (cx/bx)^exponen;
      fu = f(u);
      zax = ax; // if a change of step size is necessary
      ax = bx;
      bx = cx;
      cx = u;
      zfa = fa;
      fa = fb;
      fb = fc;
      fc = fu;
   }
   if( fc > fb ) return;

}

/* Function brent */

func brent( ax, bx, cx, f, tol, &xmin)
/* DOCUMENT fval = brent( ax, bx, cx, f, tol, >xmin)

   Finds the minimum of function 'f' (of one variable)
   given the bracketing values ax, bx, cx (as from 'mnbrak')
   'tol' is the tolerance where a value of 1.e-4 is often
   sensible. The best x-value is returned in 'xmin' and the
   'brent' returns the value of the function in that point.

   From Numerical Recipes, 2009-11-06/NJW
*/
{
   ITMAX = 200;
   CGOLD = 0.3819660;
   ZEPS = 1.0e-10;
   e = 0.0;

   a = ((ax < cx) ? ax : cx);
   b = ((ax > cx) ? ax : cx);
   x = w = v = bx;
   fw = fv = fx = f(x);
   for( iter = 1; iter <= ITMAX; iter++ ) {
      xm = 0.5*(a+b);
      //+ tol2 = 2.0*(tol1 = tol*abs(x)+ZEPS);
      //    abs(x) changed to max(1.,abs(x)) to avoid absurdly
      //    low tolerance when minimum is at x=0, 2009-11-06/NJW
      tol2 = 2.0*(tol1 = tol*max(1.,abs(x))+ZEPS);
      if( abs(x-xm) <=  (tol2-0.5*(b-a))) {
         xmin = x;
         return fx;
      }
      if( abs(e) > tol1) {
         r = (x-w)*(fx-fv);
         q = (x-v)*(fx-fw);
         p = (x-v)*q-(x-w)*r;
         q = 2.0*(q-r);
         if( q > 0.0) p = -p;
         q = abs(q);
         etemp = e;
         e = d;
         if( abs(p) >= abs(0.5*q*etemp) || p <= q*(a-x) || p >= q*(b-x))
            d = CGOLD*(e = (x >= xm ? a-x : b-x));
         else {
            d = p/q;
            u = x+d;
            if (u-a < tol2 || b-u < tol2)
               d = SiGn(tol1,xm-x);
         }
      } else {
         d = CGOLD*(e = (x >=  xm ? a-x : b-x));
      }
      u = (abs(d) >=  tol1 ? x+d : x+SiGn(tol1,d));
      fu = f(u);
      if( fu <=  fx) {
         if( u >=  x) a = x; else b = x;
         //+ ShFt(v,w,x,u)
         //+ ShFt(fv,fw,fx,fu)
         v = w;
         w = x;
         x = u;
         fv = fw;
         fw = fx;
         fx = fu;
      } else {
         if (u < x) a = u; else b = u;
         if (fu <= fw || w == x) {
            v = w;
            w = u;
            fv = fw;
            fw = fu;
         } else if (fu <= fv || v == x || v == w) {
            v = u;
            fv = fu;
         }
      }
   }
   error,"Too many iterations in BRENT";
   xmin = x;
   return fx;
}

/* Function powell */

func powell( &p, &xi, n, ftol, &iter, &fret, fnuc, itmax=, chat= )
/* DOCUMENT powell( (>)p, (>)xi, n, ftol, >iter, >fret, fnuc, itmax=, chat= )

   Minimization of a function 'fnuc' of 'n' variables.
   Input consists of an initial starting point 'p' that is
   a vector of length 'n'; an initial matrix 'xi' of dimensions 'n' by 'n',
   and whose columns contain the initial set of directions (usually
   the 'n' unit vectors); and 'ftol' the fractional tolerance in the
   function value such that failure to decrease by more than this amount
   on one iteration signals doneness. On output, 'p' is set to the
   best point found, 'xi' is the then-current direction set, 'fret' is
   the returned function value at 'p', and 'iter' is the number of
   iterations taken.
   Used: linmin

   From Numerical Recipes  2009-11-06/NJW
*/
{
   extern _powell_ftol;
   if( is_void(itmax) ) itmax = 300;

   _powell_ftol = ftol;

   //+ pt = array(double, n);
   //+ ptt = array(double, n);
   //+ xit = array(double, n);
   fret = fnuc(p);
   if(chat) {
      for( i = 1; i <= numberof(p); i++ ) {
         write,format="Powell p(%i) = %.5f, fret = %.6f\n", i, p(i), fret;
      }
   }

   pt = p;
   for( iter = 1;; iter++) {
      fp = fret;
      ibig = 0;
      del = 0.0;
      for( i = 1; i <= n; i++ ) {
         //+ for( j = 1; j <= n; j++ ) xit(j) = xi(j,i);
         xit = xi(,i);
         fptt = fret;
         linmin, p, xit, n, fret, fnuc;
         if( abs(fptt-fret) > del ) {
            del = abs(fptt-fret);
            ibig = i;
         }
      }
      //+ if( 2.0*abs(fp-fret) <=  ftol*(abs(fp)+abs(fret)))
      //  avoid absurdly low tolerance for near zero values
      //       2009-11-06/NJW
      if( 2.0*abs(fp-fret) <=  ftol*max(1.,abs(fp)+abs(fret))) {
         return;
      }
      if( iter == itmax ) error,"Too many iterations in routine POWELL";
      //+ for( j = 1; j <= n; j++ ) {
      //+    ptt(j) = 2.0*p(j)-pt(j);
      //+    xit(j) = p(j)-pt(j);
      //+    pt(j) = p(j);
      //+ }
      ptt = 2.0*p - pt;
      xit = p - pt;
      pt = p;
      fptt = fnuc(ptt);
      if( fptt < fp) {
         t = 2.0*(fp-2.0*fret+fptt)*(fp-fret-del)^2-del*(fp-fptt)^2;
         if( t < 0.0) {
            linmin, p, xit, n, fret, fnuc;
            //+ for( j = 1; j <= n; j++ ) xi(j,ibig) = xit(j);
            xi(,ibig) = xit;
         }
      }
   }
}

/* Function powell_lg */

func powell_lg( &p, &xi, n, ftol, &iter, &fret, fnuc, itmax=, chat= )
/* DOCUMENT powell_lg( (>)p, (>)xi, n, ftol, >iter, >fret, fnuc, itmax=, chat= )

   Minimization of a function 'fnuc' of 'n' variables.
   Input consists of an initial starting point 'p' that is
   a vector of length 'n'; an initial matrix 'xi' of dimensions 'n' by 'n',
   and whose columns contain the initial set of directions (usually
   the 'n' unit vectors); and 'ftol' the fractional tolerance in the
   function value such that failure to decrease by more than this amount
   on one iteration signals doneness. On output, 'p' is set to the
   best point found, 'xi' is the then-current direction set, 'fret' is
   the returned function value at 'p', and 'iter' is the number of
   iterations taken.
   Used: linmin_lg

   From Numerical Recipes  2009-11-06/NJW
*/
{
   extern _powell_ftol;
   if( is_void(itmax) ) itmax = 300;

   _powell_ftol = ftol;

   //+ pt = array(double, n);
   //+ ptt = array(double, n);
   //+ xit = array(double, n);
   fret = fnuc(p);
   if(chat) {
      for( i = 1; i <= numberof(p); i++ ) {
         write,format="Powell_lg p(%i) = %.5f, fret = %.6f\n", i, p(i), fret;
      }
   }

   pt = p;
   for( iter = 1;; iter++) {
      fp = fret;
      ibig = 0;
      del = 0.0;
      for( i = 1; i <= n; i++ ) {
         //+ for( j = 1; j <= n; j++ ) xit(j) = xi(j,i);
         xit = xi(,i);
         fptt = fret;
         linmin_lg, p, xit, n, fret, fnuc;
         if( abs(fptt-fret) > del ) {
            del = abs(fptt-fret);
            ibig = i;
         }
      }
      //+ if( 2.0*abs(fp-fret) <=  ftol*(abs(fp)+abs(fret)))
      //  avoid absurdly low tolerance for near zero values
      //       2009-11-06/NJW
      if( 2.0*abs(fp-fret) <=  ftol*max(1.,abs(fp)+abs(fret))) {
         return;
      }
      if( iter == itmax ) error,"Too many iterations in routine POWELL";
      //+ for( j = 1; j <= n; j++ ) {
      //+    ptt(j) = 2.0*p(j)-pt(j);
      //+    xit(j) = p(j)-pt(j);
      //+    pt(j) = p(j);
      //+ }
      ptt = 2.0*p - pt;
      xit = p - pt;
      pt = p;
      fptt = fnuc(ptt);
      if( fptt < fp) {
         t = 2.0*(fp-2.0*fret+fptt)*(fp-fret-del)^2-del*(fp-fptt)^2;
         if( t < 0.0) {
            linmin_lg, p, xit, n, fret, fnuc;
            //+ for( j = 1; j <= n; j++ ) xi(j,ibig) = xit(j);
            xi(,ibig) = xit;
         }
      }
   }
}

/* Function f1dim */

func f1dim( x ) // Auxiliary function for 'linmin'
{
   extern _linmin_n, _linmin_p, _linmin_xi;
     // _linmin_n is the number of variables
     // _linmin_p is an array of _linmin_n values
     // _linmin_xi is an array of _linmin_n values
   extern _powell_func; // for the function to minimize

   //+ xt = array(double, _linmin_n); // will be input to function
   //+ for( j = 1; j <= _linmin_n; j++ ) {
   //+    xt(j) = _linmin_p(j) + x*_linmin_xi(j);
   //+ }
   xt = _linmin_p + x * _linmin_xi;
   return _powell_func(xt);
}

/* Function linmin */
   
func linmin( &p, &xi, n, &fret, fnuc)
/* DOCUMENT linmin, (>)p, (>)xi, n, >fret, fnuc

    Auxiliary function for 'powell'

    Numerical Recipes,   2009-11-06/NJW
*/
{
   extern _linmin_n, _linmin_p, _linmin_xi;
   extern _powell_func, _powell_ftol;

   local ax, xx, bx, fa, fx, fb, xmin;

   _linmin_n = n;
   //+ _linmin_p  =  array(double, n);
   //+ _linmin_xi  =  array(double, n);

   _powell_func = fnuc; // called by 'f1dim'
   //+ for( j = 1; j <= n; j++ ) {
   //+    _linmin_p(j) = p(j);
   //+    _linmin_xi(j) = xi(j);
   //+ }
   _linmin_p = p;
   _linmin_xi = xi;
   ax = 0.0;
   xx = 1.0;
   bx = 2.0;
   mnbrak, ax, xx, bx, fa, fx, fb, f1dim;
   fret = brent( ax, xx, bx, f1dim, _powell_ftol, xmin);
   //+ for( j = 1; j <= n; j++ ) {
   //+    xi(j) *= xmin;
   //+    p(j) += xi(j);
   //+ }
   xi *= xmin;
   p += xi;
}

/* Function linmin_lg */
   
func linmin_lg( &p, &xi, n, &fret, fnuc)
/* DOCUMENT linmin_lg, (>)p, (>)xi, n, >fret, fnuc

    Auxiliary function for 'powell_lg'

    Numerical Recipes,   2009-11-06/NJW
*/
{
   extern _linmin_n, _linmin_p, _linmin_xi;
   extern _powell_func, _powell_ftol;

   local ax, xx, bx, fa, fx, fb, xmin;

   _linmin_n = n;
   //+ _linmin_p  =  array(double, n);
   //+ _linmin_xi  =  array(double, n);

   _powell_func = fnuc; // called by 'f1dim'
   //+ for( j = 1; j <= n; j++ ) {
   //+    _linmin_p(j) = p(j);
   //+    _linmin_xi(j) = xi(j);
   //+ }
   _linmin_p = p;
   _linmin_xi = xi;
   ax = 0.0;
   xx = 1.0;
   bx = 2.0;
   mnbrak_lg_simple, ax, xx, bx, fa, fx, fb, f1dim;
   fret = brent( ax, xx, bx, f1dim, _powell_ftol, xmin);
   //+ for( j = 1; j <= n; j++ ) {
   //+    xi(j) *= xmin;
   //+    p(j) += xi(j);
   //+ }
   xi *= xmin;
   p += xi;
}

/* Function expodecayfit */

func expodecayfit( t, y, dy, pl=, ini= )
/* DOCUMENT p = expodecayfit( t, y, dy, pl=, ini= )

    Fits  p(1) * exp( -p(2) * t ) to y
    using weight 1/dy^2
    (defined in function edf_modelf)

    Keyword 'pl' for plotting
            'ini' for initial parameters 'p'
*/
{

   // initial guess
   if( is_void(ini) ) {
      p = [10., 1.];
   } else {
      p = ini;
   }

   res = lmfit( edf_modelf, t, p, y, 1./dy^2 );
   write,format="Chi2 red = %12.6f\n", res.chi2_last/res.nfree;
   write,format="Constant: %10.4f\n", p(1);
   write,format="   Decay: %10.4f\n", p(2);
   if( pl ) {
      dataplot, t, y, dy, xbar=1;
      oplot,t,edf_modelf( t, p ),color="red";
   }
   return p;

}
func edf_modelf( t, p )
{
   return p(1) * exp( -p(2) * t );
}

/* Function lmfit */

/* 
 * lmfit.i --
 *
 *      Non-linear least-squares fit by Levenberg-Marquardt method.
 *
 * Copyright (c) 1997, Eric THIEBAUT (thiebaut@obs.univ-lyon1.fr, Centre de
 * Recherche Astrophysique de Lyon, 9 avenue Charles  Andre,  F-69561 Saint
 * Genis Laval Cedex).
 *
 * This program is free software; you can redistribute it and/or  modify it
 * under the terms of the GNU General Public License  as  published  by the
 * Free Software Foundation; either version 2 of the License,  or  (at your
 * option) any later version.
 *
 * This program is distributed in the hope  that  it  will  be  useful, but
 * WITHOUT  ANY   WARRANTY;   without   even   the   implied   warranty  of
 * MERCHANTABILITY or  FITNESS  FOR  A  PARTICULAR  PURPOSE.   See  the GNU
 * General Public License for more details (to receive a  copy  of  the GNU
 * General Public License, write to the Free Software Foundation, Inc., 675
 * Mass Ave, Cambridge, MA 02139, USA).
 *
 * History:
 *      $Id: lmfit.i,v 1.2 1997/07/28 08:26:25 eric Exp $
 *      $Log: lmfit.i,v $
 *      Revision 1.2  1997/07/28 08:26:25  eric
 *       - Fix the doc.
 *
 *      Revision 1.1  1997/04/21 08:34:04  eric
 *      Initial revision
 *-----------------------------------------------------------------------------
 */

require, "random.i";

struct lmfit_result {
/* DOCUMENT lmfit_result -- structure returned by lmfit
 */
    long        neval;
    long        niter;
    long        nfit;
    long        nfree;
    long        monte_carlo;
    double      chi2_first;
    double      chi2_last;
    double      conv;
    double      sigma;
    double      lambda;
    pointer     stdev;
    pointer     stdev_monte_carlo;
    pointer     correl;
}

func lmfit(f, x, &a, y, w, fit=, correl=, stdev=, gain=, tol=, deriv=, itmax=,
        lambda=, eps=, monte_carlo=)
/* DOCUMENT lmfit -- Non-linear least-squares fit by Levenberg-Marquardt
                     method.

   DESCRIPTION:
     Implement Levenberg-Marquardt method  to  perform  a  non-linear least
     squares fit to a function of an arbitrary number of  parameters.   The
     function  may  be  any  non-linear  function.   If  available, partial
     derivatives can be calculated by the user function, else  this routine
     will  estimate  partial   derivatives   with   a   forward  difference
     approximation.

   CATEGORY:
     E2 - Curve and Surface Fitting.

   SYNTAX:
     result= lmfit(f, x, a, y, w, ...);

   INPUTS:
     F:  The model function  to  fit.   The  function  must  be  written as
         described under RESTRICTIONS, below.
     X:  Anything useful for the model function, for  instance: independent
         variables, a complex structure of  data  or  even  nothing!.   The
         LMFIT routine does not manipulate or use values  in  X,  it simply
         passes X to the user-written function F.
     A:  A vector that contains the initial estimate for each parameter.
     Y:  Array of dependent variables (i.e., the  data).   Y  can  have any
         geometry, but it must be the same as the result returned by F.
     W:  Optional weight,  must be conformable  with Y and all  values of W
         must be positive  or null (default = 1.0).   Data points with zero
         weight are not fitted.  Here are some examples:
           - For no weighting (lest square fit): W = 1.0
           - For instrumental weighting: W(i) = 1.0/Y(i)
           - Gaussian noise: W(i) = 1.0/Var(Y(i))

   OUTPUTS:
     A:  The vector of fitted parameters.
     Returns a structure lmfit_result with fields:
       NEVAL:  (long) number of model function evaluations.
       NITER:  (long) number of iteration, i.e. successful CHI2 reductions.
       NFIT:   (long) number of fitted parameters.
       NFREE:  (long) number of degrees of freedom  (i.e.,  number of valid
               data points minus number of fitted parameters).
       MONTE_CARLO: (long) number of Monte Carlo simulations.
       CHI2_FIRST: (double) starting error value: CHI2=sum(W*(F(X,A)-Y)^2).
       CHI2_LAST: (double) last best error value: CHI2=sum(W*(F(X,A)-Y)^2).
       CONV:   (double) relative variation of CHI2.
       SIGMA:  (double) estimated uniform standard deviation of data.  If a
               weight is provided, a  value  of  SIGMA  different  from one
               indicates  that,  if  the  model  is  correct,  W  should be
               multiplied    by     1/SIGMA^2.      Computed     so    that
               sum(W*(F(X,A)-Y)^2)/SIGMA^2=NFREE.
       LAMBDA: (double) last value of LAMBDA.
       STDEV:  (pointer) standard deviation vector of the parameters.
       STDEV_MONTE_CARLO: (pointer)  standard   deviation  vector  of  the
               parameters estimated by Monte Carlo simulations.
       CORREL: (pointer) correlation matrice of the parameters.

   KEYWORDS:
     FIT: List  of  indices  of  parameters  to  fit,  the  others  remaing
          constant.  The default is to tune all parameters.
     CORREL: If  set  to a  non  zero and  non-nil  value, the  correlation
          matrice of the parameters is stored into LMFIT result.
     STDEV: If set to a non zero and non-nil value, the standard deviation
          vector of the parameters is stored into LMFIT result.
     DERIV: When set to a non zero and non-nil  value,  indicates  that the
          model function F is able to compute its derivatives  with respect
          to  the  parameters   (see   RESTRICTIONS).    By   default,  the
          derivatives will be estimated by LMFIT using  forward difference.
          If analytical derivatives are  available  they  should  always be
          used.
     EPS: Small positive value  used  to  estimate  derivatives  by forward
          difference.  Must be such that 1.0+EPS  and  1.0  are numerically
          different  and   should   be   about  sqrt(machine_precision)/100
          (default = 1e-6).
     TOL: Stop criteria for the convergence (default =  1e-7).   Should not
          be smaller  than  sqrt(machine_precision).   The  routine returns
          when  the  relative  decrease of  CHI2 is  less  than  TOL  in an
          interation.
     ITMAX: Maximum number of iterations. Default = 100.
     GAIN: Gain factor for tuning LAMBDA (default = 10.0).
     LAMBDA: Starting value for parameter LAMBDA (default = 1.0e-3).
     MONTE_CARLO: Number of Monte Carlo  simulations to perform to estimate
          standard  deviation  of  parameters  (by default  no Monte  Carlo
          simulations are undergone).  May spend a lot of time if you use a
          large number; but should not be too small!

   GLOBAL VARIABLES:
     None.

   SIDE EFFECTS:
     The values of the vector of parameters A are modified.

   PROCEDURE:
     The function to be fitted must be defined as follow:

       func F(x, a) {....}

     and returns a model with same shape as data Y.  If you want to provide
     analytic derivatives, F should be defined as:

       func F(x, a, &grad, deriv=)
       {
           y= ...;
           if (deriv) {
               grad= ...;
           }
           return y;
       }

     Where X are the independent variables (anything the function  needs to
     compute synthetic data except the model parameters), A  are  the model
     parameters, DERIV is  a  flag  set  to  non-nil  and  non-zero  if the
     gradient is needed and the output gradient GRAD  is  a  numberof(Y) by
     numberof(A) array: GRAD(i,j) = derivative of ith data point model with
     respect to jth parameter.

     LMFIT tune  parameters A so as to minimize:  CHI2=sum(W*(F(X,A)-Y)^2).
     The  Levenberg-Marquardt  method   consists  in  varying  between  the
     inverse-Hessian  method  and the  steepest  descent  method  where the
     quadratic  expansion of  CHI2  does  not  yield  a better  model.  The
     initial  guess of  the parameter  values  should  be as  close to  the
     actual values as possible or the solution may not converge or may give
     a wrong answer.

   RESTRICTIONS:
     Beware that  the result  does depend on your  initial guess A.  In the
     case of  numerous  local  minima,  the only  way to  get  the  correct
     solution is to start with A close enough to this solution.
     
     The estimates of  standard  deviation of the  parameters are  rescaled
     assuming that, for a correct model  and weights, the expected value of
     CHI2 should  be of the  order of NFREE=numberof(Y)-numberof(A)  (LMFIT
     actually compute NFREE from the number of valid data points and number
     of fitted parameters).   If you don't like this you'll have to rescale
     the returned  standard  deviation  to meet  your needs  (all necessary
     information are in the structure returned by LMFIT).

   EXAMPLE:
     This example is from ODRPACK (version 2.01).  The function to fit is
     of the form:
       f(x) = a1+a2*(exp(a3*x)-1.0)^2
     Starting guess:
       a= [1500.0,  -50.0,   -0.1];
     Independent variables:
       x= [   0.0,    0.0,    5.0,    7.0,    7.5,   10.0,
             16.0,   26.0,   30.0,   34.0,   34.5,  100.0];
     Data:
       y= [1265.0, 1263.6, 1258.0, 1254.0, 1253.0, 1249.8,
           1237.0, 1218.0, 1220.6, 1213.8, 1215.5, 1212.0];
     Function definition (without any optimization):
       func foo(x, a, &grad, deriv=)
       {
           if (deriv)
               grad= [array(1.0, dimsof(y)),
                      (exp(a(3)*x)-1.0)^2,
                      2.0*a(2)*x*exp(a(3)*x)*(exp(a(3)*x)-1.0)];
           return a(1)+a(2)*(exp(a(3)*x)-1.0)^2;
       }
       
     Fitting this model by:
       r= lmfit(foo, x, a, y, 1., deriv=1, stdev=1, monte_carlo=500, correl=1)
     produces typically the following result:
        a                   = [1264.84, -54.9987, -0.0829835]
        r.neval             = 12
        r.niter             = 6
        r.nfit              = 3
        r.nfree             = 9
        r.monte_carlo       = 500
        r.chi2_first        = 40.4383
        r.chi2_last         = 40.4383
        r.conv              = 3.84967e-09
        r.sigma             = 0.471764
        r.lambda            = 1e-09
       *r.stdev             = [1.23727, 1.78309, 0.00575123]
       *r.stdev_monte_carlo = [1.20222, 1.76120, 0.00494790]
       *r.correl            = [[ 1.000, -0.418, -0.574],
                               [-0.418,  1.000, -0.340],
                               [-0.574, -0.340,  1.000]]

   HISTORY:
     - Basic ideas borrowed from "Numerical Recipes in C", CURVEFIT.PRO (an
       IDL version by DMS, RSI, of the routine "CURFIT: least squares fit to
       a non-linear function", Bevington, Data Reduction and Error Analysis
       for the Physical Sciences) and ODRPACK ("Software for Weigthed
       Orthogonal Distance Regression" freely available at: www.netlib.org).
     - Added: fitting of a subset of the parameters, Monte-Carlo
       simulations...
 */
{
    local grad;

    /* Maybe subset of parameters to fit. */
    na= numberof(a);
    if (is_void(fit))
        fit= indgen(na);
    else if (dimsof(fit)(1) == 0)
        fit= [fit];
    nfit= numberof(fit);
    if (!nfit)
        error, "no parameters to fit.";
    
    /* Check weights. */
    if (is_void(w)) w= 1.0;
    else if (anyof(w < 0.0))
        error, "bad weights.";
    if (numberof(w) != numberof(y))
        w += array(0.0, dimsof(y));
    nfree= sum(w != 0.0) - nfit;        // Degrees of freedom
    if (nfree <= 0)
        error, "not enough data points.";

    /* Other settings. */
    diag= indgen(1:nfit^2:nfit+1);      // Subscripts of diagonal elements
    if (is_void(lambda)) lambda= 1e-3;
    if (is_void(gain)) gain= 10.0;
    if (is_void(itmax)) itmax= 100;
    if (is_void(eps)) eps= 1e-6;        // sqrt(machine_precision)/100
    if (1.0+eps <= 1.0)
        error, "bad value for EPS.";
    if (is_void(tol)) tol= 1e-7;
    monte_carlo= is_void(monte_carlo) ? 0 : long(monte_carlo);
    warn_zero= 0;
    warn= "*** Warning: LMFIT ";
    neval= 0;
    conv= 0.0;
    niter= 0;
    
    while (1) {
        if (deriv) {
            m= f(x, a, grad, deriv=1);
            neval++;
            grad= nfit == na ? grad(*,) : grad(*,fit);
        } else {
            if (!niter) {
                m= f(x, a);
                neval++;
            }
            inc= eps * abs(a(fit));
            if (numberof((i= where(inc <= 0.0)))) inc(i)= eps;
            grad= array(double, numberof(y), nfit);
            for (i=1; i<=nfit; i++) {
                anew= a;        // Copy current parameters
                anew(fit(i)) += inc(i);
                grad(,i)= (f(x,anew)-m)(*)/inc(i);
            }
            neval += nfit;
        }
        beta= w * (chi2= y-m);
        if (niter) chi2= chi2new;
        else       chi2= chi2_first= sum(beta * chi2);
        beta= grad(+,) * beta(*)(+);
        alpha= ((w(*)(,-) * grad)(+,) * grad(+,));
        gamma= sqrt(alpha(diag));
        if (anyof(gamma <= 0.0)) {
            /* Some derivatives are null (certainly because of rounding
             * errors). */
            if (!warn_zero) {
                write, warn+"founds zero derivatives.";
                warn_zero= 1;
            }
            gamma(where(gamma <= 0.0))= eps * max(gamma);
            /* lines insertd by NJW 2007-02-22 to avoid floating point
               interrupt */
            if( allof(gamma <= 0.0 ) ) {
                write,warn+"all gamma values <= 0";
                goto done;
             }
        }
        gamma= 1.0 / gamma;
        beta *= gamma;
        alpha *= gamma(,-) * gamma(-,);
        
        while (1) {
            alpha(diag)= 1.0 + lambda;
            anew= a;
            anew(fit) += gamma * LUsolve(alpha, beta);
            m= f(x, anew);
            neval++;
            d= y-m;
            chi2new= sum(w*d*d);
            if (chi2new < chi2)
                break;
            lambda *= gain;
            if (allof(anew == a)) {
                /* No change in parameters. */
                write, warn+"makes no progress.";
                goto done;
            }
        }
        a= anew;
        lambda /= gain;
        niter++;
        conv= 2.0*(chi2-chi2new)/(chi2+chi2new);
        if (conv <= tol)
            break;
        if (niter >= itmax) {
            write, format=warn+"reached maximum number of iterations (%d).\n",
                itmax;
            break;
        }
    }
    
done:
    sigma= sqrt(nfree/chi2);
    result= lmfit_result(neval=neval, niter=niter, nfree=nfree, nfit=nfit,
            lambda=lambda, chi2_first=chi2_first, chi2_last=chi2, conv=conv,
            sigma=sigma);
    if (correl || stdev) {
        /* Compute correlation matrice and/or standard deviation vector. */
        alpha(diag)= 1.0;
        alpha= LUsolve(alpha);
        if (anyof((tmp1= alpha(diag)) < 0.0))
            write, format=warn+"%s\n", "found negative variance(s)";
        tmp1= sqrt(abs(tmp1));
        if (stdev) {
            /* Standard deviation is rescaled assuming that statistically
             * chi2 = nfree +/- sqrt(2*nfree). */
            (tmp2= array(double,na))(fit)= gamma * tmp1 / sigma;
            result.stdev= &tmp2;
        }
        if (correl) {
            gamma= 1.0 / tmp1;
            alpha *= gamma(-,) * gamma(,-);
            if (nfit == na) {
                result.correl= &alpha;
            } else {
                (tmp2= array(double, na, na))(fit,fit)= alpha;
                result.correl= &tmp2;
            }
        }
    }
    alpha= beta= gamma= [];     // Free some memory.
    if (monte_carlo >= 1) {
        saa= 0.0*a;
        sig= (w > 0.0) /(sqrt(max(nfree/chi2*w, 0.0)) + (w == 0.0));
        for (i=1; i<=monte_carlo; i++) {
            anew= a;
            ynew= y + sig * random_n(dimsof(y));
            lmfit, f, x, anew, ynew, w, fit=fit, gain=gain, tol=tol,
                deriv=deriv, itmax=itmax, lambda=lambda, eps=eps;
            anew -= a;
            saa += anew * anew;
        }
        result.monte_carlo= monte_carlo;
        result.stdev_monte_carlo= &sqrt(saa / monte_carlo);
    }
    return result;
}

/* Function autoesti */

func autoesti( x, y )
/* DOCUMENT esti = autoesti( x, y )

    Returns estimation of [amplitude, mean, sigma] when the (x,y) values
    represent a peak on a roughly zero background.

    SEE ALSO: gaussfit
*/
{
   // assumes peaked distribution with background approx. zero

   sumy = sum(y);
   if( sumy <= 0.0 ) error,"AUTOESTI: problem with sum(y) being zero or negative";
   meaan = sum(x*y)/sumy;
   return [max(y), meaan, sqrt(sum((x-meaan)^2*y)/sumy)]
}
