extern eulerdoc;
/* DOCUMENT euler package ****************************
 *
 *   Various coordinate transformation tools
 *   between spherical systems; in particular
 *   Equatorial and Galactic.
 * 
 *   2004-11-26/NJW
 *
         aitoff
         crossprod
         equatorial
         euler
         galactic
         jconv_coord
         lonlat
         rever_aitoff
         vector
 *************************************************/

/* Function euler */

func euler( a, b, c )
/* DOCUMENT matrix = euler( a, b, c )

   Returns a matrix that defines the transformation
   through the thre Euler angles a, b, and c expressed
   in radians.

   2004-11-25/NJW

*/

{
   co = cos(a);
   si = sin(a);
   ta = [[co,-si,0],[si,co,0],[0,0,1]];
   co = cos(b);
   si = sin(b);
   tb = [[co,0,-si],[0,1,0],[si,0,co]];
   co = cos(c);
   si = sin(c);
   tc = [[co,-si,0],[si,co,0],[0,0,1]];
   return tc(,+)*(tb(,+)*ta(+,))(+,);
}

/* Function crossprod */

func crossprod( a, b )
/* DOCUMENT c = crossprod( a, b )

   Returns the cross product of 2 3-element arrays
*/

{
   return [a(2)*b(3) - a(3)*b(2),
           a(3)*b(1) - a(1)*b(3),
           a(1)*b(2) - a(2)*b(1)];
}

/* Function equatorial */

func equatorial( gl, gb )
/* DOCUMENT radec = equatorial( gl, gb )

   Get equatorial coordinates from galactic when expressed in degrees
   Value 'n' in radec(n,).
   All Right Ascensions are in radec(,1) and all declinations in radec(,2)

   SEE ALSO: galactic

*/

{
   require, "string.i";

   dr = pi / 180;
   //+ ez = [cos(192.8604*dr)*cos(27.1278*dr), \
   //+       sin(192.8604*dr)*cos(27.1278*dr), \
   //+                        sin(27.1278*dr)];
   //+ ez = [-0.86766645981482, -0.19809110265025,0.45597678621091];
   //+ ex = [cos(266.4051*dr)*cos(-28.9362*dr), \
   //+       sin(266.4051*dr)*cos(-28.9362*dr), \
   //+                        sin(-28.9362*dr)];
   //+ ex = [-0.05487394293840,-0.87343697152730,-0.48383541329218];
   //+ ra_NGP = 192.8604; // old value
   //+ ra_NGP = 192.859508 * dr; // J2000

   ra_NGP = 192.859588 * dr; // J2000
   //+ dec_NGP = 27.128336 * dr; // J2000
   dec_NGP = 27.12825285 * dr; // J2000
   ez = [cos(ra_NGP)*cos(dec_NGP), \
         sin(ra_NGP)*cos(dec_NGP), sin(dec_NGP)];
   //+ ez = [-0.86766645981482, -0.19809110265025,0.45597678621091];
   ra_CGAL = 266.4051 * dr; // J2000
   dec_CGAL = -28.936175 * dr; // J2000
   ex = [cos(ra_CGAL)*cos(dec_CGAL), \
         sin(ra_CGAL)*cos(dec_CGAL), sin(dec_CGAL)];

   ey = crossprod( ez, ex );
   //+ ey = [0.49411047375510,-0.44482900433204,0.74698192509303];
   toeq = [ex, ey, ez];
   c = cos(gb*dr);
   sgal = transpose([cos(gl*dr)*c,sin(gl*dr)*c,sin(gb*dr)]);
   seq = toeq(,+)*sgal(+,);
   //+ seq2 = seq^2;
   //+ norms = sqrt(seq2(sum,));
   //+ seq /= norms(-,); // normalize for numerical round-off
   // normalize to suppress numerical round-off errors
   seq /= sqrt((seq^2)(sum,))(-,);
   alf = atan(seq(2,),seq(1,))/dr;
   if( is_scalar(alf) ) {
      if( alf < 0 ) alf += 360;
   } else {
      w = where( alf < 0 );
      nw = numberof(w);
      if( nw > 0 ) alf(w) += 360;
   }
   dec = asin(seq(3,)) / dr;
   return [alf,dec];
}


/* Function galactic */

func galactic( ra, dec )
/* DOCUMENT glb = galactic( ra, dec )

   Get galactic coordinates from equatorial when expressed in degrees
   Value 'n' in glb(n,).
   All longitudes are in glb(,1) and all latitudes in glb(,2)

   SEE ALSO: equatorial

*/

{
   dr = pi / 180;
   //+ ra_NGP = 192.8604; // old value
   //+ ra_NGP = 192.859508 * dr; // J2000
   ra_NGP = 192.859588 * dr; // J2000
   //+ dec_NGP = 27.128336 * dr; // J2000
   dec_NGP = 27.12825285 * dr; // J2000
   ez = [cos(ra_NGP)*cos(dec_NGP), \
         sin(ra_NGP)*cos(dec_NGP), sin(dec_NGP)];
   //+ ez = [-0.86766645981482, -0.19809110265025,0.45597678621091];
   ra_CGAL = 266.4051 * dr; // J2000
   dec_CGAL = -28.936175 * dr; // J2000
   ex = [cos(ra_CGAL)*cos(dec_CGAL), \
         sin(ra_CGAL)*cos(dec_CGAL), sin(dec_CGAL)];
   //+ ex = [-0.05487394293840,-0.87343697152730,-0.48383541329218];
   ey = crossprod( ez, ex );
   //+ ey = [0.49411047375510,-0.44482900433204,0.74698192509303];
   togal = transpose([ex, ey, ez]);
   c = cos(dec*dr);
   seq = transpose([cos(ra*dr)*c,sin(ra*dr)*c,sin(dec*dr)]);
   sgal = togal(,+)*seq(+,);
   //+ sgal2 = sgal^2;
   //+ norms = sqrt(sgal2(sum,));
   // normalize to suppress numerical round-off errors
   sgal /= sqrt((sgal^2)(sum,))(-,);
   //+ sgal /= norms(-,); // normalize for numerical round-off
   gl = atan(sgal(2,),sgal(1,))/dr;
   gb = asin(sgal(3,)) / dr;
   return [gl,gb];
}

/* Function vector */

func vector( lon, lat, deg= )
/* DOCUMENT unit_vector = vector( lon, lat, deg= )

   Returns a unit vector when longitude and latitude
   (or Right Ascension and Declination)
   are given in radians. Keyword 'deg' changes unit to degrees.

   If only one argument is supplied then it must be an array
   of dimension N x 2 so that lon(,1) are the longitudes and
   lon(,2) are the latitudes.

   SEE ALSO: lonlat

*/

{
   if( is_void( lat ) ) {
      dms = dimsof(lon);
      if( dms(1) != 2 ) error,"Bad dimensionality";
      if( dms(3) != 2 ) error,"Bad dimensionality or 2. dim";
      lat = lon(,2);
      lon = lon(,1);
   }
   dr = deg ? pi / 180 : 1.0;
   c = cos(lat * dr);
   return [cos(lon*dr)*c, sin(lon*dr)*c, sin(lat*dr)];
}

/* Function lonlat */

func lonlat( unit_vec, deg= )
/* DOCUMENT lonlat = lonlat( unit_vec, deg= )

   Returns a two_element array with longitude and latitude
   (or Right Ascension and Declination)
   in radians when unit vector is given. Keyword 'deg'
   changes unit to degrees.

   'unit_vec' can be an array of dimension N x 3; then the
   returned value will be an array of dimension N x 2

   SEE ALSO: vector
*/

{
   dms = dimsof( unit_vec );
   dr = deg ? pi / 180 : 1.0;
   if( dms(1) == 1 ) {
      unit_vec /= sqrt(sum(unit_vec^2));
      res = array(double,2);
      res(2) = asin(unit_vec(3));
      res(1) = atan( unit_vec(2), unit_vec(1) );
   } else if( dms(1) == 2 ) {
      N = dms(2);
      for(i=1;i<=N;i++) {
         unit_vec(i,) /= sqrt(sum(unit_vec(i,)^2));
      }
      res = array(double,N,2);
      res(,2) = asin(unit_vec(,3));
      res(,1) = atan( unit_vec(,2), unit_vec(,1) );
   } else error,"Illegal dimension";

   return res/dr;

}

/* Function aitoff */

func aitoff(l,b)
/* DOCUMENT res = aitoff(l,b)

 NAME:
       AITOFF
 PURPOSE:
       Convert longitude, latitude to X,Y using an AITOFF projection.
 EXPLANATION:
       This procedure can be used to create an all-sky map in Galactic 
       coordinates with an equal-area Aitoff projection.  Output map 
       coordinates are zero longitude centered.

 INPUTS:
       L - longitude - scalar or vector, in degrees
       B - latitude - same number of elements as L, in degrees

 OUTPUTS:
       X - X coordinate, same number of elements as L.   X is normalized to
               be between -180 and 180
       Y - Y coordinate, same number of elements as L.  Y is normalized to
               be between -90 and 90.

 SEE ALSO: rever_aitoff

 NOTES:
       See AIPS memo No. 46, page 4, for details of the algorithm.  This
       version of AITOFF assumes the projection is centered at b=0 degrees.

 REVISION HISTORY:
       Written  W.B. Landsman  STX          December 1989
       Modified for Unix:
               J. Bloch        LANL SST-9      5/16/91 1.1
       Converted to IDL V5.0   W. Landsman   September 1997
       Converted to Yorick     Niels J. Westergaard 2007

*/
{
   degrad = pi / 180.0;
   sa = l;
   n = numberof(sa);
   if( n == 1 ) sa = array(sa,1);
   x180 = where (sa > 180.0);
   if( numberof(x180) > 0) sa(x180) -= 360.;
   alpha2 = degrad * sa/2;
   delta = b * degrad;
   cdec = cos(delta);
   denom = sqrt(1. + cdec*cos(alpha2));
   res = array(0.0,2,n);
   res(1,) = 180. * cdec*sin(alpha2)/denom;
   res(2,) = 90. * sin(delta)/denom;

   return res;
}

/* Function rever_aitoff */

func rever_aitoff(x_in,y_in)
/* DOCUMENT res = rever_aitoff( x, y)


  Reverse Aitoff projection, assume degrees
    -180 <  x  < 180
     -90 <  y  <  90

   ALSO SEE: aitoff

   2004-11-03/NJW

*/

{

   radeg = 180.0 / pi;
   x = x_in;
   n = numberof(x);
   if( n == 1 ) x = array(x,1);
   w = where( x > 180. );
   nw = numberof(w);
   if( nw > 0 ) x(w) -= 360;
   w = where( x < -180. );
   nw = numberof(w);
   if( nw > 0 ) x(w) += 360;
   x = x / 180.0;
   y = y_in / 90.0;
   
   r2 = x^2 + y^2;
   
   if( max(r2) >= 1.0 ) {
      //+ print,"REVER_AITOFF error, x,y outside valid region";
      return [];
   }
   
   d = 2.0 - r2;
   sqd = sqrt(d);
   del = asin(y*sqd);
   alp = asin(x*sqd/cos(del));
   
   res = array(0.0,2,n);
   res(1,) = 2 * alp * radeg;
   res(2,) = del * radeg;
   
   return res;
}

/* Function precess */

func precess( &ra, &dec, equinox1, equinox2, radian= )
/* DOCUMENT
;+
; NAME:
;      PRECESS
; PURPOSE:
;      Precess coordinates from EQUINOX1 to EQUINOX2.
; EXPLANATION:
;      For interactive display, one can use the procedure ASTRO which calls
;      PRECESS or use the /PRINT keyword.   The default (RA,DEC) system is
;      FK5 based on epoch J2000.0 but FK4 based on B1950.0 is available via
;      the /FK4 keyword.
;
;      Use BPRECESS and JPRECESS to convert between FK4 and FK5 systems
; CALLING SEQUENCE:
;      PRECESS, ra, dec, [ equinox1, equinox2, /PRINT, /FK4, /RADIAN ]
;
; INPUT - OUTPUT:
;      RA - Input right ascension (scalar or vector) in DEGREES, unless the
;              /RADIAN keyword is set
;      DEC - Input declination in DEGREES (scalar or vector), unless the
;              /RADIAN keyword is set
;
;      The input RA and DEC are modified by PRECESS to give the
;      values after precession.
;
; OPTIONAL INPUTS:
;      EQUINOX1 - Original equinox of coordinates, numeric scalar.  If
;               omitted, PRECESS will query for EQUINOX1 and EQUINOX2.
;      EQUINOX2 - Equinox of precessed coordinates.
;
; OPTIONAL INPUT KEYWORDS:
;      /PRINT - If this keyword is set and non-zero, ) the precessed
;               coordinates are displayed at the terminal.    Cannot be used
;               with the /RADIAN keyword
;      /FK4   - If this keyword is set and non-zero, the FK4 (B1950.0) system
;               will be used otherwise FK5 (J2000.0) will be used instead.
;      /RADIAN - If this keyword is set and non-zero, ) the input and
;               output RA and DEC vectors are in radians rather than degrees
;
; RESTRICTIONS:
;       Accuracy of precession decreases for( declination values near 90
;       degrees.  PRECESS should not be used more than 2.5 centuries from
;       2000 on the FK5 system (1950.0 on the FK4 system).
;
; EXAMPLES:
;       (1) The Pole Star has J2000.0 coordinates (2h, 31m, 46.3s,
;               89d 15" 50.6"); compute its coordinates at J1985.0
;
;       IDL> precess, ten(2,31,46.3)*15, ten(89,15,50.6), 2000, 1985, /PRINT
;
;               ====> 2h 16m 22.73s, 89d 11" 47.3"
;
;       (2) Precess the B1950 coordinates of Eps Ind (RA = 21h 59m,33.053s,
;       DEC = (-56d, 59", 33.053") to equinox B1975.
;
;       IDL> ra = ten(21, 59, 33.053)*15
;       IDL> dec = ten(-56, 59, 33.053)
;       IDL> precess, ra, dec ,1950, 1975, /fk4
;
; PROCEDURE:
;       Algorithm from Computational Spherical Astronomy by Taff (1983),
;       p. 24. (FK4). FK5 constants from "Astronomical Almanac Explanatory
;       Supplement 1992, page 104 Table 3.211.1.
;
; PROCEDURE CALLED:
;       Function PREMAT - computes precession matrix
;
; REVISION HISTORY
;       Written, Wayne Landsman, STI Corporation  August 1986
;       Correct negative output RA values   February 1989
;       Added /PRINT keyword      W. Landsman   November, 1991
;       Provided FK5 (J2000.0)  I. Freedman   January 1994
;       Precession Matrix computation now in PREMAT   W. Landsman June 1994
;       Added /RADIAN keyword                         W. Landsman June 1997
;       Converted to IDL V5.0   W. Landsman   September 1997
;       Correct negative output RA values when /RADIAN used    March 1999
;-
*/
{

   deg_to_rad = pi/180.0;

   npts = numberof(ra);
   if( npts == 0 ) error,"PRECESS error, bad argument";

   if( radian ) {
      ra_rad= double(ra);
      dec_rad = double(dec);
   } else {
      ra_rad = ra*deg_to_rad;     // Convert to radians
      dec_rad = dec*deg_to_rad;
   }

   a = cos( dec_rad );

   if( npts == 1 ) {

      x = [a*cos(ra_rad), a*sin(ra_rad), sin(dec_rad)]; //input direction

   } else {

      x = array(double,npts,3);
      x(,1) = a*cos(ra_rad);
      x(,2) = a*sin(ra_rad);
      x(,3) = sin(dec_rad);
      x = transpose(x);

   }


   sec_to_rad = deg_to_rad/3600.0;

// Use PREMAT function to get precession matrix from Equinox1 to Equinox2

   r = premat(equinox1, equinox2);

   x2 = r(,+)*x(+,);      // rotate to get output direction cosines

   if( npts == 1 ) {                 // Scalar

      ra_rad = atan(x2(2),x2(1));
      dec_rad = asin(x2(3));

   } else {                // Vector

      ra_rad = atan(x2(2,),x2(1,));
      dec_rad = asin(x2(3,));

   }

   ra_rad = zero2pi(ra_rad);
   if( radian ) {
      ra = ra_rad;
      dec = dec_rad;
   } else {
      ra = ra_rad/deg_to_rad;
      dec = dec_rad/deg_to_rad;
   }

}

/* Function premat */

func premat( equinox1, equinox2 )
/* DOCUMENT
;+
; NAME:
;       PREMAT
; PURPOSE:
;       Return the precession matrix needed to go from EQUINOX1 to EQUINOX2.
; EXPLANTION:
;       This matrix is used by the procedures PRECESS and BARYVEL to precess
;       astronomical coordinates
;
; CALLING SEQUENCE:
;       matrix = PREMAT( equinox1, equinox2 )
;
; INPUTS:
;       EQUINOX1 - Original equinox of coordinates, numeric scalar.
;       EQUINOX2 - Equinox of precessed coordinates.
;
; OUTPUT:
;      matrix - double precision 3 x 3 precession matrix, used to precess
;               equatorial rectangular coordinates
;
; OPTIONAL INPUT KEYWORDS:
    FK4 option has been removed in this version; works with FK5 (NJW/2009-08-07)

;       /FK4   - If this keyword is set, the FK4 (B1950.0) system precession
;               angles are used to compute the precession matrix.   The
;               default is to use FK5 (J2000.0) precession angles
;
; EXAMPLES:
;       Return the precession matrix from 1950.0 to 1975.0 in the FK4 system
;
;       IDL> matrix = PREMAT( 1950.0, 1975.0, /FK4)
;
; PROCEDURE:
;       FK4 constants from "Computational Spherical Astronomy" by Taff (1983),
;       p. 24. (FK4). FK5 constants from "Astronomical Almanac Explanatory
;       Supplement 1992, page 104 Table 3.211.1.
;
; REVISION HISTORY
;       Written, Wayne Landsman, HSTX Corporation, June 1994
;       Converted to IDL V5.0   W. Landsman   September 1997
;-
*/
{

   deg_to_rad = pi/180.0;
   sec_to_rad = deg_to_rad/3600.0;

   t = 0.001*( equinox2 - equinox1);

   st = 0.001*( equinox1 - 2000.);

//  Compute 3 rotation angles
   a = sec_to_rad * t * (23062.181 + st*(139.656 +0.0139*st) \
            + t*(30.188 - 0.344*st+17.998*t));

   b = sec_to_rad * t * t * (79.280 + 0.410*st + 0.205*t) + a;

   c = sec_to_rad * t * (20043.109 - st*(85.33 + 0.217*st) \
              + t*(-42.665 - 0.217*st -41.833*t));


   sina = sin(a);  sinb = sin(b) ; sinc = sin(c);
   cosa = cos(a);  cosb = cos(b) ; cosc = cos(c);

   r = array(double,3,3);
   r(,1) = [ cosa*cosb*cosc-sina*sinb, sina*cosb+cosa*sinb*cosc,  cosa*sinc];
   r(,2) = [-cosa*sinb-sina*cosb*cosc, cosa*cosb-sina*sinb*cosc, -sina*sinc];
   r(,3) = [-cosb*sinc, -sinb*sinc, cosc];

   return r;
}

/* Function jconv_coord */

func jconv_coord( ra_scx, dec_scx, posang, &ra_src, \
                  &dec_src, &cosy_src, &cosz_src, \
                  to_eq=, to_sc=, chat= )
/* DOCUMENT omega = jconv_coord( ra_scx, dec_scx, posang, >ra_src,
                         >dec_src, >cosy_src, >cosz_src,
                         to_eq=, to_sc=, chat= )

 Procedure to perform coordinate transformations between equatorial
 coordinates and _SPACECRAFT_ coordinates.

 Returns the applied transformation matrix.

 Pointing information ra_scx, dec_scx, and posang must be given
 Keyword  to_eq  will return ra_src and dec_src when cosy_src and cosz_src
                                                  are given
 Keyword  to_sc  will return cosy_src and cosz_src when
                                ra_src and dec_src are given

   000726/NJW (IDL version)
   2009-01-28/NJW translated to Yorick from IDL

 At input: all angles are given in degrees
           cosy_src and cosz_src have no unit

*/
{

   // Check presence and number of keywords

   control = 0;
   if( to_eq ) {
      control = 1;
   }
   if( to_sc ) {
      control++;
      to_eq = 0;
   }
   if( control  !=  1 ) {
      error,"JCONV_COORD error, zero or two \"to_\" keywords";
   }


   dr = pi / 180; // degrees to radians

   ra_scx_rad = ra_scx * dr;
   dec_scx_rad = dec_scx * dr;
   posang_rad = posang * dr;

   if( ! to_eq ) {
      if( !is_void(ra_src) && !is_void(dec_src) ) {
         ra_src_rad = ra_src * dr;
         dec_src_rad = dec_src * dr;
      }
   }

   // Pointing direction: x-axis

   // RA && Dec of two other axes: y and z

   ra_scy_rad = ra_scx_rad - atan(-cos(posang_rad), -sin(dec_scx_rad)*sin(posang_rad));
   if( ra_scy_rad  <  0. ) ra_scy_rad = ra_scy_rad + 2*pi;

   dec_scy_rad = asin(cos(dec_scx_rad)*sin(posang_rad));

   ra_scz_rad = ra_scx_rad - atan(sin(posang_rad), -sin(dec_scx_rad)*cos(posang_rad));
   if( ra_scz_rad  <  0. ) ra_scz_rad = ra_scz_rad + 2*pi;
   dec_scz_rad = -asin(-cos(dec_scx_rad)*cos(posang_rad));

   if( chat ) {
      write,format="RA,DEC of SC Y-axis: %7.3 %7.3\n", ra_scy_rad/dr, dec_scy_rad/dr;
      write,format="RA,DEC of SC Z-axis: %7.3 %7.3\n", ra_scz_rad/dr, dec_scz_rad/dr;
   }

   // Pointing in EQ system

   x_eq = [cos(ra_scx_rad)*cos(dec_scx_rad), \
           sin(ra_scx_rad)*cos(dec_scx_rad), \
           sin(dec_scx_rad)];

   // SC Y-axis direction in EQ system

   y_eq = [-sin(ra_scx_rad)*cos(posang_rad) \
                 - sin(dec_scx_rad)*cos(ra_scx_rad)*sin(posang_rad),\
            cos(ra_scx_rad)*cos(posang_rad) \
                 - sin(dec_scx_rad)*sin(ra_scx_rad)*sin(posang_rad),\
            cos(dec_scx_rad)*sin(posang_rad)];

   // SC Z-axis direction in EQ system

   z_eq = [sin(ra_scx_rad)*sin(posang_rad) \
                 - sin(dec_scx_rad)*cos(ra_scx_rad)*cos(posang_rad),\
           -cos(ra_scx_rad)*sin(posang_rad) \
                 - sin(dec_scx_rad)*sin(ra_scx_rad)*cos(posang_rad),\
            cos(dec_scx_rad)*cos(posang_rad)];



   if( to_eq ) {

      // Define transformation matrix

      omega = array(double,3,3);
      omega(*,1) = x_eq;
      omega(*,2) = y_eq;
      omega(*,3) = z_eq;

      if( !is_void(cosy_src) && !is_void(cosz_src) ) {
         s_sc = array(double,3);
         s_sc(2) =   cosy_src;
         s_sc(3) =   cosz_src;
         s_sc(1) = sqrt(1. - s_sc(2)^2 - s_sc(3)^2);

         s_eq = omega(,+) * s_sc(+);

         dec_src_rad = asin(s_eq(3));
         ra_src_rad = zero2pi(atan(s_eq(2),s_eq(1)));

         dec_src = dec_src_rad / dr;
         ra_src = ra_src_rad / dr;
      }

      return omega;

   } else {

      // Define transformation matrix

      omega = array(double,3,3);
      omega(1,*) = x_eq;
      omega(2,*) = y_eq;
      omega(3,*) = z_eq;

      if( !is_void(ra_src) && !is_void(dec_src) ) {
         s_eq = array(double,3);
         s_eq(1) = cos(ra_src_rad) * cos(dec_src_rad);
         s_eq(2) = sin(ra_src_rad) * cos(dec_src_rad);
         s_eq(3) = sin(dec_src_rad);

         s_sc = omega(,+) * s_eq(+);

         cosy_src =  s_sc(2);
         cosz_src =  s_sc(3);
      }

      return omega;

   }

}

