extern geometrydoc;
/* DOCUMENT A package for various geometry tasks

    2008-07-31/NJW

   A polygon is an array of points (struct s_Point) and it is
   assumed to be closed i.e. first point connecting to last point.
   To be useful for hatching three or more points are
   demanded as well as no 'crossing' i.e. no sides may
   be intersecting.

   Types: point, line, delline, semidelline, circle

   Functions:
       box2plg                  draw_polygon             
       circle2plg               hatch                    
       cur_circle               init_canvas              
       cur_delline              inter_c_c                
       cur_line                 inter_dl_dl              
       cur_point                inter_dl_l               
       cur_polygon              inter_l_c                
       dcder                    inter_l_l                
       delline_p_p              inter_l_plg              
       dist_p_l                 line_p_angle             
       dist_p_p                 line_p_p                 
       draw_circle              line_perp_p_p            
       draw_delline             mirror_p_l               
       draw_line                verify_plg               
       draw_point               inside_plg
       ginter_l_l               inter_sdl_l
       inter_sdl_dl             inter_sdl_sdl
       vec_p_p                  get_vec_angle
       vec_r_angle              draw_garrow
   Structs:
       s_Circle
       s_DelLine
       s_Line
       s_Point
       s_Arc

*/

Inf_px = 300.; Inf_mx = -300.;
Inf_py = 300.; Inf_my = -300.;
Inf_r = 300.;

//+ window, 1, style="boxed.gs";
//+ window, 0, style="axes.gs";

// Straight line: ax + by = c

struct s_Line{
   string type;
   double a;
   double b;
   double c;
}

// Point (x,y) and Vector

struct s_Point{
   string type;
   double x;
   double y;
}

// Delimited straight line: ax + by = c
//  between p1 and p2
// and Semi-delimited line, then p1 is the real delimiter
// and p2 is the indicator of which side.

struct s_DelLine{
   string type;
   double a;
   double b;
   double c;
   s_Point p1;
   s_Point p2;
}

// Circle (center,radius)

struct s_Circle{
   string type;
   s_Point center;
   double radius;
}

// Arc (center, radius, angle1, angle2)

struct s_Arc{
   string type;
   s_Point center;
   double radius;
   double angle1;
   double angle2;
}

// Initialize plotting

func init_canvas( dimen, axes=, pane= )
/* DOCUMENT init_canvas, dimension, axes=, pane=
    opens a plotting window limited by +-dimension
    in both x and y.

    Axes will be shown when keyword 'axes' is set.
    Keyword 'pane' defines the window.
*/
{
   if( is_void(dimen) ) {
      write,"Syntax: init_canvas, dimension[, axes=][, pane=]";
      return;
   }
   if( is_void(pane) ) pane = 0;
   if( axes ) {
      window,pane;
   } else window,pane,style="nobox.gs";

   plot,dimen*[-0.5,0.5],dimen*2*[1.,1.],xr=dimen*[-1.,1.],yr=dimen*[-1.,1.];
}

// Enter a point by the cursor

func cur_point(void, lbl= )
/* DOCUMENT p = cur_point( lbl= )
    Defines a point (struct s_Point) by cursor input
    on the current canvas. If keyword 'lbl' is given,
    'draw_point' will be called with the label.
*/
{
   p = s_Point();
   p.type = "point";
   c = curmark1(ps=12,symsize=0.3);
   p.x = c(1);
   p.y = c(2);

   if( numberof(lbl) ) draw_point, p, lbl=lbl;

   return p;
}

// Enter a circle by the cursor

func cur_circle(void)
/* DOCUMENT c = cur_circle()
    Defines a circle (struct s_Circle) by cursor input
    for center and a point on the periphery on the current canvas.
*/
{
   write,"Define the center of the circle ...";
   c = cur_point();
   write,"Select any point on the periphery: ...";
   r = cur_point();
   res = s_Circle();
   res.type = "circle";
   res.center = c;
   res.radius = dist_p_p( c, r );
   draw_circle, res;

   return res;
}

// Enter an arc by the cursor

func cur_arc(void)
/* DOCUMENT a = cur_arc()
    Defines an arc (struct s_Arc) by cursor input
    for center, first point on circle (defines radius and 'angle1')
    and terminating angle.
*/
{
   write,"Define the center of the circle ...";
   c = cur_point();
   write,"Select the first point on the periphery: ...";
   p1 = cur_point();
   a = s_Arc();
   a.type = "arc";
   a.center = c;
   a.radius = dist_p_p( c, p1 );
   v1 = vec_p_p( c, p1 );
   a.angle1 = get_vec_angle( v1 );
   write,"Define the second angle: ...";
   p2 = cur_point();
   v2 = vec_p_p( c, p2 );
   a.angle2 = get_vec_angle( v2 );

   draw_arc, a;

   return a;
}

// Define a vector by start point and end point

func vec_p_p( start_point, end_point )
/* DOCUMENT v = vec_p_p( start_point, end_point )
     defines a vector (same struct as point)
*/
{
   v = s_Point();
   v.type = "vector";
   v.x = end_point.x - start_point.x;
   v.y = end_point.y - start_point.y;
   return v;
}

// Define a vector by radius and angle (in degrees)

func vec_r_angle( radius, angle )
/* DOCUMENT v = vec_r_angle( radius, angle )
     defines a vector (same struct as point)
*/
{
   v = s_Point();
   v.type = "vector";
   v.x = radius * cos(angle*pi/180.);
   v.y = radius * sin(angle*pi/180.);
   return v;
}

// Get angle (in degrees) of vector

func get_vec_angle( vector )
/* DOCUMENT angle = get_vec_angle( vector )
     Angle in degrees
*/
{
   if( vector.type != "vector" ) error,"Illegal argument";
   return atan(vector.y, vector.x)*180./pi;
}

// Enter a line by the cursor

func cur_line(void)
/* DOCUMENT l = cur_line()
    Defines a line (struct s_Line) by cursor input
    for two points on the line on the current canvas.
*/
{
   write,"Define the first point of the line ...";
   p1 = cur_point();
   write,"Define the second point of the line ...";
   p2 = cur_point();
   return line_p_p( p1, p2 );
}

// Enter a delimited line by the cursor

func cur_delline(void)
/* DOCUMENT l = cur_delline()
    Defines a delimited line (struct s_DelLine) by cursor input
    for the two endpoints of the line on the current canvas.
*/
{
   write,"Define the first end point of the line ...";
   p1 = cur_point();
   write,"Define the second end point of the line ...";
   p2 = cur_point();
   line = line_p_p( p1, p2 );
   res = s_DelLine();
   res.type = "delline";
   res.a = line.a;
   res.b = line.b;
   res.c = line.c;
   res.p1 = p1;
   res.p2 = p2;
   return res;
}

// Enter a semi-delimited line by the cursor

func cur_semidelline(void)
/* DOCUMENT l = cur_semidelline()
    Defines a semi-delimited line (struct s_DelLine) by cursor input
    for an endpoint and a guide point of the line on the current canvas.
*/
{
   write,"Define the end point of the line ...";
   p1 = cur_point();
   write,"Define the guide point of the line ...";
   p2 = cur_point();
   line = line_p_p( p1, p2 );
   res = s_DelLine();
   res.type = "semidelline";
   res.a = line.a;
   res.b = line.b;
   res.c = line.c;
   res.p1 = p1;
   res.p2 = p2;
   return res;
}

// Enter a polygon by the cursor

func cur_polygon( void, ps=, symsize= )
/* DOCUMENT plyg = cur_polygon( ps=, symsize= )
    Defines a polygon (struct s_Polygon) by cursor input
    for each corner of the polygon on the current canvas.
    Keywords: ps - plotsymbol, symsize - symbol size
*/
{
   write,"Enter polygon points: ... ";
   if( is_void(ps) ) ps = 2;
   if( is_void(symsize) ) symsize = 1.;
   res = curmark( ps=ps, symsize=symsize );
   nres = numberof(res);
   pol = array(s_Point, nres/2);
   pol(*).type = "point";
   pol(*).x = res(1::2);
   pol(*).y = res(2::2);
   return pol;
}

func verify_plg( polygon )
/* DOCUMENT result = verify_plg( polygon )
    Returns 1 if there are no crossing sides of the polygon
    else 0 (zero).
*/
{
   n = numberof( polygon );
   if( n < 3 ) return 0; // must have 3 or more sides
   // test all sides against all other sides
   // a side is from point i to point i+1;
   // if i+1 > n then 1 is to be used
   for( i = 1; i <= n; i++ ) {
      i2 = i == n ? 1 : i+1;
      dlinei = delline_p_p(polygon(i), polygon(i2));
      for( j = i+1; j <= n; j++ ) {
         j2 = j == n ? 1 : j+1;
         dlinej = delline_p_p(polygon(j), polygon(j2));
         q = inter_dl_dl( dlinei, dlinej );
         if( !is_void(q) ) {
            // if 'q' is (almost) identical to an apex then it
            // is rejected as an intersection
            if( min(abs(polygon.x-q.x)) + min(abs(polygon.y-q.y)) > 1.e-9 ) \
                 return 0; // side crossing occurred
         }
      }
   }

   // when you get here all sides have been tested

   return 1;
}

// Distance from point to point

func dist_p_p( point1, point2 )
{
   return sqrt((point1.x-point2.x)^2 + (point1.y-point2.y)^2);
}

// Distance from line to point

func dist_p_l( point, line )
{
   a = line.a;
   b = line.b;
   c = line.c;
   m = sqrt(a*a+b*b);

   return (point.x*a + point.y*b - c)/m;
}

// Mirror point in line

func mirror_p_l( point, line )
{
   d = dist_p_l( point, line );
   m = sqrt(line.a^2 + line.b^2);
   mp = s_Point();
   mp.type = "point";
   mp.x = point.x - 2*d*line.a/m;
   mp.y = point.y - 2*d*line.b/m;

   return mp;
}

// Get point from x,y values

func xy2point( x, y )
/* DOCUMENT p = xy2point( x, y)

*/
{
   p = s_Point()
   p.type = "point";
   p.x = double(x);
   p.y = double(y);
   return p;
}

// Line through two points

func line_p_p( point1, point2 )
{
   l = s_Line();
   l.type = "line";
   l.a = point1.y - point2.y;
   l.b = point2.x - point1.x;
   l.c = point1.x*l.a + point1.y*l.b;

   return l;
}

// Line through a point at an angle

func line_p_angle( point, angle )
/* DOCUMENT l = line_p_angle( point, angle )

     returns an unlimited line.
     'angle' in degrees.
*/
{
    // angle in degrees
    point2 = s_Point();
    point2.type = "point";
    point2.x = point.x + cos(angle*pi/180.);
    point2.y = point.y + sin(angle*pi/180.);
    return line_p_p( point, point2 );
}

// Delimited line between two points

func delline_p_p( point1, point2 )
{
   dl = s_DelLine();
   dl.type = "delline";
   dl.a = point1.y - point2.y;
   dl.b = point2.x - point1.x;
   dl.c = point1.x*dl.a + point1.y*dl.b;
   dl.p1 = point1;
   dl.p2 = point2;

   return dl;
}

// Semi-delimited line between two points
//  First point is the end point, second is guide point

func semidelline_p_p( point1, point2 )
{
   dl = s_DelLine();
   dl.type = "semidelline";
   dl.a = point1.y - point2.y;
   dl.b = point2.x - point1.x;
   dl.c = point1.x*dl.a + point1.y*dl.b;
   dl.p1 = point1;
   dl.p2 = point2;

   return dl;
}

// Line perpendicular and midway between two points

func line_perp_p_p( point1, point2 )
{
   x1 = point1.x; y1 = point1.y;
   x2 = point2.x; y2 = point2.y;
   x0 = (x1 + x2)/2;
   y0 = (y1 + y2)/2;
   l = s_Line();
   l.a = x1 - x2;
   l.b = y1 - y2;
   l.c = (x1*x1-x2*x2+y1*y1-y2*y2)/2;

   return l;
}

// Generic intersection of two lines
// meaning that the line types are read from the struct

func ginter_l_l( line1, line2 )
{
   if( line1.type == "line" ) {
      if( line2.type == "line" ) {
         return inter_l_l( line1, line2 );
      } else if( line2.type == "delline" ) {
         return inter_dl_l( line2, line1 );
      } else if( line2.type == "semidelline" ) {
         return inter_sdl_l( line2, line1 );
      } else {
        error,"Illegal line type for line2";
      }
   } else if( line1.type == "delline" ) {
      if( line2.type == "line" ) {
         return inter_dl_l( line1, line2 );
      } else if( line2.type == "delline" ) {
         return inter_dl_dl( line1, line2 );
      } else if( line2.type == "semidelline" ) {
         return inter_sdl_dl( line2, line1 );
      } else {
        error,"Illegal line type for line2";
      }
   } else if( line1.type == "semidelline" ) {
      if( line2.type == "line" ) {
         return inter_sdl_l( line1, line2 );
      } else if( line2.type == "delline" ) {
         return inter_sdl_dl( line1, line2 );
      } else if( line2.type == "semidelline" ) {
   
      } else {
        error,"Illegal line type for line2";
      }
   } else {
     error,"Illegal line type for line1";
   }
}

// Intersection of two lines

func inter_l_l( line1, line2 )
{
   D = line1.a*line2.b - line1.b*line2.a;
   if( abs(D) < 1.e-10 ) return [];
   p = s_Point();
   p.type = "point";
   p.x = (line1.c*line2.b - line1.b*line2.c)/D;
   p.y = (line1.a*line2.c - line1.c*line2.a)/D;
   return p;
}

// Intersection of two delimited lines

func inter_dl_dl( delline1, delline2 )
{
   // distances from line1 to p1 and p2 of line2 must have
   // different signs and vice versa
   d1_1 = dist_p_l( delline1.p1, delline2 );
   d1_2 = dist_p_l( delline1.p2, delline2 );
   if( d1_1*d1_2 > 0.0 ) return []; // they have same sign
   d2_1 = dist_p_l( delline2.p1, delline1 );
   d2_2 = dist_p_l( delline2.p2, delline1 );
   if( d2_1*d2_2 > 0.0 ) return []; // they have same sign
   q = inter_l_l( delline1, delline2 );
   if( is_void(q) ) error,"inter_dl_dl error: surprise! should not happen";
   return q;
}

// Intersection of delimited line and (unlimited) line

func inter_dl_l( delline1, line2 )
{
   // distances from line2 to p1 and p2 of delline1 must have
   // different signs
   d1_1 = dist_p_l( delline1.p1, line2 );
   d1_2 = dist_p_l( delline1.p2, line2 );
   if( d1_1*d1_2 > 0.0 ) return []; // they have same sign
   q = inter_l_l( delline1, line2 );
   if( is_void(q) ) error,"inter_dl_l error: surprise! should not happen";
   return q;
}

// Intersection of line and semi-delimited line

func inter_sdl_l( semidelline1, line2 )
{
   // Find intersection as if both were unlimited lines
   // and then check for guide point
   p = inter_l_l( semidelline1, line2 );
   vsdl = [(semidelline1.p2).x - (semidelline1.p1).x, (semidelline1.p2).y - (semidelline1.p1).y];
   vcross = [p.x - (semidelline1.p1).x, p.y - (semidelline1.p1).y];
   return sum(vsdl*vcross) > 0.0 ? p : [];
}

// Intersection of a semi-delimited line and a delimited line

func inter_sdl_dl( semidelline1, delline2 )
{
   // distances from line1 to p1 and p2 of line2 must have
   // different signs
   d2_1 = dist_p_l( delline2.p1, semidelline1 );
   d2_2 = dist_p_l( delline2.p2, semidelline1 );
   if( d2_1*d2_2 > 0.0 ) return []; // they have same sign
   p = inter_l_l( semidelline1, delline2 );
   vsdl = [(semidelline1.p2).x - (semidelline1.p1).x, (semidelline1.p2).y - (semidelline1.p1).y];
   vcross = [p.x - (semidelline1.p1).x, p.y - (semidelline1.p1).y];
   return sum(vsdl*vcross) > 0.0 ? p : [];
}

// Intersection between two semi-delimited lines

func inter_sdl_sdl( semidelline1, semidelline2 )
{
   // Find intersection as if both were unlimited lines
   // and then check for both guide points
   p = inter_l_l( semidelline1, semidelline2 );
   vsdl = [(semidelline1.p2).x - (semidelline1.p1).x, (semidelline1.p2).y - (semidelline1.p1).y];
   vcross = [p.x - (semidelline1.p1).x, p.y - (semidelline1.p1).y];
   if( sum(vsdl*vcross) < 0.0 ) return [];
   vsdl = [(semidelline2.p2).x - (semidelline2.p1).x, (semidelline2.p2).y - (semidelline2.p1).y];
   vcross = [p.x - (semidelline1.p2).x, p.y - (semidelline2.p1).y];
   if( sum(vsdl*vcross) < 0.0 ) return [];
   return p;
}

// Intersection between line and circle

func inter_l_c( line, circle )
{
   a = line.a;
   b = line.b;
   c = line.c;
   p = circle.center.x;
   q = circle.center.y;
   r = circle.radius;

   if( abs(a) > abs(b) ) { // solve for x
      A = a^2 + b^2;
      B = -2*b*b*p - 2*a*c + 2*b*q*a;
      C = c^2 - 2*b*q*c + b^2 * p^2 + b^2 * q^2 - r^2 * b^2;

      D = B^2 - 4*A*C;

      if( D < 0 ) {
         write, "Sorry, no solution";
         return [];
      }
      x = [(-B+sqrt(D))/(2*A), (-B-sqrt(D))/(2*A)];
      y = (c - a*x)/b;
   } else { // solve for y
      A = a^2 + b^2;
      B = -2*a*a*q - 2*b*c + 2*b*p*a;
      C = c^2 - 2*a*p*c + a^2 * p^2 + a^2 * q^2 - r^2 * a^2;

      D = B^2 - 4*A*C;

      if( D < 0 ) {
         write, "Sorry, no solution";
         return [];
      }
      y = [(-B+sqrt(D))/(2*A), (-B-sqrt(D))/(2*A)];
      x = (c - b*y)/a;
   }
   res = array( s_Point, 2 );
   res(*).type = "point";
   res(1).x = x(1);
   res(1).y = y(1);
   res(2).x = x(2);
   res(2).y = y(2);

   return res;

}

// Generic line drawing

func gdraw_line( line, li=, thick=, color=, ends= )
/* DOCUMENT gdraw_line, line, li=, thick=, color=, ends=

    Draws a line whatever type it has.
*/
{
   if( line.type == "line" ) {
      draw_line, line, li=li, thick=thick, color=color;
   } else if( line.type == "delline" ) {
      draw_delline, line, li=li, thick=thick, color=color, ends=ends;
   } else if( line.type == "semidelline" ) {
      draw_semidelline, line, li=li, thick=thick, color=color, ends=ends;
   } else error,"Illegal line type";
}

// Draw a given line on existing window

func draw_line( line, li=, thick=, color= )
{
   if( line.b == 0.0 ) { // vertical x = c/a
      oplot, [1,1]*line.c/line.a, [Inf_my, Inf_py];
   } else if( line.a == 0.0 ) { // horizontal y = c/b
      oplot, [Inf_mx, Inf_px], [1,1]*line.c/line.b;
   } else {
      if( abs(line.a) > abs(line.b) ) {
         y1 = Inf_my;
         x1 = (line.c - line.b * y1)/line.a;
         y2 = Inf_py;
         x2 = (line.c - line.b * y2)/line.a;
      } else {
         x1 = Inf_mx;
         y1 = (line.c - line.a * x1)/line.b;
         x2 = Inf_px;
         y2 = (line.c - line.a * x2)/line.b;
      }
      oplot,[x1,x2],[y1,y2], li=li, thick=thick, color=color;
   }
}

// Draw a given delimited line on existing window

func draw_delline( delline, li=, color=, thick=, ends= )
{
   oplot,[delline.p1.x,delline.p2.x],[delline.p1.y,delline.p2.y], \
      li=li, color=color, thick=thick;
   if( ends ) {
      draw_point, delline.p1, lbl="P1";
      draw_point, delline.p2, lbl="P2";
   }
}

// Draw a given semi-delimited line on existing window

func draw_semidelline( semidelline, li=, color=, thick=, ends= )
{
   d = dist_p_p( semidelline.p1, semidelline.p2 );
   distant_point = [semidelline.p2.x - semidelline.p1.x, semidelline.p2.y - semidelline.p1.y]*Inf_r/d;
   oplot,[semidelline.p1.x,distant_point(1)],[semidelline.p1.y,distant_point(2)], \
      li=li, color=color, thick=thick;
   if( ends ) {
      draw_point, semidelline.p1, lbl="P1";
   }
}

// Draw a polygon on existing window

func draw_polygon( polygon, li=, color=, fill=, thick=, ends= )
/* DOCUMENT draw_polygon, polygon, li=, color=, fill=, thick=, ends=

   'polygon' must be array of struct 's_Point'

   If keyword 'fill' is set then the polygon is filled.
*/
{
   n = numberof( polygon );
   // a side is from point i to point i+1;
   // if i+1 > n then 1 is to be used
   for( i = 1; i <= n; i++ ) {
      i2 = i == n ? 1 : i+1;
      dlinei = delline_p_p(polygon(i), polygon(i2));
      draw_delline, dlinei, li=li, color=color, thick=thick;
      if( ends ) draw_point, polygon(i),lbl="p"+itoa(i);
   }
   if( fill ) poly_fillc, polygon.x, polygon.y,color=color;
}

func draw_point( point, lbl= )
/* DOCUMENT draw_point, point, lbl=

     Keyword 'lbl' can be an integer or a string
*/   
{
   oplot,[point.x],[point.y],ps=12,symsize=0.3;
   if( numberof(lbl) ) {
      if( structof(lbl) == int || structof(lbl) == long ) lbl = itoa(lbl);
      xyouts,point.x,point.y,lbl;
   }
}

func draw_arc( arc, li=, color=, thick=, cnt=, lbl= )
/* DOCUMENT draw_arc, arc, li=, color=, thick=, cnt=, lbl=
*/
{
   draw_circle, arc, li=li, color=color, thick=thick, cnt=cnt, lbl=lbl;
}

func draw_circle( circle, li=, color=, thick=, cnt=, lbl= )
/* DOCUMENT draw_circle, circle, li=, color=, thick=, cnt=, lbl=
       Works also for arc.
*/
{
   if( circle.type == "circle" ) {
      v = span(0,2*pi,100);
   } else if( circle.type == "arc" ) {
      v = span(circle.angle1, circle.angle2, 100)*pi/180.;
   }
   x = circle.radius*cos(v) + circle.center.x;
   y = circle.radius*sin(v) + circle.center.y;


   oplot,x,y,li=li,color=color,thick=thick;
   if( cnt ) {
      draw_point, circle.center, lbl=lbl;
   }
}

func circle2plg( circle, num )
{
   if( is_void(num) ) num = 100;
   v = span(0,2*pi,num+1)(1:-1); // avoid repetition
   p = array(s_Point,num);
   p(*).type = "point";
   p.x = circle.radius*cos(v) + circle.center.x;
   p.y = circle.radius*sin(v) + circle.center.y;
   return p;
}

func inter_c_c( circ1, circ2 )
{

   extern Circ1, Circ2;

   x1 = circ1.center.x;
   y1 = circ1.center.y;
   r1 = circ1.radius;

   x2 = circ2.center.x;
   y2 = circ2.center.y;
   r2 = circ2.radius;

   window, 0;

   xmx = max([circ1.center.x+circ1.radius, circ2.center.x+circ2.radius]);
   xmn = max([circ1.center.x-circ1.radius, circ2.center.x-circ2.radius]);
   ymx = max([circ1.center.y+circ1.radius, circ2.center.y+circ2.radius]);
   ymn = max([circ1.center.y-circ1.radius, circ2.center.y-circ2.radius]);
   range = 0.6*max([xmx-xmn,ymx-ymn]);
   u = [-1,1];
   plot,range*u,range*u,ps=1;
   draw_circle, circ1, cnt=1, lbl="C_1";
   draw_circle, circ2, cnt=1, lbl="C_2";

   d = sqrt((x1-x2)^2 + (y1-y2)^2);
   if( d > r1 + r2 ) {
      write,"Sorry, no solution, circles too far apart";
      return [];
   }
   if( d < abs(r1 - r2) ) {
      write,"Sorry, no solution, one circle completely inside the other";
      return [];
   }

   Circ1 = circ1;
   Circ2 = circ2;

   // angles for min and max

   vmax = zero2pi(atan(y1-y2,x1-x2));
   vmin = zero2pi(vmax + pi);

   v = span(0,2*pi,1000);
   dcder, v, dc, ddcdv;
   dcder, vmax, dcmax, ddcdvmax;
   dcder, vmin, dcmin, ddcdvmin;

   window,1;
   plot,v,dc;

   oplot, [vmax],[dcmax],ps=3;
   oplot, [vmin],[dcmin],ps=4;

   window,0;

   v0 = nraphson( dcder, vmin, vmax, 1.e-10 );
   s1 = s_Point(type="point", x=circ1.radius*cos(v0)+circ1.center.x, \
                y=circ1.radius*sin(v0)+circ1.center.y);
   draw_point,s1,lbl="S_1";
   l = line_p_p(circ1.center, circ2.center);
   s2 = mirror_p_l( s1, l );
   draw_point,s2,lbl="S_2";

   solut = array( s_Point, 2 );
   solut(*).type = "point";
   solut(1) = s1;
   solut(2) = s2;

   return solut;

}

func dcder( v, &dc, &ddcdv )
/* DOCUMENT dcder, angle, >dc, >ddcdv

     returns the smallest distance between a point on circle1 (given by
     the angle 'angle' (in radians)) and the closest point on circle2
     - positive if outside circle1 and negative if inside
     as the second parameter and its derivative as the third parameter.
     This is compatible with the 'nraphson' root-finding function.

     Externals Circ1 and Circ2 must be defined beforehand.

     2008-08-01/NJW
*/
{

      extern Circ1, Circ2;

      cv = cos(v);
      sv = sin(v);
      x = Circ1.radius*cv + Circ1.center.x;
      y = Circ1.radius*sv + Circ1.center.y;
      g = (x-Circ2.center.x)^2 + (y-Circ2.center.y)^2;
      dc = sqrt(g) - Circ2.radius;
      dgdv = -2*Circ1.radius*sv*(Circ1.radius*cv + Circ1.center.x - Circ2.center.x) \
           +  2*Circ1.radius*cv*(Circ1.radius*sv + Circ1.center.y - Circ2.center.y);
      ddcdv = 0.5*dgdv/sqrt(g);
}

// Intersections between line and polygon

func inter_l_plg( line, polygon )
{
   n = numberof(polygon);
   qq = [];
   for( i = 1; i <= n; i++ ) {
      i2 = i == n ? 1 : i+1;
      dlinei = delline_p_p(polygon(i), polygon(i2));
      q = inter_dl_l( dlinei, line );
      if( !is_void(q) ) grow, qq, q;
   }
   return qq;
}

// Intersections between delimited line and polygon

func inter_dl_plg( delline, polygon )
{
   n = numberof(polygon);
   qq = [];
   for( i = 1; i <= n; i++ ) {
      i2 = i == n ? 1 : i+1;
      dlinei = delline_p_p(polygon(i), polygon(i2));
      q = inter_dl_dl( dlinei, delline );
      if( !is_void(q) ) grow, qq, q;
   }
   return qq;
}

// Hatch an area defined by a polygon

func hatch( polygon, angle, dist, color=, li=, thick=, wor= )
/* DOCUMENT hatch, polygon, angle, dist, color=, li=, thick=, wor=

   Draws a hatched polygon (array of struct 's_Point'). Hatch lines are tilted an
   'angle' (in degrees) with respect to horizontal. Their distance are given by
   'dist'.

   Keyword 'wor' is to be set if the polygon is given in
    'world coordinates'

   Other keywords are as for 'plot'.
*/
{
   local ndc_ply, wqq;

   if( !verify_plg(polygon) ) {
      write,"The polygon could not be verified - do nothing";
      return;
   }
   if( angle > 89. || angle < -89. ) {
      write,"Angle not in acceptable range - do nothing";
      return;
   }

   if( wor ) {
      gcoord_conv, polygon, ndc_ply, from="wor", to="ndc";
      polyg = ndc_ply;
   } else polyg = polygon;

   // Get extremes of polygon i.e. enclosing rectangle
   n = numberof( polyg );
   xmax = max(polyg.x);
   xmin = min(polyg.x);
   ymax = max(polyg.y);
   ymin = min(polyg.y);
   recta = box2plg( xmin, xmax, ymin, ymax );
   //+ recta = array(s_Point,4);
   //+ recta(*).type = "point";
   //+ recta(1).x = xmax;
   //+ recta(1).y = ymin;
   //+ recta(2).x = xmin;
   //+ recta(2).y = ymin;
   //+ recta(3).x = xmin;
   //+ recta(3).y = ymax;
   //+ recta(4).x = xmax;
   //+ recta(4).y = ymax;

   p = s_Point();
   p.type = "point";
   if( angle >= 0.0 ) {
      p.x = xmax;
      p.y = ymin;
   } else {
      p.x = xmin;
      p.y = ymin;
   }
   l = line_p_angle( p, angle );
   norm = sqrt(l.a^2 + l.b^2);
   dx = dist*l.a/norm;
   dy = dist*l.b/norm;



   do {

      p.x += dx;
      p.y += dy;
      l = line_p_angle( p, angle );
      qrect = inter_l_plg( l, recta );

      // get the intersections between line and polygon

      qq = inter_l_plg( l, polyg );
      nqq = numberof(qq);
      if( nqq == 0 ) continue;
      if( nqq%2 == 1 ) error,"odd number of intersections with polygon";
      // sort according x value
      is = sort(qq.x);
      qq = qq(is);

      if( wor ) {
         gcoord_conv, qq, wqq, from="ndc", to="wor";
      } else wqq = qq;

      for(i = 1; i <= nqq/2; i++ ) {
         oplot,[wqq(2*(i-1)+1).x,wqq(2*(i-1)+2).x], \
               [wqq(2*(i-1)+1).y,wqq(2*(i-1)+2).y], \
               color=color, li=li, thick=thick;
      }

   } while( numberof(qrect) );

}

func box2plg( xmin, xmax, ymin, ymax )
/* DOCUMENT b = box2plg( xmin, xmax, ymin, ymax )

    returns a 4-apex polygon.
*/
{
   recta = array(s_Point,4);
   recta(*).type = "point";
   recta(1).x = xmax;
   recta(1).y = ymin;
   recta(2).x = xmin;
   recta(2).y = ymin;
   recta(3).x = xmin;
   recta(3).y = ymax;
   recta(4).x = xmax;
   recta(4).y = ymax;
   return recta;
}

/* Function gcoord_conv */

func gcoord_conv( ply_in, &ply_out, from=, to= )
/* DOCUMENT gcoord_conv, ply_in, >ply_out, from=, to=

   Convert coordinates between NDC (Normalized Device Coordinates)
   keyword value "ndc", virtual coordinates (keyword value "vir")
   and world (i.e. data space) (keyword value "wor") coordinates.
   The (A4) device coordinates have been expanded by a factor of
   sqrt(2) in the y direction compared to NDC. [not implemented]

   Input and result is a polygon i.e. array of struct s_Point

   2009-03-19/NJW cloned from mcoord_conv of package 'plot.i'
*/
{
   if( is_void(from) ) {
      write,"Keyword 'from' not defined";
      return;
   }
   if( is_void(to) ) {
      write,"Keyword 'to' not defined";
      return;
   }
   // avoid changing input coords
   ply = ply_in;

   v = viewport()(1:4);
   w = limits()(1:4);
   wind = 1 + window();
   local xlog, ylog;
   mlogxy, xlog, ylog, get=1;

   from = strlowcase(from);
   to = strlowcase(to);
   if( from == "ndc" ) fco = v;
   if( from == "vir" ) fco = [0.,1.,0.,1.];
   if( from == "wor" ) fco = w;
   if( to == "ndc" ) tco = v;
   if( to == "vir" ) tco = [0.,1.,0.,1.];
   if( to == "wor" ) tco = w;

   if( from == "wor" ) {
      // World to linear
      if( xlog ) {
         fco(1) = log(fco(1));
         fco(2) = log(fco(2));
         ply.x = log(ply.x);
      }
      if( ylog ) {
         fco(3) = log(fco(3));
         fco(4) = log(fco(4));
         ply.y = log(ply.y);
      }
   }
   if( to == "wor" ) {
      // linear to world
      if( xlog ) {
         tco(1) = log(tco(1));
         tco(2) = log(tco(2));
      }
      if( ylog ) {
         tco(3) = log(tco(3));
         tco(4) = log(tco(4));
      }
   }
   ply_out = ply_in;
   fac = (ply.x - fco(1))/(fco(2)-fco(1));
   ply_out.x = tco(1) + fac*(tco(2) - tco(1));
   fac = (ply.y - fco(3))/(fco(4) - fco(3));
   ply_out.y = tco(3) + fac*(tco(4) - tco(3));

   if( to == "wor" ) {
      if( xlog ) ply_out.x = exp(ply_out.x);
      if( ylog ) ply_out.y = exp(ply_out.y);
   }

   return;
}

// Translate a polygon

func translate_plg( polygon, dx, dy )
/* DOCUMENT new_plg = translate_plg( polygon, dx, dy )
            new_plg = translate_plg( polygon, vector )
*/
{
   if( typeof(dx) == "struct_instance" ) {
      if( dx.type != "vector" ) error,"Illegal argument";
      dy = dx.y;
      dx = dx.x;
   }
   n = numberof(polygon);
   r = polygon;
   for( i = 1; i <= n; i++ ) {
      r(i).x += dx;
      r(i).y += dy;
   }
   return r;
}

// Rotate a polygon

func rotate_plg( polygon, angle )
/* DOCUMENT  plg = rotate_plg( polygon, angle )
     Where angle is in degrees. The axis of rotation is the average position.
*/
{
   n = numberof(polygon);
   res = polygon;
   cen = s_Point();
   cen.x = avg(polygon.x);
   cen.y = avg(polygon.y);
   an = angle*pi/180.;
   can = cos(an); san = sin(an);
   for( i = 1; i <= n; i++ ) {
      v = vec_p_p(cen,polygon(i));
      x = v.x * can - v.y * san;
      y = v.x * san + v.y * can;
      res(i).x = cen.x + x;
      res(i).y = cen.y + y;
   }

   return res;
}


// Decide if a given point is inside a polygon

func inside_plg( pnt, polyg )
/* DOCUMENT ans = inside_plg( pnt, polyg )

   returns 1 if pnt is inside polyg and 0 otherwise
*/
{
   if( !verify_plg(polyg) ) {
      write,"The polygon could not be verified - do nothing";
      return [];
   }
   // First determine a point that is definitely inside
   // by checking the diagonals of the enclosing box
   // Get extremes of polygon i.e. enclosing rectangle
   n = numberof( polyg );
   eps = 1.e-3;
   xmax = max(polyg.x) + eps;
   xmin = min(polyg.x) - eps;
   ymax = max(polyg.y) + eps;
   ymin = min(polyg.y) - eps;
   recta = box2plg( xmin, xmax, ymin, ymax );
   if( pnt.x < xmin ) return 0;
   if( pnt.x > xmax ) return 0;
   if( pnt.y < ymin ) return 0;
   if( pnt.y > ymax ) return 0;

   ll = xy2point(xmin,ymin);
   ur = xy2point(xmax,ymax);
   diagonal = line_p_p( ll, ur );
   // get the intersections between diagonal and polygon
   qq = inter_l_plg( diagonal, polyg );
   nqq = numberof(qq);
   if( nqq > 0 ) {
      if( nqq%2 == 1 ) error,"odd number of intersections with polygon";
      // sort according x value
      is = sort(qq.x);
      qq = qq(is);
      pinside = xy2point( 0.5*(qq(1).x + qq(2).x), 0.5*(qq(1).y + qq(2).y ) );
      draw_point,pinside,lbl="A1";
   } else {
      ul = xy2point(xmin,ymax);
      lr = xy2point(xmax,ymin);
      diagonal = line_p_p( ll, ur );
      // get the intersections between diagonal and polygon
      qq = inter_l_plg( diagonal, polyg );
      nqq = numberof(qq);
      if( nqq > 0 ) {
         if( nqq%2 == 1 ) error,"odd number of intersections with polygon";
         // sort according x value
         is = sort(qq.x);
         qq = qq(is);
         pinside = xy2point( 0.5*(qq(1).x + qq(2).x), 0.5*(qq(1).y + qq(2).y ) );
      draw_point,pinside,lbl="A2";
      } else error,"##43##";
   }

   // define line connecting the given point and the inside point

   lcon = delline_p_p( pnt, pinside );
   qq = inter_dl_plg( lcon, polyg );
   nqq = numberof(qq);
   return (nqq+1)%2;
}

//   Draw an arrow

func draw_garrow( point1, point2, sw, hw, hl, color=, fill=, thick=, tri= )
/* DOCUMENT draw_garrow, point1, point2, sw, hw, hl, color=, fill=, thick=, tri=

     Uses function 'draw_arrow' in plot.i to draw an arrow from point1 to point2.

     Arguments: sw   stem width in NDC coordinates
                hw   head width
                hl   head length

     Keywords:  color, thick   standard
                tri   for triangular arrowhead
                fill  for filled arrow
*/
{
   if( point1.type != "point" ) error,"Illegal point1 argument";
   if( point2.type != "point" ) error,"Illegal point2 argument";
   draw_arrow, point1.x, point1.y, point2.x, point2.y, sw, hw, hl, \
                color=color, fill=fill, thick=thick, tri=tri;
}
