extern mcomplexdoc;
/* DOCUMENT  mcomplex.i  Additional complex functions

    rhotheta  :  change to rho * exp(i theta) representation
    cubicroot :  Three cubic roots
    nthroot   :  N Nth roots
    cubic_eq  : Solves cubic equation

*/

/* Function rhotheta */

func rhotheta( z )
/* DOCUMENT res = rhotheta( z )

    Returns 2 element array [modulus, argument (theta angle)]
    of the complex number, z (works also for non-complex numbers)

    2008-03-29/NJW
*/
{
   rho = sqrt(double(z*conj(z)));
   theta = typeof(z) == "complex" ? atan( z.im, z.re ) : pi*(1-sign(z))/2;
   return [rho, theta];
}

/* Function cubicroot */

func cubicroot( z )
/* DOCUMENT roots = cubicroot( z )

   Returns the 3 cubic complex roots of a number

   2008-03-29/NJW
*/
{
   q = rhotheta( z );
   rho = q(1)^(1./3.);
   th1 = q(2)/3;
   th2 = th1 + (2*pi)/3;
   th3 = th2 + (2*pi)/3;
   res = array(complex,3);
   res(1).re = rho*cos(th1);
   res(1).im = rho*sin(th1);
   res(2).re = rho*cos(th2);
   res(2).im = rho*sin(th2);
   res(3).re = rho*cos(th3);
   res(3).im = rho*sin(th3);

   return res;

}

/* Function nthroot */

func nthroot( z, n )
/* DOCUMENT roots = nthroot( z, n )

   Returns the n nth complex roots of a number
   (z^(1./n))

   2008-03-29/NJW
*/
{
   q = rhotheta( z );
   rho = q(1)^(1./n);
   theta = q(2)/n;
   phi = (2*pi)/n;
   res = array(complex,n);
   for( i = 1; i <= n; i++ ) {
      res(i).re = rho*cos(theta);
      res(i).im = rho*sin(theta);
      theta += phi;
   }

   return res;

}

/* Function cubic_eq */

func cubic_eq( a, b, c, s= )
/* DOCUMENT x = cubic_eq( a, b, c, s= )

   Returns the three (two of them may be complex) roots to:
    x^3 + a*x^2 + b*x + c = 0
    s=1 : one solution (default)
    s=-1 : another solution

    2008-03-27/NJW
*/
{
   third = 1.0/3.0;
   if( is_void(s) ) s = 1;
   p = b - a^2/3;
   q = c + (2*a^3 - 9*a*b)/27;
   det = complex();
   det.re = q^2/4 + p^3/27;
   sqdet = sqrt(det);
   u1 = q/2 + s*sqdet;
   u = cubicroot(u1); // returns 3 roots
   //+ v = p/(3*u);
   return p/(3*u) - u - a/3;

}

/* Function cmplx */

func cmplx( a, b )
/* DOCUMENT  res = cmplx( a, b)
*/
{
   na = numberof(a);
   nb = numberof(b);
   if( is_scalar(a) && !is_scalar(b) ) a = array(a,nb);
   if( !is_scalar(a) && is_scalar(b) ) b = array(b,na);
   if( na != numberof(b) ) error,"##2## Inconsistent arguments";
   if( is_scalar(a) ) {
      res = complex();
      res.re = a;
      res.im = b;
   } else {
      res = array(complex,na);
      res.re = a;
      res.im = b;
   }
   return res;
}
