extern organizedoc;
/* DOCUMENT organize

   Functions:
    arrange_words_in_columns
    dirsync
    extra_files
    find_all_files
    get_identical_files
    list_file_attributes
    lsdirpath
    mv2special
    mvall2special
    nice_externs
    syco (commands for 'dirsync')
    ynclude

    You may keep in mind that checksum_string = md5sum( file, hex=1)
                           or checksum_array  = md5sum( file )
    (found in ../yorick-v_v/i/md5.i)
    could help to verify file identities. (note added 2008-08-29/NJW)

    2008-08-14/NJW

    Wanted functionality is synchronization between two directories that
    eventually should contain identical files for backup purposes.
    *** the above has been achieved with dirsync + syco ***

*/

/* Function extra_files */

func extra_files( dir1, dir2 )
/* DOCUMENT list = extra_files( dir1, dir2 )

   Return list of files in dir2 that do not exist in dir1
   2007-09-22/NJW
*/
{

   list1 = find_all_files( dir1 );

   list2 = find_all_files( dir2 );

   // Files to copy from dir2 to dir1
   list = filter_done( list1, list2 );
}

/* Function find_all_files */

func find_all_files ( dir, chat= )
/* DOCUMENT list = find_all_files( dir, chat= )

   Return a list with all files (not directories) in or
   below directory 'dir'.

   2007-09-23/NJW
*/
{
   //+ require, "c:/yo/filter_done.i";
   local nsubs, list_all, sublist, list_subdirs;

   if( is_void(chat) ) chat = 0;

   if(chat>0) write,format="FAF starting on directory %s ...\n", dir;
   list_all = lsdirpath( dir, list_subdirs );
   // If 'dir' exists then list_all is either a string array or nil
   // Else list_all == 0 (long)
   if( typeof(list_all) == "long" ) return list_all;
   nsubs = numberof( list_subdirs );
   if(chat>0) write,format="FAF: Found %i subdirectories\n", nsubs;
   if( nsubs > 0 ) {
      for( i = 1; i <= nsubs; i++ ) {
         sublist = find_all_files( list_subdirs(i) );
         grow, list_all, sublist;
      }
   }
   return list_all;
}

/* Function lsdirpath */

func lsdirpath( dir, &subs )
/* DOCUMENT files = lsdirpath( dir, >subs )

   same functionality as 'lsdir' with two arguments, except that
   full paths beginning from 'dir' are returned.

   2007-09-23/NJW
*/
{

   list = lsdir( dir, subs );
   // If the directory 'dir' does not exist, lsdir returns '0'
   // lsdirpath must inherit the same behaviour
   if( typeof(list) == "long" ) {
      if( list != 0 ) error,"LSDIRPATH should not happen, list is long and != 0";
      write,format="No such directory: %s\n", dir;
      return list;
   }
   if( !is_void(subs) ) subs = dir+"/"+subs;
   if( !is_void(list) ) list = dir+"/"+list;
   return list;
}

/* Function arrange_words_in_columns */

func arrange_words_in_columns( filename, ncols, colwidth, outfile=, embed=, margin= )
/* DOCUMENT arrange_words_in_columns, infilename, ncols, \
                 colwidth, outfile=, embed=, margin=

   The first argument must be a text file (scalar string) or an array of
   strings (may contain just one element).
   The words are separated out by
   space(s), sorted, and output column by column.

   Keyword 'outfile' can be set as 1 in which case 'filename' is overwritten
           if it is indeed a filename
           or a string that becomes the name of the output file.
           If zero or not given no writing will be done.

   Keyword 'embed' (a two element integer array) implies that the words are given
   between the two line numbers given incl.

   If 'ncols' or 'colwidth' is given as [] (void) then the values are reproduced
   from the input table.

   Keyword margin : Width of left margin

   2008-08-01/NJW
   2009-04-24/NJW updated with 'embed' keyword and functionality
   2010-02-24/NJW updated to accept a string array as well
*/
{

   if( typeof(filename) != "string" ) error,"AWIC called with illegal first argument";

   is_file = is_scalar(filename);
   part1 = part2 = [];

   if( is_file ) { // A text file is expected

      // get the table of words and isolate the table if
      // embedded in a text file

      if( !file_test(filename) ) {
         write,"AWIC Warning: file not found";
         return;
      }
      table = read_slist(filename);
      nt = numberof(table);
      write,format="%i lines in %s\n", nt, filename;

      if( !is_void(embed) ) {
         if( embed(2) > nt ) error,"AWIC error, keyword 'embed'";
         part1 = embed(1) == 1 ? [] : table(1:embed(1)-1);
         part2 = embed(2) == nt ? [] : table(embed(2)+1:0);
         table = table(embed(1):embed(2));
      }

   } else {         // assume that input is the string array

      table = filename;
      if( !is_void(outfile) ) { // must be a genuine filename
         if( typeof(outfile) != "string" ) error,"AWIC, outfile must be a string (filename)";
      }
   }

   lun = [];
   if( typeof(outfile) == "string" ) {
      lun = open(outfile,"w");
   } else if( !is_void(outfile) ) {
      if( outfile ) {
         lun = open(filename,"w");
      }
   }

   nt = numberof(table);
   // avoid empty lines
   ctable = [];
   xtable = [];
   for( i = 1; i <= nt; i++ ) {
      s = strtrim(strcompress(table(i)));
      if( strlen(s) > 0 ) {
         grow, xtable, table(i);
         grow, ctable, s;
      }
   }
   table = xtable; // now without empty lines else unchanged
   nt = numberof(table);

   // determine number of columns if ncols == []
   // as the maximum encountered in (c)table

   if( is_void(ncols) ) {
      ncols = 1;
      for( i = 1; i <= nt; i++ ) {
         n = pos = 0;
         // count number of spaces
         while((pos=strpos(ctable(i)," ",pos+1))) n++;
         if( n >= ncols ) ncols = n+1;
      }
   }

   // determine column width if colwidth == []

   if( is_void(colwidth) ) {
      if( ncols == 1 ) {
         colwidth = 1; // not applicable
      } else {
         // find first line with more that one word
         for( i = 1; i <= nt; i++ ) {
            n = pos = 0;
            // count number of spaces
            while((pos=strpos(ctable(i)," ",pos+1))) n++;
            if( n >= 1 ) break;
         }
         wds = strsplit(ctable(i)," ");
         pos1 = strpos(table(i),wds(1),1);
         pos2 = strpos(table(i),wds(2),strlen(wds(1))+1);
         colwidth = pos2 - pos1;
         if( is_void(margin) ) margin = pos1 - 1;
      }
   }

   // form the list of words in 'list'

   list = [];
   for(i=1;i<=nt;i++) grow,list,strsplit(ctable(i)," ");
   list = list(sort(list));
   nlist = numberof(list); // number of words
   nrows = nlist/ncols;    // number of lines with 'ncols' words
   rem = nlist - nrows*ncols; // number of remaining words

   // 's_ind' is the start index at the top of each column
   s_ind = array(long, ncols);
   s_ind(1) = 1;

   // get next start index as adding number of rows plus the
   // partially fillled extra row at bottom
   for( j = 1; j <= ncols; j++ ) {
      nr = nrows + (j <= rem);
      if( j < ncols ) {
         s_ind(j+1) = nr + s_ind(j);
      }
   }

   // outputting the reformatted table
   // start with part1 if defined

   if( !is_void(part1) && typeof(lun) == "text_stream" ) {
      for(k=1;k<=numberof(part1);k++) write,lun,format="%s\n",part1(k);
   }

   for( i = 1; i <= nrows + (rem > 0); i++ ) {
      line = "";
      if(!is_void(margin)) line = strpadd(line,margin," ");
      for( j = 1; j <= ncols; j++ ) {
         index = s_ind(j) + i-1;
         if( i > nrows && j > rem ) break;
         line += strpadd(list(index),colwidth," ");
      }
      write,format="%s\n",line;
      if( typeof(lun) == "text_stream" ) write,lun,format="%s\n",line;
   }

   // end with part2 if defined

   if( !is_void(part2) && typeof(lun) == "text_stream" ) {
      for(k=1;k<=numberof(part2);k++) write,lun,format="%s\n",part2(k);
   }

   if( typeof(lun) == "text_stream" ) close, lun;

}

/* Function dirsync */

func dirsync ( dir1, dir2, rec=, syco= )
/* DOCUMENT dirsync, dir1, dir2, rec=, syco=

   Make arrays with
      a) files in dir1 that are absent in dir2 : Files_dir1_alone
      b) files in dir2 that are absent in dir1 : Files_dir2_alone
      c) files common for the two directories  : Files_common

   Keyword rec (recursive) descend into subdirectories default=0
   Keyword syco is only used when called from function 'syco'

   SEE ALSO: syco

   2008-11-29/NJW
*/
{
  /*
   *  Assume that identical file names represent identical files
   *
   */

   extern Rec, Dir1, Dir2, Files_dir1_alone, \
          Files_dir2_alone, Files_common, \
          List_dir1, List_dir2;

   local pathname, basename;

   Dir1 = dir1;
   Dir2 = dir2;

   len1 = strlen(dir1);
   len2 = strlen(dir2);

   if( is_void(rec) ) rec = 0;
   Rec = rec;

   if( rec ) {
      List_dir1 = find_all_files( dir1 );
      List_dir2 = find_all_files( dir2 );
   } else {
      List_dir1 = lsdirpath( dir1 );
      List_dir2 = lsdirpath( dir2 );
   }
   d = 1;
   if( typeof(List_dir1) == "long" ) {
      write,format="Dir1 (%s) does not exist\n", dir1;
      write,format="    You may want to use: > mkdirp, \"%s\"\n", dir1;
      d = 0;
   }
   if( typeof(List_dir2) == "long" ) {
      write,format="Dir2 (%s) does not exist\n", dir2;
      write,format="    You may want to use: > mkdirp, \"%s\"\n", dir2;
      d = 0;
   }
   if( !d ) return;

   n1 = numberof(List_dir1);
   if( n1 ) {
      restnames1 = array(string, n1);
      for( i = 1; i <= n1; i++ ) {
         pos = strpos(List_dir1(1),dir1,1);
         if( pos != 1 ) error,"##45##";
         restnames1(i) = strpart( List_dir1(i), len1+2:0);
      }
   }
   n2 = numberof(List_dir2);
   if( n2 ) {
      restnames2 = array(string, n2);
      for( i = 1; i <= n2; i++ ) {
         pos = strpos(List_dir2(1),dir2,1);
         if( pos != 1 ) error,"##45##";
         restnames2(i) = strpart( List_dir2(i), len2+2:0);
      }
   }
   if( n1 ) {
      if( n2 ) {
         // There are files in both directories
         Files_dir1_alone = filter_done( restnames2, restnames1 );
         Files_dir2_alone = filter_done( restnames1, restnames2 );
         n1_alone = numberof( Files_dir1_alone );
         n2_alone = numberof( Files_dir2_alone );
         Files_common = filter_common( restnames1, restnames2 );
         n_common = numberof( Files_common );
         write,format="%6i files in total in Dir1: %s\n", n1, dir1;
         write,format="%6i files in total in Dir2: %s\n", n2, dir2;
         write,format="%6i files in Dir1 and not in Dir2\n", n1_alone;
         write,format="%6i files in Dir2 and not in Dir1\n", n2_alone;
         write,format="%6i files are common\n", n_common;
      } else {
         // There are only files in dir1
         Files_dir1_alone = restnames1;
         write,format="%6i files in total in Dir1: %s\n", n1, dir1;
         write,format="and no files in Dir2: %s\n", dir2;
      }
   } else {
      if( n2 ) {
         Files_dir2_alone = restnames2;
         write,format="%6i files in total in Dir2: %s\n", n2, dir2;
         write,format="and no files in Dir1: %s\n", dir1;
      } else {
         write,"No files at all";
      }
   }
   if( !syco ) {
      write,"External variables defined:";
      write,"      Dir1 and Dir2 - directory names";
      write,"      Files_dir1_alone - list of files";
      write,"      Files_dir2_alone - list of files";
      write,"      Files_common - files common to both directories";
      write,"";
      write," Call 'syco' to make an action";
      write,"";
   }
}

/* Function syco */

func syco( a )
/* DOCUMENT syco
    Sync Command - must be preceded by a call of dirsync that sets a number
    of external variables

    SEE ALSO: dirsync, setup_dir
*/
{

   extern Rec, Dir1, Dir2, Files_dir1_alone, \
          Files_dir2_alone, Files_common, \
          List_dir1, List_dir2;

   cmds = ["full","cp2dir1","cp2dir2","cmpr","cmpu1", \
           "cmpu2","disy","sz1","sz2","x","?"];

   command = "";

   while( command != "x" ) {
      read,prompt="Enter command ('?' for help) : ... ", command;
      if( !numberof(where(command == cmds))) {
         write,format="Illegal command : %s\n", command;
         continue;
      }

      if( command == "?" ) {
         write,"full    - full synchronization";
         write,"cp2dir1 - copy missing files to dir1";
         write,"cp2dir2 - copy missing files to dir2";
         write,"cmpr    - compare common files, just report";
         write,"cmpu1   - compare common files, update in dir1";
         write,"cmpu2   - compare common files, update in dir2";
         write,"disy    - run 'dirsync' to update file lists";
         write,"sz1     - report total size for dir1 + 10 biggest files";
         write,"sz2     - report total size for dir2 + 10 biggest files";
         write,"x       - exit";
      }
      if( command == "full" ) {
         n = numberof(Files_dir2_alone);
         for( i = 1; i <= n; i++ ) {
            write,format="Going to copy: %s\n", Dir2+"/"+Files_dir2_alone(i);
            setup_dir, Dir1+"/"+Files_dir2_alone(i);
            cp,Dir2+"/"+Files_dir2_alone(i),Dir1+"/"+Files_dir2_alone(i);
         }
         n = numberof(Files_dir1_alone);
         for( i = 1; i <= n; i++ ) {
            write,format="Going to copy: %s\n", Dir1+"/"+Files_dir1_alone(i);
            setup_dir, Dir2+"/"+Files_dir1_alone(i);
            cp,Dir1+"/"+Files_dir1_alone(i),Dir2+"/"+Files_dir1_alone(i);
         }
      }
      if( command == "cp2dir1" ) {
         n = numberof(Files_dir2_alone);
         for( i = 1; i <= n; i++ ) {
            write,format="Going to copy: %s\n", Dir2+"/"+Files_dir2_alone(i);
            setup_dir, Dir1+"/"+Files_dir2_alone(i);
            cp,Dir2+"/"+Files_dir2_alone(i),Dir1+"/"+Files_dir2_alone(i);
         }
      }
      if( command == "cp2dir2" ) {
         n = numberof(Files_dir1_alone);
         for( i = 1; i <= n; i++ ) {
            write,format="Going to copy: %s\n", Dir1+"/"+Files_dir1_alone(i);
            setup_dir, Dir2+"/"+Files_dir1_alone(i);
            cp,Dir1+"/"+Files_dir1_alone(i),Dir2+"/"+Files_dir1_alone(i);
         }
      }
      if( strpart(command,1:3) == "cmp" ) {
         n_diff = 0;
         n = numberof(Files_common);
         for( i = 1; i <= n; i++ ) {
            if( i%10 == 0 ) write,format="%s",".";
            chsum1 = md5sum( Dir1+"/"+Files_common(i), hex=1 );
            chsum2 = md5sum( Dir2+"/"+Files_common(i), hex=1 );
            if( chsum1 != chsum2 ) {
               write,format="\nDifference %s\n", Dir1+"/"+Files_common(i);
               write,format="       and %s\n", Dir2+"/"+Files_common(i);
               n_diff++;
               if( command == "cmpu1" ) {
                  write,format="\nUpdating %s\n", Dir1+"/"+Files_common(i);
                  cp,Dir2+"/"+Files_common(i), Dir1+"/"+Files_common(i);
               }
               if( command == "cmpu2" ) {
                  write,format="\nUpdating %s\n", Dir2+"/"+Files_common(i);
                  cp,Dir1+"/"+Files_common(i), Dir2+"/"+Files_common(i);
               }
            }
         }
         plural = n_diff == 1 ? "" : "s";
         write,format="\n%i files compared, %i differing file%s\n", \
                        n, n_diff, plural;
      }
      if( command == "disy" ) {
         dirsync, Dir1, Dir2, rec=Rec, syco=1;
      }
      if( command == "sz1" ) {
         n = numberof(List_dir1);
         top10 = 0;
         top10name = "";
         totsz = 0.0;
         for( i = 1; i <= n; i++ ) {
            ff = open(List_dir1(i),"rb");
            sz = sizeof(ff);
            close,ff;
            totsz += sz;
            if( sz > min(top10) ) {
               grow, top10, sz;
               grow, top10name, List_dir1(i);
               if( numberof(top10) > 10 ) {
                  is = sort(top10);
                  top10 = top10(is)(2:0);
                  top10name = top10name(is)(2:0);
               }
            }
         }
         local unit;
         _bytes, totsz, unit;
         write,format="Total size of %s: %.3f %s\n", Dir1, totsz, unit;
         for( i = numberof(top10); i >= 1; i-- ) {
            dsz = double(top10(i));
            _bytes, dsz, unit;
            write,format="%7.3f %s %s\n", dsz, unit, top10name(i);
         }
      }
      if( command == "sz2" ) {
         n = numberof(List_dir2);
         top10 = 0;
         top10name = "";
         totsz = 0.0;
         for( i = 1; i <= n; i++ ) {
            ff = open(List_dir2(i),"rb");
            sz = sizeof(ff);
            close,ff;
            totsz += sz;
            if( sz > min(top10) ) {
               grow, top10, sz;
               grow, top10name, List_dir2(i);
               if( numberof(top10) > 10 ) {
                  is = sort(top10);
                  top10 = top10(is)(2:0);
                  top10name = top10name(is)(2:0);
               }
            }
         }
         local unit;
         _bytes, totsz, unit;
         write,format="Total size of %s: %.3f %s\n", Dir2, totsz, unit;
         for( i = numberof(top10); i >= 1; i-- ) {
            dsz = double(top10(i));
            _bytes, dsz, unit;
            write,format="%7.3f %s %s\n", dsz, unit, top10name(i);
         }
      }
   }
}

func _bytes( &sz, &unit )
{
   unit = "bytes";
   if( sz > 1024. ) { unit = "kB"; sz /= 1024.; }
   if( sz > 1024. ) { unit = "MB"; sz /= 1024.; }
   if( sz > 1024. ) { unit = "GB"; sz /= 1024.; }
}

/* Function list_file_attributes */

func list_file_attributes( dir, outfile, rec=, app= )
/* DOCUMENT list_file_attributes, dir, outfile, rec=

    Keywords:  rec  :   set for recursive search
               app  :   append to existing 'outfile'

   2008-12-15/NJW
*/
{
   errmode = 1;
   mode = app ? "a" : "w";

   if( rec ) {
      list = find_all_files( dir );
   } else {
      list = lsdirpath( dir, subdirs );
   }
   nlist = numberof(list);
   szarr = [];
   chsumarr = [];
   listarr = [];

   for( i = 1; i <= nlist; i++ ) {
      f = open( list(i),"rb", errmode );
      if( f ) {
         sz = sizeof(f);
         close,f;
         chsum = md5sum( list(i), hex=1 );
         grow, szarr, sz;
         grow, chsumarr, chsum;
         grow, listarr, list(i);
      } else {
         write,format="%s is probably a link\n", list(i);
      }
   }
   nlist = numberof(listarr);

   is = sort(szarr);
   szarr = szarr(is);
   chsumarr = chsumarr(is);
   list = list(is);

   f = open( outfile, mode );
   for( i = 1; i <= nlist; i++ ) {
      write,f,format="%12i %36s %s\n", szarr(i), chsumarr(i), list(i);
   }
   close,f;

   write,"Done! Result in "+outfile;

}

/* Function get_identical_files */

func get_identical_files( filename, outfile= )
/* DOCUMENT get_identical_files, filename, outfile=

   'filename' is supposed to te an output file from 'list_file_attributes'
   or in the same format.

*/
{

   if( !file_test(filename) ) {
      write,format="File not found: %s\n", filename;
      return;
   }
   if( !is_void(outfile) ) {
      fout = open( outfile, "w" );
   } else fout = [];

   lines = rdfile( filename );
   nlines = numberof(lines);
   sz = 0;
   chsum = "";
   nam = "";
   szarr = array(long,nlines);
   chsumarr = array(string,nlines);
   namarr = array(string,nlines);
   for( i = 1; i <= nlines; i++ ) {
      sread,lines(i),sz,chsum,nam;
      szarr(i) = sz;
      chsumarr(i) = chsum;
      namarr(i) = nam;
   }

   // sort according to file size

   is = sort(szarr);
   szarr = szarr(is);
   chsumarr = chsumarr(is);
   namarr = namarr(is);

   u = uniq(szarr);
   grow,u,numberof(szarr)+1;
   grps = shift(u,1) - u;
   grps = grps(1:-1);
   g = where(grps > 1);
   ngrps = numberof(g);
   for( i = 1; i <= ngrps; i++ ) {
      nmem = grps(g(i));
      idx = u(g(i));
      chs = chsumarr(idx:idx+nmem-1);
      nms = namarr(idx:idx+nmem-1);
      iss = sort(chs);
      chs = chs(iss);
      nms = nms(iss);
      us = uniq(chs); // Index of start in chs and nms
      grow,us,numberof(chs)+1;
      grpss = shift(us,1) - us;
      grpss = grpss(1:-1);
      gs = where(grpss > 1); // Index of groups in 'us'
      ngrpss = numberof(gs);
      for( j = 1; j <= ngrpss; j++ ) {
         nmems = grpss(gs(j));
         write,format="\n%s\n", nms(us(gs(j)));
         if(fout)write,fout,format="\n%s\n", nms(us(gs(j)));
         for( k = 1; k < nmems; k++ ) {
            write,format="%s\n", nms(us(gs(j))+k);
            if(fout)write,fout,format="%s\n", nms(us(gs(j))+k);
         }
      }
   }
   if( fout ) {
      close, fout;
      write,"\nOutput in "+outfile;
   }
}

/* Function ynclude */

/**************************************************************
 *
 *   A primitive way to include a fixed set of lines
 *   in a Yorick code
 *
 * The line in a text file:
 *
 * // ynclude = include_file.ixc
 *
 * where the "// ynclude =" must be reproduced in an exact way
 * will, with the call of ynclude,"your_text_file.txt"
 * be followed by the contents of "include_file.ixc" and
 * immediately by the line
 *
 * // yxclude =
 *
 * to signal 'end-of-include-file'
 *
 * If the text file contains "// ynclude =" as well as "// yxclude ="
 * then the action will be like this:
 * The lines between the two lines will be written to the file
 * pointed to by "// ynclude = filename", they will be removed from
 * the present file as well as the "// yxclude =" line.
 *
 * 2010-02-09/NJW
 *
 *******************************************************/

func ynclude( filename, silent=, clean= )
/* DOCUMENT ynclude, filename, silent=, clean=

    Lines between '// ynclude = incl_filename'
              and '// yxclude ='
    will be written to 'incl_filename' and removed from original file

    If no '// yxclude =' is found then the contents of 'incl_filename'
    is inserted after '// ynclude = incl_filename' and supplemented
    with the line '// yxclude ='

    It toggles between two states of the file with respect to inclusion.

    Keywords
       silent     suppresses output
       clean      removes include files following inclusion

    2010-02-09/NJW
*/
{
   vb = is_void(silent);

   lines = rdfile(filename);

   // check for applicability ie. presence of "// ynclude ="

   wn = where(strpart(lines,1:12) == "// ynclude =");
   if( (nwn = numberof(wn)) == 0 ) {
      write,"No 'ynclude' found - skip any action";
      return;
   }

   // check for status ie. presence of "// yxclude"

   wx = where(strpart(lines,1:12) == "// yxclude =");
   if( (nwx = numberof(wx)) == 0 ) {

      if(vb) write,"No 'yxclude' found - in inclusion mode";

      lines_out = [];
      in_names = [];
      for( i = 1; i <= nwn; i++ ) {
         if( i == 1 ) {
            grow, lines_out, lines(1:wn(i));
         } else {
            grow, lines_out, lines(wn(i-1)+1:wn(i));
         }
         in_name = strtrim(strpart(lines(wn(i)),13:0));
         if(vb) write,format=" Including file: %s\n", in_name;
         if( file_test(in_name) ) {
            grow, lines_out, rdfile(in_name);
            grow, in_names, in_name;
         } else write,"Warning: Include file '"+in_name+"' is missing!";
         grow, lines_out, "// yxclude =";
      }
      // get the remainder - if any
      if( wn(i-1)+1 <= numberof(lines) ) grow, lines_out, lines(wn(i-1)+1:0);
      write_slist, filename, lines_out;

      if( clean ) {
         for( i = 1; i <= numberof(in_names); i++ ) {
            remove, in_names(i);
         }
      }

   } else {

      if(vb) write,"'yxclude' was found - in reduction mode";

      // check for pairing up

      if( nwn != nwx ) {
         write,format="%i ynclude and %i yxclude were found - skip action\n",\
                  nwn, nwx;
         return;
      }

      if( anyof( wx < wn ) ) {
         write,"ynclude and yxclude appear in wrong order - skip action";
         return;
      }
      if( nwn > 1 ) {
         if( anyof( wx(1:-1) > wn(2:0) ) ) {
            write,"ynclude and yxclude appear in wrong order - skip action";
            return;
         }
      }

      lines_out = [];
      for( i = 1; i <= nwn; i++ ) {
         if( i == 1 ) {
            grow, lines_out, lines(1:wn(i));
         } else {
            grow, lines_out, lines(wx(i-1)+1:wn(i));
         }
         out_name = strtrim(strpart(lines(wn(i)),13:0));
         save_lines = lines(wn(i)+1:wx(i)-1);
         if(vb) write,format=" Saving %i lines to file: %s\n", \
                  numberof(save_lines), out_name;
         write_slist,out_name,save_lines;
      }
      // get the remainder - if any
      if( wx(i-1)+1 <= numberof(lines) ) grow, lines_out, lines(wx(i-1)+1:0);
      write_slist,filename, lines_out;

   }
}

/* Function nice_externs */

func nice_externs( filename, ned= )
/* DOCUMENT nice_externs, filename, ned=

    An auxiliary function to 'ynclude' to edit and make a
    nice appearance of extern variables.

   SEE ALSO: ynclude
*/
{
   if( !ned )vi, filename;
   text = rdfile( filename );
   words = str_get_words( text );
   w = where( words != "extern" );
   words = words(w)+",";
   len = max(strlen(words))+2;
   arrange_words_in_columns, words, 80/len, len, outfile=filename, margin=10;
   text = strtrim(rdfile( filename ), 2);
   text(1) = strput( text(1), "extern", 3 );
   nlines = numberof(text);
   if( nlines > 1 ) text(1:-1) += " \\";
   pos = strpos( text(0), ",",rev=1);
   text(0) = strput( text(0), ";", pos );
   write_slist,filename,text;
   //+write,"  ----   Result in "+filename+"  ----";
   //+prstrarr, text;
   //+write,"  ----        -----          ----";
}

/* Function mvall2special */

func mvall2special( chat= )
/* DOCUMENT mvall2special, chat=

   Invoke 'mv2special' for each *.i file present in the
   current directory.

   2013-02-27/NJW
*/
{
   // Never use this function in a */yorick or */yorick/special directory
   curdir = rem_slash(get_cwd());
   bcurdir = basename(curdir);
   if( bcurdir == "yorick" ) {
      write,"Not to be used in this directory: "+curdir;
      return;
   }
   if( bcurdir == "special" ) {
      if( strlen(curdir) > 14 ) {
         if( strpart(curdir,-13:0) == "yorick/special" ) {
            write,"Not to be used in this directory: "+curdir;
            return;
         }
      }
   }

   list = rdfile(popen("find . -maxdepth 1 -type f -name \\*.i",0));
   n = numberof(list);
   if( n ) {
      list = basename(list);
      for( i = 1; i <= n; i++ ) mv2special, list(i), chat=chat;
   } else {
      write,"No *.i type f files found";
   }
}

/* Function mv2special */

func mv2special( filename, chat= )
/* DOCUMENT mv2special, filename, chat=

    An auxiliary function to copy a yorick function file to
    /home/njw/yorick/special and create a link in the current
    directory.

    Example:    mv2special,"asdf.i"
       will move asdf.i to /home/njw/yorick/special/asdf_0001.i
       where 0001 is the next serial number for asdf_nnnn.i files.
       A symbolic link asdf.i -> /home/njw/yorick/special/asdf_0001.i
       is created. An entry is added to the file
       /home/njw/yorick/special/index.txt to tell where the file
       asdf_0001.i originated.

    'filename' must be a file where the name has the suffix .i

   2012-04-27/NJW
*/
{
   local fdir, fname;

   if( is_link(filename) ) {
      write,filename+" is already a symbolic link - return ...";
      return;
   }

   cwd = rem_slash(get_cwd()); // remember from where the function was called
   splitfname, filename, fdir, fname;
   if( fdir != "." ) cd, fdir;

   // Never use this function in a */yorick or */yorick/special directory
   curdir = rem_slash(get_cwd());
   bcurdir = basename(curdir);
   if( bcurdir == "yorick" ) {
      write,"Not to be used in this directory: "+curdir;
      return;
   }
   if( bcurdir == "special" ) {
      if( strlen(curdir) > 14 ) {
         if( strpart(curdir,-13:0) == "yorick/special" ) {
            write,"Not to be used in this directory: "+curdir;
            return;
         }
      }
   }

   // get name without .i suffix
   pos = strpos( fname, ".i", rev=1 );
   len = strlen(fname);
   if( pos+1 != len || len < 3 ) {
      write,"Not a *.i file - return";
      return;
   }

   //  --  get part of name without .i
   sname = strpart( fname, 1:pos-1 );

   //  --  see what the serial number in ~/yorick/special must be
   dir = "/home/njw/yorick/special";
   nname = get_next_filename( sname+"_????.i", dir=dir );

   //  --  move the file
   system,"mv "+fname+" "+nname;

   //  --  note where it came from in index file
   f = open("/home/njw/yorick/special/index.txt","a");
   write,f,format="%s %s\n", strpadd(basename(nname),50," "), get_cwd();
   close, f;
   //  -- create the link to maintain the filename
   system,"ln -s "+nname+" "+fname;

   if( chat ) {
      write,"Move: "+filename;
      write,"  to: "+nname+" ...";
   }

   if( fdir != "." ) cd, cwd;
}
