extern rmf_funcsdoc;
/* DOCUMENT ****************************************

   A collection of function for RMF manipulations of
   various kinds including ARF:

     charf      :  Arithmetic changes of an ARF in a table
     attach_arf :  Update ANCRFILE content of '-SPE' file
     arf2phaii  :  Save arf array to adequate FITS format
     comp_rmf_files : Compare two RMF files with usual JEM-X
                      extensions.
     create_rmf :  Create an RMF file with a RDM
                          (Redistribution matrix)
     read_arf   :  Return an ARF with energy boundaries
     read_ogip_rmf :  Read an OGIP standard RMF
     rebin_rdm  :  Rebin an RDM

     2009-06-22/NJW, updated 2010-10-05/NJW
     2011-09-13/NJW, updated with RMF OGIP read

******************************************************/

/* Function charf */

func charf( filename, spe_num, expr=, outfile= )
/* DOCUMENT charf, filename, spe_num, expr=, outfile=

   If 'filename' is a spectrum file then the ANCRFILE will be used
   as input ARF, else if it is an ARF file it itself will be used.

   Default for spe_num is 1
   Keyword 'expr' can be "*1.05", "-0.02", "+0.2", "/1.05"

   SEE ALSO: attach_arf

   2008-10-28/NJW
*/
{
   if( typeof(expr) != "string" ) error,"Keyword expr must be a string";

   dol = filename+"+1";

   // Find out whether it is a spectral file or an ARF file

   fh = headfits( dol );
   extname = fxpar( fh, "extname" );
   psp = strpos( extname, "SPE" );
   par = strpos( extname, "ARF" );
   psr = strpos( extname, "SPECRESP" );
   if( psr > 0 ) psp = 0; // Finding SPECRESP overrides finding SPE
   is_spec = -1;
   if( psp > 0 && par <= 0 ) {
      is_spec = 1;
      write,"This is a SPEC file";
   } else if( psp <= 0 && (psr > 0 || par > 0 ) ) {
      is_spec = 0;
      write,"This is an ARF file";
   } else {
      write,format="EXTNAME = %s - cannot decide its type\n", extname;
      return;
   }

   if( is_spec ) {
      // We have a SPEC file

      // get ARF from ANCRFILE
      colnum = fits_colnum( fh, "ANCRFILE");
      if( is_void(colnum) ) error,"Cannot find ANCRFILE column";
      arf_file = strtrim(fits_bintable_peek( dol,spe_num,colnum));
      arf_file = strip_curly_br(arf_file);
   } else {
      arf_file = filename;
      arf_dol = dol;
   }
   fha = headfits( arf_dol );
   colnuma = fits_colnum( fha, "SPECRESP" );
   if( is_void(colnuma) ) error,"Cannot find SPECRESP column";
   arf = fits_bintable_peek( arf_file+"+1", spe_num, colnuma );
   ty = typeof(arf);

   oper = strpart(expr,1:1);
   val  = atof(strpart(expr,2:0));

   if( oper == "*" ) {
      arf *= val;
   } else if( oper == "+" ) {
      arf += val;
   } else if( oper == "-" ) {
      arf -= val;
   } else if( oper == "/" ) {
      arf /= val;
   } else error,"Operand "+oper+" is not supported";

   if( typeof(outfile) == "string" ) {
      cp,arf_file,outfile;
   } else outfile = arf_file;

   if( ty == "float" ) arf = float(arf);
   fits_bintable_poke, outfile+"+1", spe_num, colnuma, arf;

   write,format="Updated %s with new ARF\n", outfile;

}

/* Function attach_arf */

func attach_arf( spe_file, arf_file, spe_num )
/* DOCUMENT     attach_arf, spe_file, arf_file
                attach_arf, spe_file, arf_file, spe_num
                attach_arf, spe_file{2}, arf_file{3}

   Insert 'arf_file' in the 'ANCRFILE' column of 'spe_file'

   First version:  All spectra are updated with corresponding
                   arf i.e. spe_file{1} with arf_file{1} etc.

   Second version: Only spe_file{spe_num} is updated with
                   arf_file{spe_num}

   Third version:  As written

   SEE ALSO: charf

   2008-10-28/NJW
*/
{
   local xxx, spe_no;

   if( is_void(spe_num) ) {  // First or third version

      spe_file = strtrim(spe_file);
      arf_file = strtrim(arf_file);
      if( strpart(spe_file,0:0) != "}" || strpart(arf_file,0:0) != "}" ) { // first version

         ver = 1;
         spe_no = arf_no = 1; // to survive the test

      } else { // third version

         get_exten_no, spe_file, xxx, spe_no;
         spe_file = xxx;
         get_exten_no, arf_file, xxx, arf_no;
         ver = 3;
      }

   } else {  // Second version

      spe_no = spe_num;
      arf_no = spe_num;
      ver = 2

   }

   // Make sure that spe_file is a spectral file

   if( !file_test(spe_file) ) error,"File "+spe_file+" not found";
   if( !file_test(arf_file) ) error,"File "+arf_file+" not found";

   fhs = headfits( spe_file+"+1" );
   extname = fxpar( fhs, "extname" );
   nrows = fxpar( fhs, "naxis2" );
   if( !nrows ) error,"No rows in "+spe_file;
   psp = strpos( extname, "SPE", 1 );
   if( psp <= 0 ) error,"First argument is not a spectral file";

   if( spe_no > nrows ) error,"Requested row not found in "+spe_file;

   colnum = fits_colnum( fhs, "ANCRFILE");
   tmpl_ancrfile = fits_bintable_peek( spe_file+"+1", 1, colnum );
   len = strlen( tmpl_ancrfile );

   fha = headfits( arf_file+"+1" );
   extname = fxpar( fha, "extname" );
   psp = strpos( extname, "ARF", 1 );
   if( psp <= 0 ) write,"Warning! Second argument may not be an ARF file";

   if( ver == 1 ) { // loop over all rows

      for( i = 1; i <= nrows; i++ ) {
         newancrfile = arf_file+"{"+itoa(i)+"}";
         l = strlen(newancrfile);
         for(j=l;j<len;j++) newancrfile += " ";
         fits_bintable_poke, spe_file+"+1", i, colnum, newancrfile;
      }

   } else { // single update, spe_no and arf_no have been defined


      newancrfile = arf_file+"{"+itoa(arf_no)+"}";
      l = strlen(newancrfile);
      for(j=l;j<len;j++) newancrfile += " ";
      fits_bintable_poke, spe_file+"+1", spe_no, colnum, newancrfile;

   }

   write,format="Updated %s with new ANCRFILE\n", spe_file;

}

/* Function arf2phaii */

func arf2phaii( filename, arf, energ_lo, energ_hi, extname=, instrume=, \
                     mission=, telescop=, nokwdsinit= )
/* DOCUMENT arf2phaii, filename, arf, energ_lo, energ_hi, extname=, \
                   instrume=, mission=, telescop=, nokwdsinit=

     Write ARF or ARFs in counts/s to PHA II format
     where each row has vector fields (columns) ENERG_LO, ENERG_HI, and SPECRESP

     Dimension of arf:    nbins x nrows
     Dimension of energ_lo, energ_hi: nbins (assumed to the identical for all rows)
                               or     nbins x nrows

     Keywords:

        These keywords are single valued (becomes part of FITS header)

             extname   (string)  EXTNAME, defaults to SPECRESP
             instrume  (string)  instrument name
             mission   (string)  mission name
             telescop  (string)  name of 'telescope'

             nokwdsinit          will inhibit the keyword initialization

    2009-06-29/NJW
*/
{

   dms = dimsof(arf);
   dmse = dimsof(energ_lo);
   // expand energ_lo if we have more than one ARF
   if( dmse(1) == 1 && dms(1) > 1 ) {
      energ_lo = energ_lo(,-:1:dms(3));
   }
   dmse = dimsof(energ_hi);
   if( dmse(1) == 1 && dms(1) > 1 ) {
      energ_hi = energ_hi(,-:1:dms(3));
   }
   if( anyof(dms-dimsof(energ_lo)) ) error,"Argument (energ_lo) dimension incompatibility";
   if( anyof(dms-dimsof(energ_hi)) ) error,"Argument (energ_hi) dimension incompatibility";

   if( dms(1) == 2 ) {
      nrows = dms(3);
      nbins = dms(2);
   } else if( dms(1) == 1 ) {
      nrows = 1;
      nbins = dms(2);
      arf = reform(arf,nbins,nrows);
      energ_lo = reform(energ_lo,nbins,nrows);
      energ_hi = reform(energ_hi,nbins,nrows);
   } else error,"Illegal dimensions of ARF array";

   ttype1 = "arf_num";
   data1 = short(indgen(nrows));
   ttype2 = "energ_lo";
   data2 = float(energ_lo);
   ttype3 = "energ_hi";
   data3 = float(energ_hi);
   ttype4 = "specresp";
   data4 = float(arf);

   if( !nokwdsinit ) kwds_init;

   if( typeof(extname) == "string" ) {
      kwds_set,"extname",extname,"Name of this extension";
   } else {
      kwds_set,"extname","SPECRESP","Extension name";
   }
   if( !is_void(instrume) ) kwds_set,"instrume",instrume,"Instrument name";
   if( !is_void(mission) ) kwds_set,"mission",mission,"Mission name";
   if( !is_void(telescop) ) kwds_set,"telescop",telescop,"Telescope name";
   kwds_set,"date",ndate(3),"CET Central European time";
   kwds_set,"hduclass","OGIP","Format conforms mostly to OGIP standards";
   kwds_set,"hduclas1","RESPONSE","Dataset contains response information";
   kwds_set,"hduclas2","SPECRESP","Ancillary response data";
   kwds_set,"hduvers","1.3.0","Version of format";
   kwds_set,"tunit2","keV","Unit of column ENERG_LO";
   kwds_set,"tunit3","keV","Unit of column ENERG_HI";
   kwds_set,"tunit4","cm**2","Unit of column SPECRESP";

   wrmfitscols,filename, ttype1, data1, ttype2, data2, ttype3, data3, \
                         ttype4, data4, clobber=1;

}

/* Function create_rmf */

func create_rmf( outfile, n_in=, e_in_min=, e_in_max=, lg=, \
      n_det=, e_det_min=, e_det_max=, expr= )
/* DOCUMENT  create_rmf, outfile, n_in=, e_in_min=, e_in_max=, lg=, \
                 n_det=, e_det_min=, e_det_max=, expr=

    'outfile' is the name of the resulting RMF-file (Default: rmf.fits).

    The following keywords can be given in the call, those not given will
    be asked for (via parameter file: create_rmf.par):

    energ_lo & energ_hi  (n_in)  belong to input spectrum
    e_in_min, e_in_max
    Set 'lg' to get logarithmic in-energy steps

    e_min & e_max        (n_det) belong to detector channels
    e_det_min, e_det_min

    'expr' is the expression for the detector resolution as a function
    of energy that MUST be called 'ener'.

    This function writes file: 'crmf.i' that must be loaded and
    executed in Yorick. This is a work-around of defining a
    function as a text string.

    2010-10-05/NJW but made in 2008 or 2009
*/
{

   extern N_in, E_in_min, E_in_max, N_det, E_det_min, E_det_max, Expr;
   extern Outfile;

   if( is_void(outfile) ) {
      Outfile = "rmf.fits";
      write,"The RMF file will be: "+Outfile;
   }
   Outfile = outfile;

   if( !file_test("create_rmf.par") ) {
      pf = open("create_rmf.par","w");
      write,pf,format="// n_in = %i; Number of columns in RDM\n", 200;
      write,pf,format="// e_in_min = %f; [keV] Lower spectral energy\n", 0.05;
      write,pf,format="// e_in_max = %f; [keV] Upper spectral energy\n", 15.0;
      write,pf,format="// lg = %i; Flag for logarithmic in-energies\n", 1;
      write,pf,format="// n_det = %i; Number of energy bins in detector\n", 100;
      write,pf,format="// e_det_min = %f; [keV] Lower detector energy\n", 0.10;
      write,pf,format="// e_det_max = %f; [keV] Upper detector energy\n", 10.0;
      write,pf,format="// expr = %s; Expression for sigma\n", "0.2*sqrt(ener)";
      close,pf;
   }

   if( is_void(n_in) ) n_in = get_spar("create_rmf.par","n_in",lng=1);
   if( is_void(e_in_min) ) e_in_min = get_spar("create_rmf.par","e_in_min");
   if( is_void(e_in_max) ) e_in_max = get_spar("create_rmf.par","e_in_max");
   if( is_void(lg) ) lg = get_spar("create_rmf.par","lg",lng=1);
   if( is_void(n_det) ) n_det = get_spar("create_rmf.par","n_det",lng=1);
   if( is_void(e_det_min) ) e_det_min = get_spar("create_rmf.par","e_det_min");
   if( is_void(e_det_max) ) e_det_max = get_spar("create_rmf.par","e_det_max");
   if( is_void(expr) ) expr = get_spars("create_rmf.par","expr");

   N_in = n_in;
   E_in_min = e_in_min;
   E_in_max = e_in_max;
   N_det = n_det;
   E_det_min = e_det_min;
   E_det_max = e_det_max;
   Expr = expr;

   f = open("crmf.i","w");

   //+ write,f,"func crmf {";
   slg = lg ? "l" : "";
   write,f,"el = span"+slg+"( E_in_min, E_in_max, N_in+1 );";
   write,f,"energ_lo = el(1:-1); energ_hi = el(2:0); energ = sqrt(energ_lo*energ_hi);";

   write,f,"e = span( E_det_min, E_det_max, N_det+1 );";
   write,f,"e_min = e(1:-1); e_max = e(2:0); edet = e(zcen);";

   write,f,"rdm = array(double,N_det,N_in);";

   write,f,"for( j = 1; j <= N_in; j++ ) { ener = energ(j);";
      write,f,format=" sigma = %s;\n",expr;
      write,f,"rdm(,j) = array_igauss(e,[1./(sigma*sqrt(2*pi)),ener,sigma])*e(dif);}";

   write,f,"kwds_init;";
   write,f,"kwds_set,\"EXTNAME\",\"MATRIX\",\"Name of this extension\";";
   write,f,"kwds_set,\"ORIGIN\",\"DTU Space/NSI\",\"Origin of FITS file\";";
   write,f,"kwds_set,\"CREATOR\",\"yorick/create_rmf(.i)\",\"Executable which created this data\";";
   write,f,"kwds_set,\"DATE\",ndate(3),\"Date/time of creation\";";
   write,f,"kwds_set,\"HDUCLASS\",\"OGIP\",\"Format conforms mostly to OGIP standards\";";
   write,f,"kwds_set,\"HDUCLAS1\",\"RESPONSE\",\"Dataset contains response information\";";
   write,f,"kwds_set,\"HDUCLAS2\",\"RSP_MATRIX\",\"Extension contains a response matrix\";";
   write,f,"kwds_set,\"HDUVERS\",\"1.3.0\",\"Version of format\";";
   write,f,"kwds_set,\"CHANTYPE\",\"PHA\",\"Type of detector channels\";";
   write,f,"kwds_set,\"DETCHANS\",N_det,\"Total number of PHA channels in the full matrix\";";
   write,f,"kwds_set,\"TLMIN4\",1,\"Channel number of first channel\";";
   write,f,"kwds_set,\"TLMAX4\",N_det,\"Channel number of last channel\";";
   write,f,"kwds_set,\"LO_THRES\",0.0,\"Lowest value in MATRIX\";";
   write,f,"kwds_set,\"TUNIT1\",\"keV\",\"Unit of energy\";";
   write,f,"kwds_set,\"TUNIT2\",\"keV\",\"Unit of energy\";";
   write,f,"kwds_set,\"COMMENT\",\"Sigma = "+expr+"\";";

   write,f,"idum = array(short(1),N_in);";
   write,f,"fh = wrmfitscols(\""+Outfile+"\",\"ENERG_LO\",float(energ_lo),\"ENERG_HI\",float(energ_hi), \\";
       write,f,"\"N_GRP\",idum,\"F_CHAN\",idum,\"N_CHAN\",idum,\"MATRIX\",float(rdm),cont=1,clobber=1);";

   write,f,"kwds_set,\"EXTNAME\",\"EBOUNDS\",\"Name of this extension\";";
   write,f,"kwds_set,\"HDUCLAS2\",\"EBOUNDS\",\"Extension contains energy boundaries\";";
   write,f,"kwds_del,\"TUNIT1\";";
   write,f,"kwds_set,\"TUNIT3\",\"keV\",\"Unit of energy\";";
   write,f,"kwds_del,\"TLMIN4\";";
   write,f,"kwds_del,\"TLMAX4\";";
   write,f,"kwds_del,\"LO_THRES\";";

   write,f,"wrmfitscols,fh,\"CHANNEL\",short(indgen(N_det)),\"E_MIN\",float(e_min), \\";
       write,f,"\"E_MAX\",float(e_max);";

//+ write,f,"}";

   write,"Now: #include \"crmf.i\"";
}

/* Function read_ogip_rmf */

func read_ogip_rmf( filename, &elo, &ehi, &e_min, &e_max )
/* DOCUMENT rdm = read_ogip_rmf( filename, >elo, >ehi, >e_min, >e_max )

   See OGIP Calibration Memo CAL/GEN/92-002 (RMFs & ARFs)
    "The Calibration Requirements for Spectral Analysis"

   NB NB NB  This is a very basic version that ignores
             1) MATRIX may not have the right dimension to begin with
             2) Columns F_CHAN, N_CHAN, MATRIX may be of variable length
             3) The mandatory keywords such as 'DETCHANS'
*/
{
   local hdr, nrows;

   /*
    *   The extension "MATRIX" or "SPECRESP MATRIX" must exist with columns
    *     ENERG_LO, ENERG_HI, N_GRP, F_CHAN, N_CHAN, MATRIX
    *   and (mandatory) keywords
    *     EXTNAME, TELESCOP, INSTRUME, FILTER, CHANTYPE, DETCHANS, HDUCLASS,
    *     HDUCLAS1, HDUCLAS2, HDUVERS, TLMIN#
    *   and (optional) keywords
    *     NUMGRP, NUMELT, PHAFILE, LO_THRES, HDUCLAS3 ('REDIST', 'DETECTOR', or 'FULL')
    */

   ptr = rdfitsbin( filename+"[MATRIX]", hdr, nrows );
   if( is_void(ptr) ) { // Try the other allowed name
      ptr = rdfitsbin( filename+"[SPECRESP MATRIX]", hdr, nrows );
      if( is_void(ptr) ) {
         error,"None of the required extension names were found";
      }
   }

  /*
   *   Following three columns MUST be present
   */
   ic = fits_colnum( hdr, "ENERG_LO" );
   if( is_void(ic) ) error,"Column ENERG_LO not found";
   elo = *ptr(ic);
   ic = fits_colnum( hdr, "ENERG_HI" );
   if( is_void(ic) ) error,"Column ENERG_HI not found";
   ehi = *ptr(ic);
   ic = fits_colnum( hdr, "MATRIX" );
   if( is_void(ic) ) error,"Column MATRIX not found";
   m = transpose(*ptr(ic)); // transposition is required to adhere to the
                            // convention in 'rdfitscol'
   mnew = m;

  /*
   *   Following three columns may be absent
   *   in which case 'm' is the final RDM
   */
   ic = fits_colnum( hdr, "N_GRP" );
   if( is_void(ic) ) goto ebounds;
   n_grp = *ptr(ic);
   ic = fits_colnum( hdr, "F_CHAN" );
   if( is_void(ic) ) goto ebounds;
   f_chan = *ptr(ic);
   ic = fits_colnum( hdr, "N_CHAN" );
   if( is_void(ic) ) goto ebounds;
   n_chan = *ptr(ic);

   // when you come here 'm' must be reworked
   detchans = fxpar( hdr, "DETCHANS" );
   numgrp = numberof(elo);
   mnew = array(double, detchans, numgrp );
   

   for( i = 1; i <= nrows; i++ ) {
      m_orig = m(,i);
      istart = 1;
      for( j = 1; j <= n_grp(i); j++ ) {
         mnew(f_chan(i,j)+1:f_chan(i,j)+n_chan(i,j),i) = m_orig(istart:istart+n_chan(i,j)-1);
         istart += n_chan(i,j);
      }
   }

  ebounds:

   /*
    *    Read the EBOUNDS extension for E_MIN and E_MAX
    */

   ptr = rdfitsbin( filename+"[EBOUNDS]", hdr, nrows );
   if( is_void(ptr) ) { // Skip and return
      write,"Extension EBOUNDS is missing - skip and only return MATRIX";
      e_min = e_max = [];
      return m;
   }

   ic = fits_colnum(hdr, "E_MIN");
   if( is_void(ic) )  error,"Column is missing";
   e_min = *ptr(ic);
   ic = fits_colnum(hdr, "E_MAX");
   if( is_void(ic) )  error,"Column is missing";
   e_max = *ptr(ic);

   return mnew;

}

/* Function rebin_rdm */

func rebin_rdm( rdm, rebin )
/* DOCUMENT new_rdm = rebin_rdm( rdm, rebin )

    Rebins a redistribution matrix (RDM) using an array 'rebin' that
    may result from 'specrebinninga' or 'specrebinningb'. Its elements represent the
    number of old bins that will make the new array.

    Its similar to the array used in 'specrebinning' except that it
    should not skip any bins (i.e. have negative values) except at the
    very end.

*/
{
   dms = dimsof( rdm );
   n_det_bins = dms(2);
   n_ener_bins = dms(3);

   // check 'rebin' for asking for too many bins

   n_wanted = sum(abs(rebin));
   //+ write,"Check 'rebin' ...";
   while( n_wanted > n_det_bins ) {
      // diminish or completely remove last element?
      n = n_wanted - n_det_bins;
      if( abs(rebin(0)) <= n ) { // remove last element
         rebin = rebin(1:-1);
         //+ write,"  remove last element";
      } else {     // diminish last element
         if( rebin(0) > 0 ) {
            rebin(0) -= n;
            //+ write,"  diminish last element to "+itoa(rebin(0));
         } else {
            rebin(0) += n;
            //+ write,"  diminish last element to "+itoa(rebin(0));
         }
      }
      n_wanted = sum(abs(rebin));
   }
   //+ write,"Checking done ...";

   n_new_det_bins = numberof(rebin) - numberof(where(rebin <= 0));

   new_rdm = array( structof(rdm), n_new_det_bins, n_ener_bins );

   m = 1;
   k1 = k2 = 0;
   for( i = 1; i <= n_new_det_bins; i++ ) {
      k1 = k2 + 1;
      while(rebin(m) <= 0) { k1 -= rebin(m); m++; }
      k2 = k1 + rebin(m++) - 1;
      //+ write,"    Summing between "+itoa(k1)+" and "+itoa(k2);
      a = rdm(k1:k2,);
      new_rdm(i,) = a(sum,);
   }

   return new_rdm;

}


/* Function read_arf */

struct s_ARF {
   double elo;
   double ehi;
   double el;
   double arf;
}
func read_arf( filename )
/* DOCUMENT r = read_arf( arf_file_name )

     Returns struct s_ARF with elements 'elo', 'ehi', 'el', and 'arf'.

     SEE ALSO: rd_arf (xray.i)

*/
{
   if( !file_test(filename) ) error,filename+" was not found!";

   dol = filename+"[SPECRESP]";
   elo = rdfitscol( dol, "energ_lo");
   res = array( s_ARF, numberof(elo) );
   res.elo = elo;
   res.ehi = rdfitscol( dol, "energ_hi");
   res.arf = rdfitscol( dol, "specresp");
   res.el = 0.5*( res.elo + res.ehi );

   return res;
}

/* Function comp_rmf_files */

func comp_rmf_files( file1, file2 )
/* DOCUMENT comp_rmf_files, file1, file2

    Compare two RMF files
     1) MATRIX in extension 2
     2) E_MIN/E_MAX in extension 3
     3) AXIS-ARF in extension 4
     4) IMAG-ARF in extension 5

   Print result to screen.

*/
{
   // Extension 2

   hdr1 = headfits( file1+"+2" );
   hdr2 = headfits( file2+"+2" );
   write,"**********  EXTENSION 2 ***************";
   write,"EXTNAME(1) = "+fxpar(hdr1,"EXTNAME");
   write,"EXTNAME(2) = "+fxpar(hdr2,"EXTNAME");
   m1 = rdfitscol(file1+"+2", "MATRIX" );
   m2 = rdfitscol(file2+"+2", "MATRIX" );
   dms1 = dimsof(m1);
   dms2 = dimsof(m2);
   if( anyof(dms1 - dms2) ) {
      write, "Differing dimension of RDM";
   } else {
      write,"Largest difference between RDMs: "+ftoa(max(abs(m1-m2)),ndec=3,sci=1);
   }

// ------- EBOUNDS -------------
   hdr1 = headfits( file1+"+3" );
   hdr2 = headfits( file2+"+3" );
   write,"**********  EXTENSION 3 ***************";
   write,"EXTNAME(1) = "+fxpar(hdr1,"EXTNAME");
   write,"EXTNAME(2) = "+fxpar(hdr2,"EXTNAME");
   emn1 = rdfitscol(file1+"+3", "E_MIN" );
   emx1 = rdfitscol(file1+"+3", "E_MAX" );
   emn2 = rdfitscol(file2+"+3", "E_MIN" );
   emx2 = rdfitscol(file2+"+3", "E_MAX" );
   if( numberof(emn1) - numberof(emn2) ) {
      write,"Differing dimension of E_MIN";
   } else {
      write,"Largest difference between E_MINs: "+ftoa(max(abs(emn1-emn2)),ndec=3,sci=1);
      write,"Largest difference between E_MAXs: "+ftoa(max(abs(emx1-emx2)),ndec=3,sci=1);
   }

// ------- AXIS-ARF -------------
   hdr1 = headfits( file1+"+4" );
   hdr2 = headfits( file2+"+4" );
   write,"**********  EXTENSION 4 ***************";
   write,"EXTNAME(1) = "+fxpar(hdr1,"EXTNAME");
   write,"EXTNAME(2) = "+fxpar(hdr2,"EXTNAME");
   axarf1 = rdfitscol(file1+"+4", "SPECRESP" );
   axarf2 = rdfitscol(file2+"+4", "SPECRESP" );
   if( numberof(axarf1) - numberof(axarf2) ) {
      write,"Differing dimension of AXIS-ARF";
   } else {
      write,"Largest difference between AXIS-ARFs: "+ftoa(max(abs(axarf1-axarf2)),ndec=3,sci=1);
   }

// ------- IMAG-ARF -------------
   hdr1 = headfits( file1+"+5" );
   hdr2 = headfits( file2+"+5" );
   write,"**********  EXTENSION 5 ***************";
   write,"EXTNAME(1) = "+fxpar(hdr1,"EXTNAME");
   write,"EXTNAME(2) = "+fxpar(hdr2,"EXTNAME");
   imarf1 = rdfitscol(file1+"+5", "SPECRESP" );
   imarf2 = rdfitscol(file2+"+5", "SPECRESP" );
   if( numberof(imarf1) - numberof(imarf2) ) {
      write,"Differing dimension of IMAG-ARF";
   } else {
      write,"Largest difference between IMAG-ARFs: "+ftoa(max(abs(imarf1-imarf2)),ndec=3,sci=1);
   }
}
